<?php

namespace WPEP\Admin\Container;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;

trait Configuration {

  public $field_prefix = 'wpep_configuration';

  public function set_field_prefix( $field_prefix ) {
    $this->field_prefix = $field_prefix;
  }

  public function display_table_wrapper_options() {
    $this->display_table_separator( __( "Wrapper Options", "wpep" ) );

    $this->display_table_select_field(
      __('Width', "wpep"),
      'layout-max-width',
      [
        1024 => sprintf( __( "Default, up to %s", "wpep" ), '1024px' ),
        1200 => sprintf( __( "Default, up to %s", "wpep" ), '1200px' ),
        1440 => sprintf( __( "Up to %s", "wpep" ), '1440px' ),
        0    => __( "Unlimited", "wpep" ),
        240  => sprintf( __( "Situational : Up to %s", "wpep" ), '240px' ),
        360  => sprintf( __( "Situational : Up to %s", "wpep" ), '360px' ),
        480  => sprintf( __( "Situational : Up to %s", "wpep" ), '480px' ),
        640  => sprintf( __( "Situational : Up to %s", "wpep" ), '640px' ),
        960  => sprintf( __( "Situational : Up to %s", "wpep" ), '960px' ),
      ],
      __( "Set the max. width of WPEP's layout, not all options may be supported by your theme.", "wpep" )
    );

    $this->display_table_color_field(
      __( 'Background', "wpep" ),
      'layout-background-color',
      __( 'The background color, leave this empty for the default.', "wpep" )
    );

    $this->display_table_select_field(
      __( 'Padding', "wpep" ),
      'layout-padding',
      [
        'default'   => __( 'Skin Default', "wpep" ),
        0           => __( "None", "wpep" ),
        10          => __( "10 Pixels", "wpep" ),
        20          => __( "20 Pixels", "wpep" ),
        '20px 10px' => sprintf( __( "%s Pixels Top & Bottom. %s Pixels Sides", "wpep" ), '20', '10' ),
        30          => __( "30 Pixels", "wpep" ),
        '30px 10px' => sprintf( __( "%s Pixels Top & Bottom. %s Pixels Sides", "wpep" ), '30', '10' ),
        '40px 10px' => sprintf( __( "%s Pixels Top & Bottom. %s Pixels Sides", "wpep" ), '40', '10' ),
        '40px 20px' => sprintf( __( "%s Pixels Top & Bottom. %s Pixels Sides", "wpep" ), '40', '20' ),
        '60px 30px' => sprintf( __( "%s Pixels Top & Bottom. %s Pixels Sides", "wpep" ), '60', '30' ),
      ],
      __( 'The space space around content.', "wpep" )
    );

    $this->display_table_select_field(
      __( 'Box Shadow', "wpep" ),
      'layout-box-shadow',
      [
        'default'                      => __( 'Skin Default'  , "wpep" ),
        '0 0 0'                        => __( "None"  , "wpep" ),
        '0 0 2px rgba(0, 0, 0, 0.33)'  => __( "Very Light"  , "wpep" ),
        '0 0 5px rgba(0, 0, 0, 0.33)'  => __( "Light" , "wpep" ),
        '0 0 10px rgba(0, 0, 0, 0.33)' => __( "Medium"  , "wpep" ),
      ],
      __( 'Adjust the shadow of the content wrapper.', "wpep" )
    );
  }

  public function get_value( $field_key, $field_default = '' ) {
    return WPEP_Settings::instance()->get_option( $field_key, $field_default );
  }

  public function get_value_translated( $field_key ) {
    return WPEP_Settings::instance()->get_option_translated( $field_key );
  }

  public function display_table_start() {
    echo    '<table class="form-table">';
    echo      '<tbody>';
  }

  public function display_table_end() {
    echo      '</tbody>';
    echo    '</table>';
  }

  public function display_table_hidden_field( $field_key, $field_value, $field_name = false ) {
    echo        '<tr style="display:none;">';
    echo          '<td>';
      $this->display_hidden_field( $field_key, $field_value, $field_name );
    echo          '</td>';
    echo        '</tr>';
  }

  public function display_table_textarea_field( $field_label, $field_key, $field_description = false, $field_name = false, $height = false ) {
    echo        '<tr id="wpep-table-row-' . $field_key . '">';
    echo          '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';
    echo          '<td>';

      $this->display_textarea_field( $field_key, $field_name, $height );

    if( $field_description != false )
      echo            '<p>' . $field_description . '</p>';

    echo          '</td>';
    echo        '</tr>';
  }

  public function display_table_text_translated_field( $field_label, $field_key, $field_description = false, $field_name = false ) {
    echo        '<tr id="wpep-table-row-' . $field_key . '">';
    echo          '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';
    echo          '<td>';

    $this->display_text_field( $field_key, $field_name, $this->get_value_translated( $field_key ) );

    if( $field_description != false )
      echo            '<p>' . $field_description . '</p>';

    echo          '</td>';
    echo        '</tr>';
  }

  public function display_table_text_field( $field_label, $field_key, $field_description = false, $field_name = false, $field_value = null ) {
    echo        '<tr id="wpep-table-row-' . $field_key . '">';
    echo          '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';
    echo          '<td>';

      $this->display_text_field( $field_key, $field_name, $field_value );

    if( $field_description != false )
      echo            '<p>' . $field_description . '</p>';

    echo          '</td>';
    echo        '</tr>';
  }

  public function display_table_image_field( $field_label, $field_key, $field_description = false, $field_name = false, $field_value = null, $image_css_attributes = [], $allow_image_remove = true ) {
    echo        '<tr id="wpep-table-row-' . $field_key . '">';
    echo          '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';
    echo          '<td>';

    $this->display_image_field( $field_key, $field_label, $field_name, $field_value, $image_css_attributes, $allow_image_remove );

    if( $field_description != false )
      echo            '<p>' . $field_description . '</p>';

    echo          '</td>';
    echo        '</tr>';
  }

  public function display_table_attributes_field_content_editor( $field_label, $field_key, $field_description = false, $field_name = false, $field_map = [], $field_default = [] ) {
    echo        '<tr id="wpep-table-row-' . $field_key . '">';
    echo          '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';
    echo          '<td>';

    $this->display_attributes_field_content_editor( $field_key, $field_name, $field_map, $field_default );

    if( $field_description != false )
      echo            '<p>' . $field_description . '</p>';

    echo          '</td>';
    echo        '</tr>';
  }

  public function display_table_attributes_field( $field_label, $field_key, $field_description = false, $field_name = false, $field_map = [], $field_default = [] ) {
    echo        '<tr id="wpep-table-row-' . $field_key . '">';
    echo          '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';
    echo          '<td>';

    $this->display_attributes_field( $field_key, $field_name, $field_map, $field_default );

    if( $field_description != false )
      echo            '<p>' . $field_description . '</p>';

    echo          '</td>';
    echo        '</tr>';
  }

  public function display_table_editor_field( $field_label, $field_key, $field_description = false, $editor_height = 150 ) {
    echo        '<tr id="wpep-table-row-' . $field_key . '">';
    echo          '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';
    echo          '<td>';

      $this->display_editor_field( $field_key, $editor_height );

    if( $field_description != false )
      echo            '<p>' . $field_description . '</p>';

    echo          '</td>';
    echo        '</tr>';
  }

  public function display_table_color_field( $field_label, $field_key, $field_description = false ) {
    echo        '<tr id="wpep-table-row-' . $field_key . '">';
    echo          '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';
    echo          '<td>';

      $this->display_color_field( $field_key );

    if( $field_description != false )
      echo            '<p>' . $field_description . '</p>';

    echo          '</td>';
    echo        '</tr>';
  }

  public function display_table_checkbox_field( $field_label, $field_key, $field_description = false, $is_checked = null, $force_checked = null ) {
    echo        '<tr id="wpep-table-row-' . $field_key . '">';
    echo          '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';
    echo          '<td>';

      $this->display_checkbox_field( $field_key, $is_checked, null, 0, $force_checked );

    if( $field_description != false )
      echo            '<p>' . $field_description . '</p>';

    echo          '</td>';
    echo        '</tr>';
  }

  public function display_table_checkbox_array_field( $field_label, $field_key, $field_value ) {
    $identifier = 'wpep-' . $field_key . '-' . $field_value . '';

    echo        '<tr id="wpep-table-row-' . $field_key . '">';
    echo          '<th scope="row"><label for="' . $identifier . '">' . $field_label . '</label></th>';
    echo          '<td>';

    $this->display_checkbox_array_field( $field_key, $field_value, null, $identifier );

    echo          '</td>';
    echo        '</tr>';
  }

  public function display_table_select_field( $field_label, $field_key, $field_options = [], $field_description = false, $select_class = 'wpep-select-field' ) {
    echo        '<tr id="wpep-table-row-' . $field_key . '">';
    echo          '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';
    echo          '<td>';
      $this->display_select_field( $field_key, $field_options, false, null, $select_class );

    if( $field_description != false )
      echo            '<p>' . $field_description . '</p>';
    echo          '</td>';
    echo        '</tr>';
  }

  public function display_table_multiple_select_field( $field_label, $field_key, $field_options = [], $field_description = false, $class = 'wpep-multiple-select' ) {
    echo        '<tr id="wpep-table-row-' . $field_key . '">';
    echo          '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';
    echo          '<td>';
    $this->display_multiple_select_field( $field_key, $field_options, false, null, $class );

    if( $field_description != false )
      echo            '<p>' . $field_description . '</p>';
    echo          '</td>';
    echo        '</tr>';
  }

  public function display_table_separator( $name, $description = '<hr/>' ) {
    $alias = sanitize_title( $name );

    echo        '<tr class="wpep-table-separator">';
    echo          '<th scope="row"><label for="wpep-' . $alias . '">' . $name . '</label></th>';
    echo          '<td>' . $description . '</td>';
    echo        '</tr>';
  }

  public function display_table_license_verification( $license, $license_is_active ) {
    $nonce_value     = wp_create_nonce( 'wpep_license_activate_nonce' );
    $licenseStatusContent = $license_is_active ? '<strong id="license-status-active">' . __("Your License is currently valid and active", "wpep" ) . '</strong>'
      : '<strong id="license-status-pending">' . __( 'Please enter your license in order to receive automatic updates and support', "wpep" ) . '</strong>';


    echo        '<tr>';
    echo          '<th scope="row"><label for="wpep-license-key">' . __('Your License Key', "wpep") . '</label></th>';
    echo          '<td>';
    echo           '<input id="license-nonce" type="hidden" value="' . $nonce_value .'">
                    <input id="license-key" class="regular-text" type="text" value="' . $license . '"/>
                    <button id="wpep-verify-license"
                            class="button button-primary"
                            type="button">' . ( $license_is_active ? 'Update' : 'Activate') .'
                    </button>
                    ' . $licenseStatusContent . '
                    <div id="license-msg"></div>
                    ';
    echo          '</td>';
    echo        '</tr>';
  }

  public function display_image_field( $field_key, $image_name, $field_name = false, $image_path = null, $image_css_attributes = [], $allow_image_remove = true ) {
    $field_name = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    $image_path = ( $image_path === null ? $this->get_value( $field_key, '' ) : $image_path );

    $image_style = '';

    foreach( $image_css_attributes as $attribute_key => $attribute_value )
      $image_style .= $attribute_key . ':' . $attribute_value . ';';

    echo '<div class="wpep-image-field' . ( $image_path == '' ? '' : ' uploaded' ) . '">';
    echo    '<input type="hidden" value="' . $image_path . '" name="' . $field_name . '">';
    echo    '<img src="' . $image_path . '" style="' . $image_style . '"/>';
    echo    '<p class="wpep-upload-image button button-primary">' . sprintf( __( "Set %s", "wpep" ), $image_name ) . '</p>';
    echo    '<p class="wpep-instructions-image">' . __( "Click the image to edit or update", "wpep" ) . '</p>';

    if( $allow_image_remove )
      echo    '<p class="wpep-remove-image button button-secondary">' . sprintf( __( "Remove %s", "wpep" ), $image_name ) . '</p>';

    echo '</div>';
  }

  public function display_image_builder_field(
    $available_fields,
    $image_field_name,
    $content_field_name,
    $default_image,
    $default_content = [],
    $available_fonts = []
  ) {

    $image   = $this->get_value( $image_field_name, $default_image, true );
    $content = $this->get_value( $content_field_name, $default_content, false );

    echo '<div class="wpep-image-builder-field">';
    echo    '<p class="wpep-change-image button button-primary">' . __( "Change Image", "wpep" ) . '</p>';
    echo    '<div class="clearfix"></div>';
    echo    '<input type="hidden" class="image-link-field" value="' . $image . '" name="' . $this->field_prefix . '[' . $image_field_name . ']' . '">';
    echo    '<div class="image-builder-container">';
    echo        '<div class="image-inner-container">';
    echo            '<img src="' . $image . '"/>';
    echo            '<div class="content">';

    $i = 1; foreach( $content as $content_part ) {
      $this->_image_builder_field_content_part($content_part, $content_field_name, $i, 'name', $available_fonts );
      $i++;
    }

    $this->_image_builder_field_content_part([], $content_field_name, 'template', 'data-name', $available_fonts );

    echo            '</div>';
    echo        '</div>';
    echo        '<div class="image-options-container">';
    echo          '<ul class="image-options-list">';

    foreach( $available_fields as $field_key => $field_value )
    echo            '<li data-field-name="' .$field_key . '">' . $field_value . '</li>';

    echo          '</ul>';
    echo        '</div>';
    echo        '<div class="clearfix"></div>';
    echo    '</div>';
    echo '</div>';
  }

  private function _image_builder_field_content_part( $content_part, $content_field_name, $position, $name_attribute = 'name', $available_fonts = [] ) {
    $content_part  = $this->image_builder_field_sanitize_content_part( $content_part );
    $current_style = '';
    $input_name_prefix = $name_attribute . '="' . $this->field_prefix . '[' . $content_field_name . '][' . $position . ']';

    foreach( $content_part as $style_property => $style_value ) {
      if( in_array( $style_property, [ 'content' ] ) )
        continue;

      $current_style .= ( $style_property . ':' . $style_value . ';' );
    }

    if( $position == 'template' )
      $current_style .= 'display:none;';

    echo '<div class="content-part" style="' . $current_style . '">';

    foreach( $content_part as $style_property => $style_value ) {
      if( in_array( $style_property, [ 'content', 'color', 'font-family' ] ) )
        continue;

      echo '<input type="hidden" 
                   ' . $input_name_prefix . '[' . $style_property . ']' . '" 
                   value="' . $style_value . '" 
                   data-property-target="' . $style_property . '"
               />';
    }

    echo    '<span class="input-container"><input autocomplete="off" type="text" ' . ( $input_name_prefix . '[content]' ) . '" value="' . $content_part['content'] . '" data-property-target="content"/></span>';
    echo    '<span class="dragging-handler"></span>';
    echo    '<span class="customize-handler"></span>';
    echo    '<div class="customize-container">';
    echo      '<span class="delete"></span>';
    echo      '<span class="close"></span>';

    if( isset( $content_part['font-family'] ) && !empty( $available_fonts ) ) {
      echo '<select class="wpep-content-part-font-family"
                   ' . $input_name_prefix . '[font-family]' . '" 
                   data-property-target="font-family">';

      foreach( $available_fonts as $available_font_file => $available_font )
        echo '<option value="' . $available_font . '" ' . wpep_utility_selected( $available_font, $content_part['font-family'], false ) . '>' . $available_font . '</option>';

      echo  '</select>';
    }

    if( isset( $content_part['color'] ) )
      echo '<input type="text" class="wpep-content-part-color-picker"
                   ' . $input_name_prefix . '[color]' . '" 
                   value="' . $content_part['color'] . '" 
                   data-property-target="color"
               />';

    echo    '</div>';
    echo '</div>';
  }

  public function image_builder_field_sanitize_content_part( $current_values ) {
    if( is_string( $current_values ) )
      $current_values = json_decode( $current_values, true );

    if( !is_array( $current_values ) ) {
      $current_values = [];
    } else {
      foreach( $current_values as $key => $value ) {
        if( !is_array( $value ) )
          continue;

        $current_values[$key] = array_shift( $value );
      }
    }


    return $current_values + [
      'content'     => 'write your content here',
      'left'        => '100px',
      'top'         => '100px',
      'width'       => '200px',
      'height'      => '22px',
      'font-size'   => '19px',
      'font-family' => 'Arial',
      'font-weight' => 300,
      'color'       => '#FFFFFF'
    ];
  }

  public function display_hidden_field( $field_key, $field_value, $field_name = false, $class = '' ) {
    $field_name = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    echo '<input type="hidden" id="wpep-' . $field_key . '" value="' . $field_value . '" name="' . $field_name . '" class="' . $class . '">';
  }

  public function display_textarea_field( $field_key, $field_name = false, $height = false, $placeholder = "" ) {
    $field_name = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    echo '<textarea class="large-text" 
                    placeholder="' . $placeholder . '"
                    ' . ( $field_key !== false ? 'id="wpep-' . $field_key . '" ' : '' ) . '
                    name="' . $field_name . '" ';

    if( $height != false )
      echo ' style="height:' . $height . '" ';

    echo '>' . $this->get_value( $field_key, '' ) . '</textarea>';
  }

  public function display_text_field( $field_key, $field_name = false, $field_value = null, $class = false, $field_placeholder = '', $data_attributes = [], $default_value = '' ) {
    $field_name = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );
    $class = ( $class == false ? 'regular-text' : $class );

    echo '<input type="text" 
                 class="' . $class . '" 
                 ' . ( $field_key !== false ? 'id="wpep-' . $field_key . '" ' : '' ) . '
                 value="' . ( $field_value === null ? $this->get_value( $field_key, $default_value ) : $field_value ) . '" 
                 name="' . $field_name . '" 
                 placeholder="' . $field_placeholder . '" ';

    foreach( $data_attributes as $data_attribute_key => $data_attribute_value )
      echo ' data-' . $data_attribute_key . '="' . $data_attribute_value . '" ';

    echo '  >';
  }

  public function display_date_field( $field_key, $field_name = false, $field_value = null, $class = false, $field_placeholder = '' ) {
    $class = ( $class == false ? 'regular-text wpep-datepicker-field' : $class );

    $this->display_text_field( $field_key, $field_name, $field_value, $class, $field_placeholder );
  }

  public function display_attributes_field_content_editor( $field_key, $field_name = false, $field_map = [], $field_default = []) {
    $field_map         = ( !empty( $field_map ) ? $field_map : $this->get_value( $field_key, [] ) );
    $field_name        = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    if( empty( $field_map ) || !is_array( $field_map ) )
      $field_map = $field_default;

    echo '<div class="wpep-attributes-field-container wpep-attributes-field-type-content-editor" data-name="' . $field_name . '">';
    echo '<table class="wp-list-table fixed widefat">';
    echo  '<thead>';
    echo    '<tr>';
    echo      '<th class="wpep-attributes-field-row-drag"></th>';
    echo      '<th class="wpep-attributes-field-row-key">' . __( "Title", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-value">' . __( "Content", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-remove"></th>';
    echo    '</tr>';
    echo  '</thead>';
    echo  '<tbody class="wpep-attributes-field-group-container">';

    foreach( $field_map as $key => $value ) {
      echo '<tr class="wpep-attributes-field-group">';
      echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
      echo  '<td class="wpep-attributes-field-row-key">';
      echo    '<textarea class="wpep-attributes-field" style="display:none;" name="' . $field_name . '[' . $key . ']">' . $value . '</textarea>';
      echo    '<input type="text" class="wpep-attributes-field-key" value="' . $key . '"/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-value">';
      echo  '<div class="wpep-content-editor-container">';
      wpep_editor( $value, 'wpep-afv-' . md5( $key ), false, [
        'editor_class'  => "wpep-attributes-field-value",
        'textarea_rows' => 5,
        'editor_css'    => '<style>body#tinymce { padding : 0 !important; }; </style>'
        ]
      );
      echo  '</div>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
      echo '</tr>';
    }

    echo '<tr class="wpep-attributes-field-group" style="display:none;">';
    echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
    echo  '<td class="wpep-attributes-field-row-key">';
    echo    '<textarea class="wpep-attributes-field" style="display:none;"></textarea>';
    echo    '<input type="text" class="wpep-attributes-field-key" value=""/>';
    echo  '</td>';
    echo  '<td class="wpep-attributes-field-row-value">';
    echo    '<div class="wpep-content-editor-container">';
    echo    '<div class="wpep-notification wpep-notification-info">' . __( "Content Box will be available shortly", "wpep" ). '</div>';
    echo    '</div>';
    echo  '</td>';
    echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
    echo '</tr>';

    echo  '</tbody>';
    echo '</table>';
    echo '<button class="button button-primary wpep-attributes-field-group-add">' . __( "New Field", "wpep" ) . '</button>';
    echo '<div class="clear"></div>';
    echo '</div>';
  }

  public function display_attributes_field_downloads( $field_key, $field_name = false, $field_map = [] ) {
    $field_map         = ( !empty( $field_map ) ? $field_map : $this->get_value( $field_key, [] ) );
    $field_name        = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    if( empty( $field_map ) || ( !is_array( $field_map ) && !is_object( $field_map ) ) )
      $field_map = [];

    echo '<div class="wpep-attributes-field-container wpep-attributes-field-type-downloads" data-name="' . $field_name . '">';
    echo  '<input type="hidden" name="' . $field_name . '" value=""/>';
    echo '<table class="wp-list-table fixed widefat" ' . ( empty( $field_map ) ? 'style="display:none;"' : '' ) . '>';
    echo  '<thead>';
    echo    '<tr>';
    echo      '<th class="wpep-attributes-field-row-drag"></th>';
    echo      '<th class="wpep-attributes-field-row-key">' . __( "Link", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-value">' . __( "Name", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-remove"></th>';
    echo    '</tr>';
    echo  '</thead>';
    echo  '<tbody class="wpep-attributes-field-group-container">';

    foreach( $field_map as $key => $value ) {
      echo '<tr class="wpep-attributes-field-group">';
      echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
      echo  '<td class="wpep-attributes-field-row-key">';
      echo    '<input type="hidden" class="wpep-attributes-field" name="' . $field_name . '[' . $key . ']" value="' . $value . '"/>';
      echo    '<input type="text" class="wpep-attributes-field-key" value="' . $key . '"/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-value">';
      echo    '<input type="text" class="wpep-attributes-field-value" value="' . $value . '"/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
      echo '</tr>';
    }

    echo '<tr class="wpep-attributes-field-group" style="display:none;">';
    echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
    echo  '<td class="wpep-attributes-field-row-key">';
    echo    '<input type="hidden" class="wpep-attributes-field"/>';
    echo    '<input type="text" class="wpep-attributes-field-key" value=""/>';
    echo  '</td>';
    echo  '<td class="wpep-attributes-field-row-value">';
    echo    '<input type="text" class="wpep-attributes-field-value" value=""/>';
    echo  '</td>';
    echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
    echo '</tr>';

    echo  '</tbody>';
    echo '</table>';
    echo '<button class="button button-primary wpep-attributes-field-group-add">' . __( "Add Download", "wpep" ) . '</button>';
    echo '<div class="clear"></div>';
    echo '</div>';
  }

  public function display_attributes_field_jump_links( $field_key, $field_name = false, $field_map = [] ) {
    $field_map         = ( !empty( $field_map ) ? $field_map : $this->get_value( $field_key, [] ) );
    $field_name        = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    if( empty( $field_map ) || !is_array( $field_map ) )
      $field_map = [];

    echo '<div class="wpep-attributes-field-container wpep-attributes-field-type-jump-links" data-name="' . $field_name . '" data-key-glue=":" data-key-default="00">';
    echo  '<input type="hidden" name="' . $field_name . '" value=""/>';
    echo '<table class="wp-list-table fixed widefat" ' . ( empty( $field_map ) ? 'style="display:none;"' : '' ) . '>';
    echo  '<thead>';
    echo    '<tr>';
    echo      '<th class="wpep-attributes-field-row-drag"></th>';
    echo      '<th class="wpep-attributes-field-row-key">' . __( "Timestamp", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-value">' . __( "Title", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-remove"></th>';
    echo    '</tr>';
    echo  '</thead>';
    echo  '<tbody class="wpep-attributes-field-group-container">';

    foreach( $field_map as $key => $value ) {
      $key_pieces = explode( ":", $key );

      echo '<tr class="wpep-attributes-field-group">';
      echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
      echo  '<td class="wpep-attributes-field-row-key wpep-jump-link-input-container">';
      echo    '<input type="hidden" class="wpep-attributes-field" name="' . $field_name . '[' . $key . ']" value="' . $value . '"/>';
      echo    '<input type="text" class="wpep-attributes-field-key" value="' . ( isset( $key_pieces[0] ) ? $key_pieces[0] : '00' ) . '" placeholder="hh"/>';
      echo    '<input type="text" class="wpep-attributes-field-key" value="' . ( isset( $key_pieces[1] ) ? $key_pieces[1] : '00' ) . '" placeholder="mm"/>';
      echo    '<input type="text" class="wpep-attributes-field-key" value="' . ( isset( $key_pieces[2] ) ? $key_pieces[2] : '00' ) . '" placeholder="ss"/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-value">';
      echo    '<input type="text" class="wpep-attributes-field-value" value="' . $value . '"/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
      echo '</tr>';
    }

    echo '<tr class="wpep-attributes-field-group" style="display:none;">';
    echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
    echo  '<td class="wpep-attributes-field-row-key wpep-jump-link-input-container">';
    echo    '<input type="hidden" class="wpep-attributes-field"/>';
    echo    '<input type="text" class="wpep-attributes-field-key" value="00" placeholder="hh"/>';
    echo    '<input type="text" class="wpep-attributes-field-key" value="00" placeholder="mm"/>';
    echo    '<input type="text" class="wpep-attributes-field-key" value="00" placeholder="ss"/>';
    echo  '</td>';
    echo  '<td class="wpep-attributes-field-row-value">';
    echo    '<input type="text" class="wpep-attributes-field-value" value=""/>';
    echo  '</td>';
    echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
    echo '</tr>';

    echo  '</tbody>';
    echo '</table>';
    echo '<button class="button button-primary wpep-attributes-field-group-add">' . __( "Add JumpLink", "wpep" ) . '</button>';
    echo '<div class="clear"></div>';
    echo '</div>';
  }

  public function display_attributes_field( $field_key, $field_name = false, $field_map = [], $field_default = [] ) {
    $field_map         = ( !empty( $field_map ) ? $field_map : $this->get_value( $field_key, [] ) );
    $field_name        = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    if( empty( $field_map ) || !is_array( $field_map ) )
      $field_map = $field_default;

    echo '<div class="wpep-attributes-field-container" data-name="' . $field_name . '">';
    echo '<table class="wp-list-table fixed widefat">';
    echo  '<thead>';
    echo    '<tr>';
    echo      '<th class="wpep-attributes-field-row-key">' . __( "Field Type", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-value">' . __( "Field Value", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-remove"></th>';
    echo    '</tr>';
    echo  '</thead>';
    echo  '<tbody class="wpep-attributes-field-group-container">';

    foreach( $field_map as $key => $value ) {
      echo '<tr class="wpep-attributes-field-group">';
      echo  '<td class="wpep-attributes-field-row-key">';
      echo    '<input type="hidden" class="wpep-attributes-field" name="' . $field_name . '[' . $key . ']" value="' . $value . '"/>';
      echo    '<input type="text" class="wpep-attributes-field-key" value="' . $key . '"/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-value">';
      echo    '<input type="text" class="wpep-attributes-field-value" value="' . $value . '"/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
      echo '</tr>';
    }

    echo  '</tbody>';
    echo '</table>';
    echo '<button class="button button-primary wpep-attributes-field-group-add">' . __( "New Field", "wpep" ) . '</button>';
    echo '<div class="clear"></div>';
    echo '</div>';
  }

  public function display_editor_field( $field_key, $editor_height = 150 ) {
    wpep_editor(
      $this->get_value( $field_key, '' ),
      'wpep-' . $field_key,
      '' . $this->field_prefix . '[' . $field_key . ']',
      [
        'editor_height' => $editor_height
      ]
    );
  }

  public function display_color_field( $field_key ) {
    echo  '<input type="text" class="wpep-color-field" id="wpep-' . $field_key . '" value="' . $this->get_value( $field_key, '' ) . '" name="' . $this->field_prefix . '[' . $field_key . ']">';
  }

  public function display_checkbox_field( $field_key, $is_checked = null, $field_name = null, $default_value = 0, $force_checked = null ) {
    $field_name = ( $field_name === null ? $this->field_prefix . '[' . $field_key . ']' : $field_name );

    if( is_array( $is_checked ) ) {
      $is_checked = array_reverse($is_checked);
      $is_checked = array_shift( $is_checked );
    }

    if( $force_checked == true )
      $is_checked = true;

    echo  '<input type="hidden" value="0" name="' . $field_name . '">';
    echo  '<input type="checkbox"
                  ' . ( $field_key !== false ? 'id="wpep-' . $field_key . '" ' : '' ) . '
                  value="1"
                  name="' . $field_name . '"
                  ' . checked( ( $is_checked === null || $is_checked == '' ? $this->get_value( $field_key, $default_value ) : $is_checked ), true, false ) . ' '
                  . ( $force_checked ? disabled( true, true, false ) : '' ) .
                  '>';
  }

  public function display_checkbox_array_field( $field_key, $field_value, $field_name = null, $identifier = null ) {
    $field_name = ( $field_name === null ? $this->field_prefix . '[' . $field_key . '][]' : $field_name );
    $identifier = ( $identifier === null ? 'wpep-' . $field_key . '-' . $field_value : $identifier );

    echo  '<input type="checkbox"
                  id="' . $identifier . '"
                  value="' . $field_value . '"
                  name="' . $field_name . '"
                  ' . checked( in_array( $field_value, $this->get_value( $field_key ) ), true, false ) . '
                  >';
  }

  public function display_radio_fields( $field_key, $field_options = [], $default_value = '' ) {
    echo '<div class="wpep-radio-container">';

    foreach( $field_options as $field_option_key => $field_option_label ) {
      echo '<div class="wpep-radio">';
      echo '<input type="radio"
                   ' . ( $field_key !== false ? 'id="wpep-' . $field_key . '" ' : '' ) . '
                   value="' . $field_option_key . '"
                   name="' . $this->field_prefix . '[' . $field_key . ']"
                   ' . checked( $this->get_value( $field_key, $default_value ), $field_option_key, false ) . '
                   >';
      echo $field_option_label;
      echo '</div>';
    }

    echo '</div>';
  }

  public function display_select_field( $field_key, $field_options = [], $field_name = false, $current_field_value = null, $class = 'wpep-select-field', $default_value = '', $attributes = [] ) {
    $current_field_value = ( $current_field_value === null ? $this->get_value( $field_key, $default_value ) : $current_field_value );
    $field_name          = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    echo '<select class="' . $class . '" ' . ( $field_key !== false ? 'id="wpep-' . $field_key . '" ' : '' ) . ' name="' . $field_name . '"';

    if( is_array( $attributes ) ) {
      foreach( $attributes as $attribute_key => $attribute_value )
        echo ' ' . $attribute_key . ' = "' . $attribute_value . '"';
    }

    echo '>';

    $this->_display_select_field_options( $field_options, $current_field_value );

    echo '</select>';
  }

  private function _display_select_field_options( $field_options, $current_field_value ) {
    foreach( $field_options as $field_option_key => $field_option_label ) {
      $attributes = [];

      if( is_array( $field_option_label ) ) {
        // This is a Field Option Group
        if( isset( $field_option_label[ 'group_title' ] ) && isset( $field_option_label['group_options'] ) ) {
          echo '<optgroup label="' . $field_option_label[ 'group_title' ] . '">';

          foreach( $field_option_label['group_options'] as $inner_value => $inner_label )
            $this->_display_select_field_option( $inner_value, $current_field_value, $inner_label, [] );

          echo '</optgroup>';

          continue;
        }

        $attributes = $field_option_label;

        unset( $attributes['label'] );

        $field_option_label = isset( $field_option_label['label'] ) ? $field_option_label['label'] : $field_option_key;
      }

      $this->_display_select_field_option( $field_option_key, $current_field_value, $field_option_label, $attributes );
    }
  }

  private function _display_select_field_option( $field_option_key, $field_value, $field_option_label, $attributes = [] ) {
    echo '<option value="' . $field_option_key . '" ';

    foreach( $attributes as $attribute_key => $attribute_value )
      echo 'data-' . str_replace( '_', '-', $attribute_key ) . '="' . $attribute_value . '" ';

    if( is_array( $field_value ) )
      selected( in_array( $field_option_key, $field_value ) );
    else
      selected( $field_option_key, $field_value );

    echo '>' . $field_option_label . '</option>';
  }

  public function display_multiple_select_field( $field_key, $field_options = [], $field_name = false, $field_values = null, $class = 'wpep-multiple-select' ) {
    $field_values = ( $field_values === null ? $this->get_value( $field_key, [], false ) : $field_values );
    $field_name   = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . '][]' );

    echo '<input type="hidden" 
                 name="' . ( wpep_string_ends_with( $field_name, '[]' ) && strpos( $field_key, '_' ) === 0 ? substr( $field_name, 0, -2 ) : $field_name ) . '" 
                 value="' . serialize( [] ) . '">';

    echo '<select ' . ( $field_key !== false ? 'id="wpep-' . $field_key . '" ' : '' ) . ' name="' . $field_name . '" class="' . $class . '" multiple="multiple">';

    $this->_display_select_field_options( $field_options, $field_values );

    echo '</select>';
  }

  public function display_time_field( $field_key, $field_name = false, $field_value = null ) {
    $field_name  = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );
    $field_value = intval( $field_value === null ? $this->get_value( $field_key, '' ) : $field_value );

    $dhms = wpep_time_to_dhms( $field_value );

    echo '<div class="wpep-time-field-container">';
    echo    '<input class="wpep-time-field-value" type="hidden" name="' . $field_name . '" value="' . $field_value . '"/>';

    echo    '<div class="wpep-time-field-input-container wpep-time-field-input-days-container">';
    echo      '<input type="number" class="wpep-time-field-days" min="0" max="360" value="' . $dhms['days'] . '" data-field-multiplier="' . ( 60 * 60 * 24 ) . '"/>';
    echo      '<span>' . __( "Days", "wpep" ) . '</span>';
    echo    '</div>';
    echo    '<div class="wpep-time-field-input-container wpep-time-field-input-hours-container">';
    echo      '<input type="number" class="wpep-time-field-hours" min="0" max="24" value="' . $dhms['hours'] . '" data-field-multiplier="' . ( 60 * 60 ) . '"/>';
    echo      '<span>' . __( "Hours", "wpep" ) . '</span>';
    echo    '</div>';
    echo    '<div class="wpep-time-field-input-container wpep-time-field-input-minutes-container">';
    echo      '<input type="number" class="wpep-time-field-minutes" min="0" max="60" value="' . $dhms['minutes'] . '" data-field-multiplier="60"/>';
    echo      '<span>' . __( "Minutes", "wpep" ) . '</span>';
    echo    '</div>';
    echo    '<div class="wpep-time-field-input-container wpep-time-field-input-seconds-container">';
    echo      '<input type="number" class="wpep-time-field-seconds" min="0" max="60" value="' . $dhms['seconds'] . '" data-field-multiplier="1"/>';
    echo      '<span>' . __( "Seconds", "wpep" ) . '</span>';
    echo    '</div>';

    echo '</div>';
  }


}
