<?php

namespace WPEP\Admin\Coordinator\LearningCenter;

use WPEP\Controller as WPEP_Controller;

class WalkTrough {

  private $_current_walk_trough       = [];
  private $_current_walk_trough_sequence = [];
  private $_current_user_meta   = [];

  public $user_meta_identifier    = '_wpep_walk_trough_sequence';
  public $url_identifier          = 'wpep-walk-trough';
  public $url_identifier_sequence = 'wpep-walk-trough-sequence';

  public function __construct() {
    add_action( 'init', [ $this, '_setup'], 10 );
  }

  public function _setup() {
    if( isset( $_GET[ $this->url_identifier_sequence ] ) && isset( $_GET[ $this->url_identifier ] ) && get_current_user_id() != 0 ) {
      update_user_meta( get_current_user_id(), $this->user_meta_identifier, [
        'started_at'  => time(),
        'sequence'    => $_GET[ $this->url_identifier_sequence ],
        'step'        => $_GET[ $this->url_identifier ]
      ] );

      $current_url = wpep_get_current_url();

      $current_url = remove_query_arg( $this->url_identifier, $current_url );
      $current_url = remove_query_arg( $this->url_identifier_sequence, $current_url );

      wp_redirect( $current_url);
      exit;
    }

    $current_process = ( isset( $_GET[ $this->url_identifier ] ) ? $_GET[ $this->url_identifier ] : '' );

    if( $current_process === '' ) {
      $this->_current_user_meta = get_user_meta( get_current_user_id(), $this->user_meta_identifier, true );

      $current_process = ( isset( $_GET[ $this->url_identifier ] ) ? $_GET[ $this->url_identifier ] : ( isset( $this->_current_user_meta['step'] ) ? $this->_current_user_meta['step'] : '' ) );

      if( $current_process === '' )
        return;

      if( !is_array( $this->_current_user_meta ) )
        $this->_current_user_meta = [];

      if( isset( $this->_current_user_meta['sequence'] ) ) {
        $this->_current_walk_trough_sequence = $this->get_by_alias( $this->_current_user_meta['sequence'] );

        $this->_current_walk_trough_sequence['sequence_reverse'] = array_flip( $this->_current_walk_trough_sequence['sequence'] );
      }
    }

    $current_walk_trough = $this->get_by_alias( $current_process );

    if( !is_array( $current_walk_trough ) )
      return;

    if( isset( $current_walk_trough[ 'page_params' ] ) ) {

      foreach( $current_walk_trough[ 'page_params' ] as $key => $value ) {
        if( !isset( $_GET[ $key ] ) )
          return;

        // Allow any $_GET value in this case, as we track it "exists"
        if( $value == '' )
          continue;

        if( $_GET[ $key ] != $value )
          return;
      }
    }

    $this->_current_walk_trough = $current_walk_trough;

    if( !empty( $this->_current_walk_trough_sequence ) )
      add_action( 'admin_notices', [ $this, 'admin_notices_sequence' ] );

    add_action( 'admin_enqueue_scripts', [ $this, '_admin_enqueue_scripts' ], 10 );
  }

  public function display() {
    $list = $this->get_list();

    if( !is_array( $list ) ) {
      echo '<div class="error notice">';
      echo  '<p>' . __( "Please refresh the page, if this issue persists, please contact support.", 'wpep') . '</p>';
      echo '</div>';

      return;
    }

    echo '<div class="wpep-walk-trough-list wpep-simple-universal-height">';

    foreach( $list as $alias => $information ) {
      if( isset( $information['show_in_list'] ) && !$information['show_in_list'] )
        continue;

      $link = $this->get_step_link( $alias, $information );

      echo '<div class="wpep-walk-trough-block wpep-simple-universal-height-block">';
      echo    '<img src="' . ( isset( $information['image'] ) && $information['image'] != '' ? $information['image'] : WPEP_PLACEHOLDER_IMAGE ) . '"/>';
      echo    '<h2>' . $information['name'] . '</h2>';
      echo    '<div class="wpep-walk-trough-block-description">';
      echo      isset( $information['description'] ) ? $information['description'] : __( "Not Available", "wpep" );
      echo    '</div>';
      echo    '<div class="wpep-walk-trough-block-operations">';
      echo      '<a class="button button-primary" href="' . $link . '">' . __( "Start", "wpep" ) . '</a>';

      if( isset( $information['buttons'] ) ) {
        foreach( $information['buttons'] as $button_text => $button_link ) {
          echo ' <a class="button button-secondary" href="' . $button_link . '" target="_blank">' . $button_text . '</a>';
        }
      }

      echo    '</div>';
      echo '</div>';
    }

    echo '</div>';
  }

  public function get_step_link( $alias, $information = false, $sequence = false ) {
    if( $information === false )
      $information = $this->get_by_alias( $alias );

    if( isset( $information[ 'sequence' ] ) ) {
      $sequence = $alias;
      $alias    = $information[ 'sequence' ][ key( $information[ 'sequence' ] ) ];

      $information = $this->get_by_alias( $alias );
    } else if( is_array( $sequence ) ) {
      $sequence = ( isset( $sequence['alias'] ) ? $sequence['alias'] : false );
    }

    $link = admin_url( $information['page_hook' ] );

    if( isset( $information[ 'page_params'] ) ) {
      foreach( $information['page_params' ] as $key => $value )
        $link = add_query_arg( $key, $value, $link );
    }

    $link = add_query_arg( $this->url_identifier, $alias, $link );

    if( $sequence != false )
      $link = add_query_arg( $this->url_identifier_sequence, $sequence, $link );

    return $link;
  }

  public function get_by_alias( $alias ) {
    $information = $this->get_list();

    return isset( $information[ $alias ] ) ? $information[ $alias ] : false;
  }

  public function get_list() {
    $information = get_transient( 'wpep_walk_troughs_content' );

    if( $information == false || WPEP_WALK_TROUGHS_ENABLE_TRANSIENT == false ) {
      $information = wp_remote_post(WPEP_WALK_TROUGHS_JSON_LINK, [ 'timeout' => 15, 'sslverify' => false ] );

      if( is_wp_error( $information ) )
        return false;

      $information = json_decode( wp_remote_retrieve_body( $information ), true );

      foreach( $information as $key => $info ) {
        if( !isset( $info[ 'name' ] )
          || ( !isset( $info[ 'steps' ] ) && !isset( $info[ 'sequence' ] ) && !isset( $info[ 'page_hook' ] ) ) )
          unset( $information[ $key ] );

        $information[ $key ]['alias'] = $key;
      }

      if( empty( $information ) )
        return false;

      set_transient( 'wpep_walk_troughs_content', $information, 24 * HOUR_IN_SECONDS );
    }

    return $information;
  }

  public function admin_notices_sequence() {
    $current_step_index = ( isset( $this->_current_walk_trough_sequence['sequence_reverse'][ $this->_current_walk_trough['alias'] ] ) ? $this->_current_walk_trough_sequence['sequence_reverse'][ $this->_current_walk_trough['alias'] ] : false );

    if( $current_step_index === false )
      return;

    echo '<div id="wpep-walk-trough-controller" class="notice notice-info is-dismissible">';
    echo    '<h2>' . sprintf(
      __( "%s %s ( step %s of %s )", "wpep" ),
        $this->_current_walk_trough_sequence[ 'name' ],
        $this->_current_walk_trough[ 'name' ],
        $current_step_index + 1,
        count( $this->_current_walk_trough_sequence['sequence'] )
    ) . '</h2>';

    if( isset( $this->_current_walk_trough['description'] ) )
      echo    '<p>' . $this->_current_walk_trough['description'] . '</p>';

    echo '<p class="wpep-operations">';

    if( isset( $this->_current_walk_trough_sequence['sequence'][ $current_step_index - 1 ] ) ) {
      $link = $this->get_step_link( $this->_current_walk_trough_sequence['sequence'][ $current_step_index - 1 ], false, $this->_current_walk_trough_sequence );
      echo    '<a class="button button-secondary wpep-button wpep-button-previous" href="' . $link . '">' .
        __( "Previous Step", "wpep" ) .
        '</a> ';
    }

    echo    '<a id="wpep-shepherd-helper-trigger-open" 
                class="button button-primary wpep-button wpep-button-primary"
                href="' . $this->get_step_link( $this->_current_walk_trough['alias'], $this->_current_walk_trough, $this->_current_walk_trough_sequence ) . '">' .
                  __( "Start Step", "wpep" ) .
            '</a> ';

    if( isset( $this->_current_walk_trough_sequence['sequence'][ $current_step_index + 1 ] ) ) {
      $link = $this->get_step_link( $this->_current_walk_trough_sequence['sequence'][ $current_step_index + 1 ], false, $this->_current_walk_trough_sequence );

      echo    '<a class="button button-secondary wpep-button wpep-button-next" href="' . $link . '">' .
                __( "Next Step", "wpep" ) .
              '</a> ';
    }

    //echo '<a class="button button-secondary wpep-end-walk-trough wpep-button">' . __( "End Walk-trough", "wpep" ) . '</a>';

    echo '</p>';
    echo '</div>';
  }

  public function _admin_enqueue_scripts( $hook ) {
    if( isset( $this->_current_walk_trough[ 'page_hook'] )
        && count( $this->_current_walk_trough['page_params'] ) <= 2 ) {
      if( is_array( $this->_current_walk_trough[ 'page_hook'] ) && !in_array( $hook, $this->_current_walk_trough[ 'page_hook'] ) )
        return;

      if( $this->_current_walk_trough[ 'page_hook'] != $hook )
        return;
    }

    $wpep_walk_trough = $this->_current_walk_trough;

    $wpep_walk_trough['open_on_load'] = intval( empty( $this->_current_walk_trough_sequence ) );

    wp_enqueue_script( 'tether', WPEP_BASE_URL_PATH . 'assets/shepherd/tether.js', [], '0.6.5' , true );
    wp_enqueue_script( 'shepherd', WPEP_BASE_URL_PATH . 'assets/shepherd/shepherd.min.js', [ 'tether' ], '1.8.0' , true );
    wp_enqueue_style( 'shepherd-theme-dark', WPEP_BASE_URL_PATH . 'assets/shepherd/shepherd-theme-dark.css', '1.8.0' );

    wp_register_script('wpep-shepherd-helper',
      WPEP_BASE_URL_PATH . 'assets/admin/shepherd-helper-min.js', [
        'shepherd'
      ], WPEP_VERSION , true
    );
    wp_localize_script( 'wpep-shepherd-helper', 'wpep_walk_trough', $wpep_walk_trough );
    wp_enqueue_script( 'wpep-shepherd-helper' );
  }

}