<?php

namespace WPEP\Admin\Coordinator\Settings;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings   as WPEP_Settings;

class General extends Skeleton {

  public $tabs = [];

  public function _setup_tabs() {
    $this->tabs = [
      'design'     => __( 'Design Options', "wpep" ),
      'auth'       => __( 'Authentication Options', "wpep" ),
      'advanced'   => __( 'Advanced Options', "wpep" ),
      'url'        => __( 'URL Settings', "wpep" ),
      'search'     => __( 'Search Settings', "wpep" )
    ];
  }

  public function get_name() {
    return __( 'General', "wpep" );
  }

  public function get_alias() {
    return 'general';
  }

  public function display() {
    $tab = ( isset( $_GET[ 'tab_inner' ] ) ? $_GET[ 'tab_inner' ] : key( $this->tabs ) );

    $this->display_tabs( $tab );

    if( $tab == 'advanced' )
      $this->display_advanced();
    else if ( $tab == 'url' )
      $this->display_url_options();
    else if( $tab == 'auth' )
      $this->display_auth();
    else if( $tab == 'search' )
      $this->display_search();
    else //if( $tab == 'design' )
      $this->display_design();
  }

  public function display_design() {
    $default_font_size   = ( isset( WPEP_Controller::instance()->templateStyle->get_current_theme()['defaults']['font_size'] )   ? WPEP_Controller::instance()->templateStyle->get_current_theme()['defaults']['font_size'] . 'px' : ' Error ' );
    $default_font_family = ( isset( WPEP_Controller::instance()->templateStyle->get_current_theme()['defaults']['font_family'] ) ? WPEP_Controller::instance()->templateStyle->get_current_theme()['defaults']['font_family']  : ' Error ' );

    $google_font_list = wpep_get_google_font_list();
    $native_font_list = wpep_get_native_font_list();

    $this->display_table_start();

    $this->display_table_select_field(
      __( 'Theme', "wpep" ),
      'wpep-theme',
      wpep_utility_map_to_array_assoc( WPEP_Controller::instance()->templateStyle->get_theme_list(), 'alias', 'name' )
    );

    $this->display_table_select_field(
      __( 'Base Font Size', "wpep" ),
      'wpep-custom-font-size',
      [
        ''   => 'Default - ' . $default_font_size,
      ] + wpep_get_indexed_number_list(10, 20, __( "%value%px", "wpep" ) )
    );

    $this->display_table_select_field(
      __( 'Font Family', "wpep" ),
      'wpep-custom-font-family',
      [
        ''              => 'Default - ' . $default_font_family,
        'inherit'       => 'Inherit Theme Fonts',
        'native_fonts'  => [
          'group_title'   => __( "Native Fonts", "wpep" ),
          'group_options' => array_combine( $native_font_list, $native_font_list )
        ],
        'google_fonts'  => [
          'group_title'   => __( "Google Fonts", "wpep" ),
          'group_options' => array_combine( $google_font_list, $google_font_list )
        ]
      ],
      false,
      'wpep-select-field wpep-js-select-element'
    );

    $this->display_table_color_field(
      __( 'Primary Color', "wpep" ),
      'wpep-custom-color'
    );

    $this->display_table_color_field(
      __( 'Secondary Color', "wpep" ),
      'wpep-custom-color-secondary',
      __( 'This should be a darker or lighter color similar to the primary one', "wpep" )
    );

    $this->display_table_wrapper_options();

    $this->display_table_separator( __( "Other Options", "wpep" ) );

    echo        '<tr id="wpep-table-row-wpep-custom-css">';
    echo          '<th scope="row"><label for="wpep-custom-css">' . __('Custom CSS', "wpep") . '</label></th>';
    echo          '<td>';
    echo            '<textarea id="wpep-custom-css" name="wpep_settings[wpep-custom-css]" style="width:400px;height:100px;">' . WPEP_Settings::instance()->get_option( 'wpep-custom-css', '' ) . '</textarea>';
    echo            '<p>' . __('Enter any custom CSS you want to add into the field above.<br>CSS added here will override the default CSS rules of WPEP.', "wpep") . '</p>';
    echo          '</td>';
    echo        '</tr>';
    

    $this->display_table_end();
  }

  public function display_advanced() {
    $this->display_table_start();

    $this->display_table_separator( __( "Frontend", "wpep" ) );

    $this->display_table_checkbox_field( __('Enable Course Bookmarks', "wpep"),  'enable-course-bookmarks' );
    $this->display_table_checkbox_field( __('Enable Course Deliverables', "wpep"),  'enable-course-deliverables' );
    $this->display_table_checkbox_field( __('Enable Student Identification', "wpep"),  'enable-sid-support' );

    $this->display_table_checkbox_field( __('Enable Member Magnet', "wpep"),  'enable-post-member-magnet-option' );
    $this->display_table_checkbox_field(
      __('Member Magnet Interaction', "wpep"),
      'enable-post-member-magnet-interaction',
      __( "For Execution Plan, Execution Plan Unfolded and Learn Map Course Types Enable Registration or Login upon Content Tab button click action.", "wpep" )
    );

    $this->display_table_checkbox_field(
      __( 'Custom CSS File', "wpep" ),
      'wpep-custom-css-by-file',
      __( "WPEP will generate a Custom CSS file to deliver the Color Variation & Custom CSS, disable this option to embed them directly on page if you encounter CSS problems.", "wpep" )
    );

    $this->display_table_checkbox_field(
      __( 'Enforce RTL Support', "wpep" ),
      'enforce-rtl-support',
      __( "NOTE: By default we're enabling RTL support if WordPress has been configured for RTL.", "wpep" )
    );


    $this->display_table_separator( __( "Administration", "wpep" ) );

    $this->display_table_select_field(
      __( 'Menu Position', "wpep" ),
      'menu-position',
      [
        3 => "Default, at the top",
        2 => "Under Dashboard",
        4 => "Under Dashboard Separator",
        5 => "Under Posts",
        10 => "Under Media",
        15 => "Under Links",
        20 => "Under Pages",
        25 => "Under Comments",
        59 => "Under Separator",
        60 => "Under Appearance",
        65 => "Under Plugins",
        70 => "Under Users",
        75 => "Under Tools",
        80 => "Under Settings",
        99 => "Under Bottom Separator",
      ]
    );

    $this->display_table_select_field(
      __( 'New Course Sections', "wpep" ),
      'admin-course-preset-sections',
      [
        1 => 1,
        2 => 2,
        3 => 3,
        4 => 4,
        5 => 5
      ],
      __( 'Choose the number of Sections that are automatically Pre-Loaded for you when adding a new Course.', "wpep" )
    );



    $this->display_table_select_field(
      __( 'New Course Lessons', "wpep" ),
      'admin-course-preset-lessons',
      [
        1  => 1,
        2  => 2,
        3  => 3,
        4  => 4,
        5  => 5,
        6  => 6,
        7  => 7,
        8  => 8,
        9  => 9,
        10 => 10
      ],
      __( 'Choose the number of Lessons that are automatically Pre-Loaded into Sections when adding a new Course.', "wpep" )
    );

    $this->display_table_checkbox_field(
      __( 'Course Layout Editor', "wpep" ),
      'course-layout-content-editor',
      __( 'If you want to use a page builder to build the course layout, enable this option', "wpep" )
    );

    $this->display_table_checkbox_field(
      __( 'Course Save Alert', "wpep" ),
      'admin-course-save-alert',
      __( "Enable keep your content save alert while editing a course.", "wpep" )
    );

    $this->display_table_checkbox_field(
      __( 'Instant Lesson Editor', "wpep" ),
      'lesson-wp-editor-preload',
      __( 'Disable this if your device behaves poorly while editing lessons.', "wpep" )
    );

    $this->display_table_checkbox_field(
      __( 'Open New Lessons Editor', "wpep" ),
      'lesson-wp-editor-enabled-new',
      __( 'Check this to automate your wysiwyg text editor to open by default when adding a new lesson.', "wpep" )
    );

    $this->display_table_checkbox_field(
      __( 'Editor Integration', "wpep" ),
      'admin-editor-integration',
      __( 'Check this to enable the WPEP editor improvements and shortcode builder.', "wpep" )
    );


    $this->display_table_select_field(
      __( 'Text Editor', "wpep" ),
      'wpep-editor-setup',
      $this->_get_wpep_editor_setup_options(),
      __( 'Chose the editor that will be displayed everywhere within WPEP.', "wpep" )
    );

    $this->display_table_end();
  }

  public function display_url_options() {
    $this->display_table_start();

    $this->display_table_separator( __( "Permalink Name", "wpep" ) );

    $this->display_table_text_field(
      __( 'Courses', "wpep" ),
      'rewrite-slug-courses',
      __( "The permalink prefix goes here, by default courses", "wpep" )
    );

    $this->display_table_text_field(
      __( 'Video', "wpep" ),
      'rewrite-slug-video',
      __( "The permalink prefix goes here, by default video", "wpep" )
    );

    $this->display_table_text_field(
      __( 'E-Book', "wpep" ),
      'rewrite-slug-ebook',
      __( "The permalink prefix goes here, by default ebook", "wpep" )
    );

    $this->display_table_text_field(
      __( 'Offer', "wpep" ),
      'rewrite-slug-offer',
      __( "The permalink prefix goes here, by default offer", "wpep" )
    );

    $this->display_table_text_field(
      __( 'Assessment', "wpep" ),
      'rewrite-slug-assessment',
      __( "The permalink prefix goes here, by default wpep_assessment", "wpep" )
    );

    $this->display_table_text_field(
      __( 'Categories', "wpep" ),
      'rewrite-slug-categories',
      __( "The permalink prefix goes here, by default course_category", "wpep" )
    );

    $this->display_table_separator( __( "Archive Page", "wpep" ) );

    $this->display_table_checkbox_field(
      __( 'Courses', "wpep" ),
      'enable-course-archive',
      __( "By default WordPress will create the Course Archive Page under /courses/ you can disable this here", "wpep" )
    );

    $this->display_table_checkbox_field(
      __( 'Videos', "wpep" ),
      'enable-videos-archive',
      __( "By default WordPress will create the Videos Archive Page under /videos/ you can disable this here", "wpep" )
    );

    $this->display_table_checkbox_field(
      __( 'E-Books', "wpep" ),
      'enable-ebooks-archive',
      __( "By default WordPress will create the E-Books Archive Page under /ebooks/ you can disable this here", "wpep" )
    );

    $this->display_table_checkbox_field(
      __( 'Offers', "wpep" ),
      'enable-offers-archive',
      __( "By default WordPress will create the Offers Archive Page under /offers/ you can disable this here", "wpep" )
    );

    $this->display_table_end();
  }

  public function display_auth() {
    $this->display_table_start();

    $this->display_table_separator( __( "General Settings", "wpep" ) );

    $this->display_table_select_field(
      __( 'Internal Auth Status', "wpep"),
      'user-auth-status-level',
      wpep_get_auth_status_level_options(),
      __( 'By default WPEP does not enforce users to login. Your WP Membership plugin will usually take care of this..', "wpep" )
    );

    $this->display_table_select_field(
      __( "Enable Registration", "wpep" ),
      'user-auth-enable-registration',
      [
        0 => __( 'Inactive', "wpep" ),
        1 => __( 'Active', "wpep" ),
        2 => __( "Active - Display Registration instead of login", "wpep" )
      ],
      __( "Enable this if you want to disable user registration from the WPEP Authentication Form.", "wpep" )
    );

    $this->display_table_checkbox_field(
      __( "Enable Marketing Content", "wpep" ),
      'user-auth-sidebar-marketing-status',
      __( "Enable a Marketing text on the right side of the Authentication Login Page. This applies only for the authentication page that WPEP creates for you..", "wpep" )
    );

    $this->display_table_editor_field(
      __( "Marketing Content", "wpep" ),
      'user-auth-sidebar-marketing-content'
    );

    $this->display_table_separator( __( "Form Fields & Actions", "wpep" ) );

    $this->display_table_text_translated_field(
      __( "Login Identifier", "wpep" ),
      'user-auth-field-login-identifier',
      __( "Username or Email Address", "wpep" )
    );
    $this->display_table_text_translated_field(
      __( "SID", "wpep" ),
      'user-auth-field-sid',
      __( "The Student Identification Field, required for cases like GA Reporting of Student Progress. Can be enabled from Advanced Options", "wpep" )
    );
    $this->display_table_text_translated_field(
      __( "Username", "wpep" ),
      'user-auth-field-username'
    );
    $this->display_table_text_translated_field(
      __( "Email Address", "wpep" ),
      'user-auth-field-email-address'
    );
    $this->display_table_text_translated_field(
      __( "First Name", "wpep" ),
      'user-auth-field-first-name'
    );
    $this->display_table_text_translated_field(
      __( "Last Name" ),
      'user-auth-field-last-name'
    );
    $this->display_table_text_translated_field(
      __( "Password", "wpep" ),
      'user-auth-field-password'
    );
    $this->display_table_text_translated_field(
      __( "Remember me", "wpep" ),
      'user-auth-field-remember-me'
    );
    $this->display_table_text_translated_field(
      __( "Register Account", "wpep" ),
      'user-auth-open-tab-register'
    );
    $this->display_table_text_translated_field(
      __( "Reset Password", "wpep" ),
      'user-auth-open-tab-reset-password'
    );
    $this->display_table_text_translated_field(
      __( "I already have an Account", "wpep" ),
      'user-auth-open-tab-login'
    );
    $this->display_table_text_translated_field(
      __( "Return to Login", "wpep" ),
      'user-auth-return-to-login'
    );

    $this->display_table_text_translated_field(
      __( "Login", "wpep" ),
      'user-auth-operation-login'
    );
    $this->display_table_text_translated_field(
      __( "Register", "wpep" ),
      'user-auth-operation-register'
    );
    $this->display_table_text_translated_field(
      __( "Reset Password", "wpep" ),
      'user-auth-operation-reset-password'
    );

    $this->display_table_separator( __( "Form Notifications", "wpep" ) );

    $this->display_table_text_translated_field(
      __( "Registration Notice", "wpep" ),
      'user-auth-notification-please-register',
      __( "Displays this message when the form defaults to register.", "wpep" )
    );
    $this->display_table_text_translated_field(
      __( "Login Details Required", "wpep" ),
      'user-auth-notification-login-details-required'
    );
    $this->display_table_text_translated_field(
      __( "Incorrect Password", "wpep" ),
      'user-auth-notification-password-incorrect'
    );
    $this->display_table_text_translated_field(
      __( "Successfully Logged In", "wpep" ),
      'user-auth-notification-logged-in'
    );
    $this->display_table_text_translated_field(
      __( "Account Registered", "wpep" ),
      'user-auth-notification-account-registered'
    );
    $this->display_table_text_translated_field(
      __( "Password Reset", "wpep" ),
      'user-auth-notification-password-reset'
    );
    $this->display_table_text_translated_field(
      __( "Already Logged In", "wpep" ),
      'user-auth-notification-already-logged-in'
    );
    $this->display_table_text_translated_field(
      __( "Notification Page Expired", "wpep" ),
      'user-auth-notification-page-expired'
    );

    $this->display_table_end();
  }

  public function display_search() {
    $this->display_table_start();

    $this->display_table_checkbox_field(
      __( "Search within Courses", "wpep" ),
      'search-course-content',
      __( "Enable this option if you want the search to also match courses using their contents, not only their titles, this option may perform poorly on shared hostings.", "wpep" )
    );
    $this->display_table_checkbox_field(
      __( "Search within Description", "wpep" ),
      'search-course-description',
      __( "Enable this option for One-Off Courses Support & Course Description Search.", "wpep" )
    );
    $this->display_table_checkbox_field(
      __( "Image Ribbon" ),
      'search-image-ribbon',
      __( "Enable this option to display the presentation card ribbon text on the image.", "wpep" )
    );

    $this->display_table_text_field(
      __( "Max Results", "wpep" ),
      'search-results-number',
      __( "The maximum number of results displayed when a search is made.", "wpep" )
    );
    $this->display_table_text_translated_field(
      __( "No Results Found", "wpep" ),
      'search-no-results-found'
    );
    $this->display_table_text_translated_field(
      __( "Search Placeholder", "wpep" ),
      'search-input-placeholder'
    );



    $this->display_table_end();
  }

  private function _get_wpep_editor_setup_options() {
    return [
      'default' => 'Default WP Editor - Visual',
      'visual'  => 'Visual WP Editor',
      'html'           => 'Default WP Editor - HTML',
      'medium-editor'  => 'Medium Editor',
      'textarea'       => 'Textarea Only'
    ];
  }

}
