<?php

namespace WPEP\Admin\Coordinator\Settings;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings   as WPEP_Settings;

class IndexPage extends Skeleton {

  public $tabs = [];

  public function _setup_tabs() {
    $this->tabs = [
      'general'   => __( 'General Options', "wpep" ),
      'course'    => __( 'Course Options', "wpep" ),
      'video'     => __( 'Video Options', "wpep" ),
      'ebook'     => __( 'E-Book Options', "wpep" ),
      'offer'     => __( 'Offer Options', "wpep" ),
      'grid'      => __( 'Grid Options', "wpep" )
    ];
  }

  public function get_name() {
    return __( 'Index Page', "wpep" );
  }

  public function get_alias() {
    return 'index_page';
  }

  public function display() {

    $tab = ( isset( $_GET[ 'tab_inner' ] ) ? $_GET[ 'tab_inner' ] : key( $this->tabs ) );

    $this->display_tabs( $tab );

    if( $tab == 'grid' )
      $this->display_grid_options();
    else if( $tab == 'course' )
      $this->display_course_options();
    else if( $tab == 'video' )
      $this->display_video_options();
    else if( $tab == 'ebook' )
      $this->display_ebook_options();
    else if( $tab == 'offer' )
      $this->display_offer_options();
    else
      $this->display_general_options();

  }

  public function display_general_options() {
    $this->display_table_start();

    echo        '<tr>';
    echo          '<th scope="row"><label for="wpep-opt-course-index">' . __('Index Page', "wpep") . '</label></th>';
    echo          '<td>';
    echo            '<select id="wpep-opt-course-index" name="wpep_settings[opt-course-index]" class="wpep-select-field">';
    echo              '<option value="0">' .
                         __( ( WPEP_Settings::instance()->get_option( 'opt-course-index', 0 ) == 0 ? "Select a Page" : "Ignore This Feature" ), "wpep" ) .
                      '</option>';

    foreach( get_posts( [ "post_type" => 'page', 'numberposts' => -1 ] ) as $page ) {
      echo '<option value="' . $page->ID . '" ' . ( selected( WPEP_Settings::instance()->get_option( 'opt-course-index', 0 ), $page->ID, false ) ) . '>' .
              $page->post_title .
           '</option>';
    }

    echo            '</select>';
    echo            '<p>' . sprintf(
                            __( 'You can also use %s to display the course grid on any page.', "wpep" ),
                            wpep_admin_shortcode_builder(
                              'wpep_index',
                              [ 'wpep_admin_shortcode_builder_wpep_index_options' ],
                              '[wpep_index]',
                              false
                            )
                           ). '</p>';
    echo          '</td>';
    echo        '</tr>';

    $this->display_table_select_field(
      'Categories Per Row',
      'index-page-categories-per-row',
      wpep_content_categories_row_options()
    );

    $this->display_table_text_translated_field ( __('Secondary Filter', "wpep"),  'opt-course-secondary-filter', __( 'Personalize the text of the secondary category level filter.', "wpep" ) );

    $this->display_table_select_field( __( "Single Category View", "wpep" ), 'index-page-single-category-primary-view', [
      0 => __( "Show secondary category navigation" ),
      1 => __( "Show primary category navigation" )
    ] );

    $this->display_table_separator( __( "All Categories Filter", "wpep" ) );

    $this->display_table_select_field(
      __( 'Status', "wpep"),
      'index-page-enable-filter-all-categories',
      wpep_flag_setting_field_options(),
      __( "Add an Category Filter tab that will show content from all categories.", "wpep" )
    );

    $this->display_table_text_translated_field( __('Title', "wpep"),  'index-page-filter-all-categories-title' );
    $this->display_table_image_field( __('Image', "wpep"),  'index-page-filter-all-categories-image' );

    $this->display_table_separator( __( "Introduction Text", "wpep" ) );

    $this->display_table_text_field( __('Main heading', "wpep"),  'opt-course-heading', __( 'Enter the main heading of your course index page.', "wpep" ) );

    $this->display_table_editor_field(
      __('Sub Heading', "wpep"),
      'opt-course-sub-heading',
      __('Enter the sub heading of your course index page.', "wpep")
    );

    $this->display_table_end();
  }

  public function display_course_options() {
    $this->display_table_start();

    $this->display_table_select_field(
      'Courses Per Row',
      'index-page-courses-per-row',
      wpep_content_primary_row_options( WPEP_POST_TYPE_COURSE )
    );

    $this->display_table_checkbox_field( __( "Enable Image Hover" ), 'index-page-enable-hover-image', __( "Enable a different image on the presentation card on the Index Page on hover.", "wpep" ) );

    $this->display_table_checkbox_field( __( 'Progress Bars', "wpep" ), 'index-page-progress-bars', __( "Display Progress Bars under each course to hint the student which courses are completed.", "wpep" ) );

    $this->display_table_separator( __( "Start Button", "wpep" ) );

    $this->display_table_text_translated_field ( __('Button Text', "wpep"),  'opt-course-start-text', __( 'Personalize the Course Start Button Text', "wpep" ) );

    $this->display_table_text_translated_field ( __('Coming Soon', "wpep"),  'opt-course-coming-soon', __( 'Personalize the Coming Soon Button Text, when an Course is going to be available soon.', "wpep" ) );

    $this->display_table_color_field( __('Text Colour', "wpep"),  'index-page-start-text-color', __( 'Personalize the Button Text Colour', "wpep" ) );
    $this->display_table_color_field( __('Text Background Colour', "wpep"),  'index-page-start-text-color-background', __( 'Personalize the Button Text Background Colour', "wpep" ) );

    $this->display_table_separator( __( "Ribbon", "wpep" ) );

    $this->display_table_color_field( __('Text Colour', "wpep"),  'index-page-ribbon-text-color', __( 'Personalize the Ribbon Text Colour', "wpep" ) );
    $this->display_table_color_field( __('Background Colour', "wpep"),  'index-page-ribbon-text-color-background', __( 'Personalize the Ribbon  Background Colour', "wpep" ) );

    $this->display_table_end();
  }

  public function display_grid_options() {
    echo '<h2>' . __( "Grid Options", "wpep" ) . '</h2>';

    $this->display_table_start();

    $this->display_table_checkbox_field( __( 'Enforce Ordering', "wpep" ), 'index-page-enforce-ordering', __('Check this to resolve ordering issues on the Index Page', "wpep" ) );

    $this->display_table_end();

    $tabs_list = [
      WPEP_POST_TYPE_COURSE  => __( "Courses", "wpep" ),
      WPEP_POST_TYPE_VIDEO   => __( "Videos", "wpep" ),
      WPEP_POST_TYPE_E_BOOK  => __( "E-Books", "wpep" ),
      WPEP_POST_TYPE_OFFER   => __( "Offers", "wpep" ),
      WPEP_TAXONOMY_CATEGORY => __( "Categories", "wpep" )
    ];

    $tab = ( isset( $_GET[ 'tab_inner_section' ] ) ? $_GET[ 'tab_inner_section' ] : false );

    if( $tab === false || !isset( $tabs_list[ $tab ] ) )
      $tab = key( $tabs_list );

    $this->display_tabs_section( $tabs_list, $tab, 'grid' );

    if( post_type_exists( $tab ) )
      $this->_post_ordering_functionality( $tab, $tabs_list[ $tab ] );
    else if( taxonomy_exists( $tab ) )
      $this->_taxonomy_ordering_functionality( $tab, $tabs_list[ $tab ] );
  }

  private function _post_ordering_functionality( $post_type, $post_name = false ) {
    $post_name = ( $post_name == false ? wpep_alias_to_label( $post_name ) : $post_name );

    echo '<p>' . __( "To re-order your " . $post_name . " Just drag and drop the items below call the point", "wpep" ) . '</p>';

    $post_list_query = new \WP_Query();

    $post_list = $post_list_query->query([
      'post_type'        => $post_type,
      'nopaging'         => true,
      'order'            => 'ASC',
      'orderby'          => 'meta_value',
      'meta_key'         => 'wpep_order',
      'suppress_filters' => true
    ] );

    $post_list = wpep_post_list_ordering( $post_list );

    if( !empty( $post_list ) ) {
      echo '<ul class="wpep-entity-order-functionality">';

      $position = 0;
      foreach( $post_list as $post ) {
        echo '<li><input type="hidden" name="wpep_settings_post_order[' . $post->ID . ']" value="' . $position . '"><span>' . $post->post_title . '</span></li>';

        $position++;
      }

      echo '</ul>';
    } else {
      echo  '<p>' . __('No ' . $post_name . ' available to order.', 'wpep') . '</p>';
    }
  }

  private function _taxonomy_ordering_functionality( $taxonomy, $taxonomy_name = false ) {
    $taxonomy_name = ( $taxonomy_name == false ? wpep_alias_to_label( $taxonomy_name ) : $taxonomy_name );

    echo '<p>' . __( "To re-order your " . $taxonomy_name . " Just drag and drop the items below call the point", "wpep" ) . '</p>';

    $taxonomy_list = get_terms( [ 'taxonomy' => $taxonomy, 'parent' => 0, 'hide_empty' => false ] );

    if( !empty( $taxonomy_list ) ) {
      $this->_taxonomy_single_ordering_functionality( $taxonomy_list, 0 );
    } else {
      echo  '<p>' . __('No ' . $taxonomy_name . ' available to order.', 'wpep') . '</p>';
    }
  }

  private function _taxonomy_single_ordering_functionality( $taxonomy_list, $position = 0 ) {
    if( empty( $taxonomy_list ) )
      return $position;

    $taxonomy_list = wpep_term_list_ordering( $taxonomy_list );

    echo '<ul class="wpep-entity-order-functionality">';

    $position = 0;
    foreach( $taxonomy_list as $taxonomy_item ) {
      echo '<li><input type="hidden" name="wpep_settings_taxonomy_order[' . $taxonomy_item->term_id . ']" value="' . $position . '">';

      echo '<span>' . $taxonomy_item->name . '</span>';

      $position = $this->_taxonomy_single_ordering_functionality(
        get_terms( [ 'taxonomy' => $taxonomy_item->taxonomy, 'parent' => $taxonomy_item->term_id, 'hide_empty' => false ] ),
        ++$position
      );

      echo '</li>';
    }

    echo '</ul>';

    return $position;
  }

  public function display_video_options() {
    $this->display_table_start();

    $this->display_table_text_translated_field( __( 'Archive Title', "wpep" ), 'video-archive-title' );
    $this->display_table_text_translated_field( __( 'Watch Now', "wpep" ), 'video-watch-now' );

    $this->display_table_end();
  }

  public function display_ebook_options() {
    $this->display_table_start();

    $this->display_table_select_field(
      __( 'Items Per Row', "wpep" ),
      'index-page-ebooks-per-row',
      wpep_content_primary_row_options()
    );

    $this->display_table_checkbox_field( __( 'Enable Redirect', "wpep" ),  'ebook-redirect-display', __( 'In case you want to protect your e-books with a membership plugin that is not integrated with WPEP.', "wpep" ) );

    $this->display_table_editor_field( __( 'Redirect Message', "wpep" ), 'ebook-redirect-message', __( 'This message will be displayed if the redirect method is enabled, after 3 seconds the user is redirected to the respective ebook link.', "wpep" ) );

    $this->display_table_text_translated_field( __( 'Redirect Button', "wpep" ), 'ebook-redirect-button' );

    $this->display_table_end();
  }

  public function display_offer_options() {
    $this->display_table_start();

    $this->display_table_separator( __( "Frontend", "wpep" ) );

    $this->display_table_select_field(
      __( 'Offers Per Row', "wpep" ),
      'index-page-offers-per-row',
      wpep_content_primary_row_options()
    );

    $this->display_table_separator( __( "Administration", "wpep" ) );

    $this->display_table_attributes_field(
      __( "Default Attributes", "wpep"),
      'offer-information-map-default',
      __( "When creating a new offer, these fields will be the default displayed, optionally you can even have default values prepared if you want.", "wpep" ),
      [],
      WPEP_Settings::instance()->get_option( 'offer-information-map-default' )
    );

    $this->display_table_end();
  }

}
