<?php

namespace WPEP\Admin\Coordinator\Statistics\Assessments;

use WPEP\Controller as WPEP_Controller;

class Dashboard {

  public function display( $filter_by = [] ) {
    $entries_per_page = 30;
    $current_page     = max( 1, ( isset( $_GET['pagination'] ) ? intval( $_GET['pagination'] ) : 1 ) );

    $table_information = $this->_get_table_data( $entries_per_page, $current_page, $filter_by );

    wpep_admin_display_table(
      __( "Assessments List", "wpep" ),
      $table_information['head'],
      $table_information['body'],
      true,
      $table_information['page_current'],
      $table_information['page_total']
    );
  }
  
  public function download( $filter_by = [] ) {
    $table_information = $this->_get_table_data( 1000000, 1, $filter_by );

    do_action(
      'wpep_admin_download_table',
      'assessments-list-' . wpep_format_date(),
      $table_information['head'],
      $table_information['body']
    );
  }

  private function _get_table_data( $entries_per_page, $current_page, $filter_by = [] ) {
    $table_information = [
      'head'          => [],
      'body'          => [],
      'page_current'  => null,
      'page_total'    => null
    ];

    $assessments_offset = ( $current_page - 1 ) * $entries_per_page;

    $entries_query_information = WPEP_Controller::instance()->assessmentActivity->get_submission_history( $entries_per_page, $assessments_offset, true, $filter_by );

    if( empty( $entries_query_information['entries'] ) )
      return $table_information;

    $table_information['head'] = [
      'name'                    => __( "Name", 'wpep' ),
      'course_identification'   => __( "Course Identification", "wpep" ),
      'students_pending'        => [
        'type'  => 'digits',
        'value' => __( "Students Pending", 'wpep' )
      ],
      'students_pending_review' => [
        'type'  => 'digits',
        'value' => __( "Students Pending Review", 'wpep' )
      ],
      'students_completed'      => [
        'type'  => 'digits',
        'value' => __( "Students Completed", 'wpep' )
      ],
      'students_failed'         => [
        'type'  => 'digits',
        'value' => __( "Students Failed", 'wpep' )
      ],
      'total_submissions'       => [
        'type'  => 'digits',
        'value' => __( "Total Submissions", 'wpep' )
      ],
      'average_score'           => [
        'type'  => 'digits',
        'value' => __( "Average Score", "wpep" )
      ],
      'view_submissions'        => [
        'type'  => 'link-item',
        'value' => ''
      ],
    ];

    if( !wpep_has_sid_support() )
      unset( $table_information['head']['course_identification'] );

    foreach( $entries_query_information['entries'] as $entry ) {
      $body = [
        'name'                    => [
          'type'      => 'link',
          'value'     => '<a href="' . wpep_admin_url_statistics( [ 'tab' => 'assessments', 'assessment_id' => $entry['assessment_id'] ] ) . '">' . $entry['assessment_title']    . '</a>',
          'value_raw' => $entry['assessment_title']
        ],
        'students_pending'        => [
          'type'      => 'digits',
          'value'     => $entry['submissions_pending'] . ' <span>' . wpep_get_percentage( $entry['submissions_pending'], $entry['submissions_total'], '( ', '% )' ) . '</span>',
          'value_raw' => $entry['submissions_pending']
        ],
        'students_pending_review' => [
          'type'      => 'digits',
          'value'     => $entry['submissions_pending_review'] . ' <span>' . wpep_get_percentage( $entry['submissions_pending_review'], $entry['submissions_total'], '( ', '% )' ) . '</span>',
          'value_raw' => $entry['submissions_pending_review']
        ],
        'students_completed'      => [
          'type'      => 'digits',
          'value'     => $entry['submissions_completed'] . ' <span>' . wpep_get_percentage( $entry['submissions_completed'], $entry['submissions_total'], '( ', '% )' ) . '</span>',
          'value_raw' => $entry['submissions_completed']
        ],
        'students_failed'         => [
          'type'      => 'digits',
          'value'     => $entry['submissions_failed'] . ' <span>' . wpep_get_percentage( $entry['submissions_failed'], $entry['submissions_total'], '( ', '% )' ) . '</span>',
          'value_raw' => $entry['submissions_failed']
        ],
        'total_submissions'       => [
          'type'      => 'digits',
          'value'     => $entry['submissions_total'],
          'value_raw' => $entry['submissions_total']
        ],
        'average_score'           => [
          'type'  => 'digits',
          'value' => ( $entry['average_score'] == 0 ? '-' : $entry['average_score'] . '%' )
        ],
        'view_submissions'        => [
          'type'  => 'link-item',
          'value' => '<a class="button button-primary" style="width: 140px;" href="' . wpep_admin_url_statistics( [ 'tab' => 'assessments', 'assessment_id' => $entry['assessment_id'] ] ) . '">' . __( "View Submissions", "wpep" )    . '</a>'
        ]
      ];

      if( wpep_has_sid_support() )
        $body['course_identification'] = get_post_meta( wpep_get_assessment_course_id( $entry['assessment_id'] ), 'wpep_course_identifier', true );

      $table_information['body'][] = $body;
    }
    
    $table_information['page_current'] = $current_page;
    $table_information['page_total']   = ceil( $entries_query_information['entries_total'] / $entries_per_page );

    return $table_information;
  }

}