<?php

namespace WPEP\Admin\Coordinator\Statistics\Assessments;

use WPEP\Controller as WPEP_Controller;

class Single {

  public function display( $assessment_id = false, $filter_by = [] ) {
    $assessment_id    = ( $assessment_id === false ? $_GET['assessment_id'] : $assessment_id );

    $table_information = $this->_get_table_data( $assessment_id, $filter_by );

    if( function_exists( 'wpep_admin_url_statistics' ) ) {
      echo wpep_breadcrumb([
        wpep_admin_url_statistics(['tab' => 'assessments']) => __("Assessment List", "wpep"),
        false => get_the_title($assessment_id)
      ]);
    }

    wpep_admin_display_table(
      __( "User List", "wpep" ),
      $table_information['head'],
      $table_information['body'],
      true
    );
  }

  public function download( $assessment_id = false, $filter_by = [] ) {
    $assessment_id     = ( $assessment_id === false ? $_GET['assessment_id'] : $assessment_id );
    $table_information = $this->_get_table_data( $assessment_id, $filter_by );

    do_action(
      'wpep_admin_download_table',
      'assessment-' . get_the_title( $assessment_id ) . '-' .
      ( isset( $filter_by['user_role'] ) && $filter_by['user_role'] !== false ? $filter_by['user_role'] . '-' : '' ) .
      wpep_format_date(),
      $table_information['head'],
      $table_information['body']
    );
  }

  private function _get_table_data( $assessment_id, $filter_by = [] ) {
    $table_information = [
      'head'          => [],
      'body'          => []
    ];

    $user_map = WPEP_Controller::instance()->userActivity->assessment->get_list_statistics_by_assessment_id( $assessment_id, $filter_by );

    if( empty( $user_map ) )
      return $table_information;

    $table_information['head'] = [
      'name'                    => __( "Name", 'wpep' ),
      'student_identification'  => __( "SID", 'wpep' ),
      'status'                  => __( "Status", 'wpep' ),
      'score'                   => [
        'type'  => 'digits',
        'value' => __( "Score", 'wpep' )
      ],
      'correct_answers'         => [
        'type'  => 'digits',
        'value' => __( "Correct Answers", 'wpep' )
      ],
      'wrong_answers'           => [
        'type'  => 'digits',
        'value' => __( "Wrong Answers", 'wpep' )
      ],
      'started_date'            => [
        'type'  => 'digits',
        'value' => __( "Started Date", 'wpep' )
      ],
      'completed_date'          => [
        'type'  => 'digits',
        'value' => __( "Completed Date", 'wpep' )
      ],
      'view_submissions'        => [
        'type'  => 'link-item',
        'value' => ''
      ],
    ];

    if( !wpep_has_sid_support() )
      unset( $table_information['head']['student_identification'] );

    foreach( $user_map as $user_id => $entry ) {

      $user_information = get_user_by( 'ID', $user_id );

      if( $user_information == null )
        continue;

      $body = [
        'name'                    => [
          'type'      => 'link',
          'value'     => '<a href="' . wpep_admin_url_statistics( [ 'tab' => 'assessments', 'user_id' => $user_id ] ) . '">' . $user_information->user_nicename . '</a>',
          'value_raw' => wpep_content_user_profile_format_name( $user_information )
        ],
        'status'                  => [
          'class'   => "status status-" . $entry['status'],
          'value'   => wpep_alias_to_label( $entry['status'] )
        ],
        'score'                   => [
          'type'  => 'digits',
          'value' => wpep_get_percentage( $entry['score'] )
        ],
        'correct_answers'         => [
          'type'      => 'digits',
          'value'     => $entry['correct_answers'] . ' <span>' . wpep_get_percentage( $entry['correct_answers'], $entry['total_answers'], '( ', '% )' ),
          'value_raw' => $entry['correct_answers']
        ],
        'wrong_answers'           => [
          'type'      => 'digits',
          'value'     => $entry['incorrect_answers'] . ' <span>' . wpep_get_percentage( $entry['incorrect_answers'], $entry['total_answers'], '( ', '% )' ),
          'value_raw' => $entry['incorrect_answers']
        ],
        'started_date'            => [
          'type'  => 'digits',
          'value' => wpep_format_date( $entry['start_time'] )
        ],
        'completed_date'          => [
          'type'  => 'digits',
          'value' => ( $entry['submission_time'] == 0 ? '-' : wpep_format_date( $entry['submission_time'] ) )
        ],
        'view_submissions'        => [
          'type'  => 'link-item',
          'value' => '<a class="button button-primary" style="width: 100px;" href="' . wpep_admin_url_statistics( [ 'tab' => 'assessments', 'assessment_id' => $assessment_id, 'user_id' => $user_id ] ) . '">' . __( "View Report", "wpep" )    . '</a>'
        ],
      ];

      if( wpep_has_sid_support() )
        $body['student_identification'] = get_user_meta( $user_information->ID, WPEP_USER_META_STUDENT_IDENTIFICATION, true );

      $table_information['body'][] = $body;
    }

    return $table_information;
  }

}