<?php

namespace WPEP\Admin\Coordinator\Statistics\Assessments;

use WPEP\Controller as WPEP_Controller;

class User {

  public function display( $user_id = false ) {
    $user_id          = ( $user_id === false ? $_GET['user_id'] : $user_id );
    $user_information = get_user_by( 'ID', $user_id );

    if( $user_information == null )
      return;

    $table_information = $this->_get_table_data( $user_id );

    if( function_exists( 'wpep_admin_url_statistics' ) ) {
      echo wpep_breadcrumb([
        wpep_admin_url_statistics(['tab' => 'assessments']) => __("Assessment List", "wpep"),
        false => sprintf(__("%ss' Assessments", "wpep"), ucfirst($user_information->user_nicename))
      ]);
    }

    wpep_admin_display_table(
      __( "User Assessment List", "wpep" ),
      $table_information['head'],
      $table_information['body'],
      true
    );
  }

  public function download( $user_id = false ) {
    $user_id          = ( $user_id === false ? $_GET['user_id'] : $user_id );
    $user_information = get_user_by( 'ID', $user_id );

    if( $user_information == null )
      return;

    $table_information = $this->_get_table_data( $user_id );

    do_action(
      'wpep_admin_download_table',
      'assessment-' . $user_information->user_nicename . '-' . wpep_format_date(),
      $table_information['head'],
      $table_information['body']
    );
  }

  private function _get_table_data( $user_id ) {
    $table_information = [
      'head'          => [],
      'body'          => []
    ];

    $list_statistics = WPEP_Controller::instance()->userActivity->assessment->get_list_statistics( $user_id );

    if( empty( $list_statistics ) )
      return $table_information;

    $table_information['head'] = [
      'name'                    => __( "Name", 'wpep' ),
      'status'                  => __( "Status", 'wpep' ),
      'score'                   => [
        'type'  => 'digits',
        'value' => __( "Score", 'wpep' )
      ],
      'correct_answers'         => [
        'type'  => 'digits',
        'value' => __( "Correct Answers", 'wpep' )
      ],
      'wrong_answers'           => [
        'type'  => 'digits',
        'value' => __( "Wrong Answers", 'wpep' )
      ],
      'started_date'            => [
        'type'  => 'digits',
        'value' => __( "Started Date", 'wpep' )
      ],
      'completed_date'          => [
        'type'  => 'digits',
        'value' => __( "Completed Date", 'wpep' )
      ],
      'view_submissions'        => [
        'type'  => 'link-item',
        'value' => ''
      ]
    ];

    foreach( $list_statistics as $entry ) {
      if( !wpep_user_has_statistics_access_for_post( $entry['assessment_id'] ) )
        continue;

      $table_information['body'][] = [
        'name'                    => [
          'type'      => 'link',
          'value'     => '<a href="' . wpep_admin_url_statistics( [ 'tab' => 'assessments', 'assessment_id' => $entry['assessment_id'] ] ) . '">' . $entry['assessment_title'] . '</a>',
          'value_raw' => $entry['assessment_title']
        ],
        'status'                  => [
          'class'   => "status status-" . $entry['status'],
          'value'   => wpep_alias_to_label( $entry['status'] )
        ],
        'score'                   => [
          'type'  => 'digits',
          'value' => wpep_get_percentage( $entry['score'] )
        ],
        'correct_answers'         => [
          'type'      => 'digits',
          'value'     => $entry['correct_answers'] . ' <span>' . wpep_get_percentage( $entry['correct_answers'], $entry['total_answers'], '( ', '% )' ),
          'value_raw' => $entry['correct_answers']
        ],
        'wrong_answers'           => [
          'type'      => 'digits',
          'value'     => $entry['incorrect_answers'] . ' <span>' . wpep_get_percentage( $entry['incorrect_answers'], $entry['total_answers'], '( ', '% )' ),
          'value_raw' => $entry['incorrect_answers']
        ],
        'started_date'            => [
          'type'  => 'digits',
          'value' => wpep_format_date( $entry['start_time'] )
        ],
        'completed_date'          => [
          'type'  => 'digits',
          'value' => ( $entry['submission_time'] == 0 ? '-' : wpep_format_date( $entry['submission_time'] ) )
        ],
        'view_submissions'        => [
          'type'  => 'link-item',
          'value' => '<a class="button button-primary" style="width: 100px;" href="' . wpep_admin_url_statistics( [ 'tab' => 'assessments', 'assessment_id' => $entry['assessment_id'], 'user_id' => $user_id ] ) . '">' . __( "View Report", "wpep" )    . '</a>'
        ],
      ];

    }

    if( empty( $table_information['body'] ) )
      $table_information['head'] = [];

    return $table_information;
  }

}