<?php

namespace WPEP\Admin\Coordinator\Statistics\Assessments;

use WPEP\Entity\Assessment as EntityAssessment;
use WPEP\Admin\Container\Configuration as WPEP_Container_Configuration;
use WPEP\Controller as WPEP_Controller;

class UserSingle {

  use WPEP_Container_Configuration;

  public function display( $assessment_id, $user_id ) {
    $user_information = get_user_by( 'ID', $user_id );

    if( $user_information == null )
      return;

    $assessment_title = get_the_title( $assessment_id );

    if( function_exists( 'wpep_admin_url_statistics' ) ) {
      echo wpep_breadcrumb( [
        wpep_admin_url_statistics( [ 'tab' => 'assessments' ] )                                    => __( "Assessment List", "wpep" ),
        wpep_admin_url_statistics( [ 'tab' => 'assessments', 'user_id' => $user_id ] )             => sprintf( __( "%ss' Assessments", "wpep" ), ucfirst( $user_information->user_nicename ) ),
        wpep_admin_url_statistics( [ 'tab' => 'assessments', 'assessment_id' => $assessment_id ] ) => sprintf( __( "Assessment %s", "wpep" ), $assessment_title )
      ] );
    }

    if( isset( $_POST[ 'submit_assessment_review' ] ) ) {
      foreach( $_POST['question_is_correct'] as $question_id => $is_correct )
        WPEP_Controller::instance()->userActivity->assessment->set_user_data( 'is_correct', $is_correct, $assessment_id, $question_id, 0, $user_id );

      WPEP_Controller::instance()->userActivity->assessment->handle_user_submission_grading( $assessment_id, $user_id );

      echo '<div class="updated notice is-dismissible">';
      echo  '<p>' . __('Assessment submission has been updated.', 'wpep') . '</p>';
      echo  '<button class="notice-dismiss" type="button"><span class="screen-reader-text">' . __( "Dismiss this notice.", 'wpep') . '</span></button>';
      echo '</div>';
    }

    $questions = EntityAssessment::instance()->get_questions_by_assessment_id( $assessment_id, true, false );

    $user_answers = wpep_utility_map_to_array_assoc( WPEP_Controller::instance()->userActivity->assessment->get_user_data(
      'answer', $assessment_id, false, false, $user_id, false
    ), 'question_id', 'value', true );

    $question_answer_status_list = wpep_utility_map_to_array_assoc( WPEP_Controller::instance()->userActivity->assessment->get_user_data(
      'is_correct', $assessment_id, false, false, $user_id, false
    ), 'question_id', 'value', true );

    $status = wpep_user_assessment_status( $assessment_id, $user_id );

    $score = WPEP_Controller::instance()->userActivity->assessment->get_user_data_value(
      WPEP_USER_ASSESSMENT_SCORE, $assessment_id, 0, 0, $user_id
    );

    $start_time = WPEP_Controller::instance()->userActivity->assessment->get_user_data_value(
      WPEP_USER_ASSESSMENT_START_TIME, $assessment_id, 0, 0, $user_id
    );
    $submitted_time = WPEP_Controller::instance()->userActivity->assessment->get_user_data_value(
      WPEP_USER_ASSESSMENT_SUBMISSION_TIME, $assessment_id, 0, 0, $user_id
    );

    echo '<form class="wpep-assessment-review-form" method="POST">';
    echo  '<div class="left-side">';

    if( wpep_has_sid_support() )
      echo  '<p>' . __( "SID", "wpep" ) . ' : ' . get_user_meta( $user_information->ID, WPEP_USER_META_STUDENT_IDENTIFICATION, true ) . '</p>';

    echo  '<p>' . sprintf( __( "Status : %s", "wpep" ), '<strong>' . wpep_alias_to_label( $status ) . '</strong>' ) . '</p>';

    if( $score != 0 )
      echo  '<p>' . sprintf( __( "Score : %s", "wpep" ), '<strong>' . wpep_get_percentage( $score ) . '</strong>'  ) . '</p>';

    if( $submitted_time != 0 )
      echo  '<p>' . sprintf( __( "Submitted Answers : %s", "wpep" ), '<strong>' . wpep_format_date( $submitted_time ) . '</strong>'  ) . '</p>';

    if( $start_time != 0 )
      echo  '<p>' . sprintf( __( "Started : %s", "wpep" ), '<strong>' . wpep_format_date( $start_time ) . '</strong>'  ) . '</p>';

    echo  '</div>';
    echo  '<div class="right-side">';
    echo  '<table class="wpep-assessment-review wp-list-table striped widefat">';
    echo    '<thead>';
    echo      '<tr>';
    echo        '<td style="width:20px;">&nbsp;</td>';
    echo        '<td style="width:100px;">' . __( "Is Correct", "wpep" ) . '</td>';
    echo        '<td>' . __( "Title", "wpep" ) . '</td>';
    echo      '</tr>';
    echo    '</thead>';
    echo    '<tbody>';

    $i = 1; foreach( $questions as $question ) {
      echo '<tr class="information-row">';
      echo    '<td>' . $i . '</td>';
      echo    '<td>';
      $this->display_checkbox_field(
        'is_correct',
        ( isset( $question_answer_status_list[$question->id] ) ? $question_answer_status_list[$question->id] : 0 ),
        'question_is_correct[' . $question->id . ']'
      );
      echo    '</td>';
      echo    '<td>' . $question->title . '</td>';
      echo '</tr>';
      echo '<tr>';
      echo    '<td colspan="2">' . __( "Answer(s)", "wpep" ) . ' : </td>';
      echo    '<td>';
      if( is_array( $user_answers[$question->id] ) ) {
        echo '<ul>';

        foreach( $user_answers[$question->id] as $user_answer )
          echo '<li>' . wpautop( make_clickable( $user_answer ) ) . '</li>';

        echo '</ul>';
      } else {
        echo '<div class="answer-container">' . $user_answers[$question->id] . '</div>';
      }
      echo    '</td>';
      echo '</tr>';
      $i++;
    }
    echo    '</tbody>';
    echo    '</table>';
    echo    '</br>';
    echo    '<input class="button button-primary" type="submit" value="' . __( "Review Assessment", "wpep" ) . '" name="submit_assessment_review"/>';
    echo  '</div>';
    echo  '<div class="clear"></div>';
    echo '</form>';
  }

}