<?php

namespace WPEP\Admin\Coordinator\Statistics\Courses;

use WPEP\Controller as WPEP_Controller;

class Single {

  protected $table_tabs = [];
  private   $_current_tab = false;

  public function __construct( $course_id ) {
    foreach( [
               new Single\Dashboard(),
               new Single\UserList(),
               new Single\LessonUserList()
             ] as $tab )
      $this->table_tabs[ $tab->get_alias() ] = $tab;

    if( wpep_course_type_from_id( $course_id ) === WPEP_COURSE_TYPE_ONE_OFF )
      unset( $this->table_tabs['dashboard'] );

    $this->table_tabs = apply_filters( "wpep_administration_statistics_courses_single_tabs", $this->table_tabs, $course_id );
  }

  private function _get_current_tab() {
    if( $this->_current_tab !== false )
      return $this->table_tabs[ $this->_current_tab ];

    $this->_current_tab = ( isset( $_GET['view'] ) ? $_GET['view'] : false );
    $this->_current_tab = ( isset( $this->table_tabs[ $this->_current_tab ] ) ? $this->_current_tab : key( $this->table_tabs ) );

    return $this->table_tabs[ $this->_current_tab ];
  }

  public function display( $course_id = false, $filter_by = [] ) {
    $lesson_id = isset( $_GET['lesson_id'] ) ? intval( $_GET[ 'lesson_id' ] ) : false;

    $bread_crumbs = [
      wpep_admin_url_statistics( [ 'tab' => 'courses' ] ) => __( "Course List", "wpep" ),
      wpep_admin_url_statistics( [ 'tab' => 'courses', 'course_id' => $course_id ] ) => get_the_title( $course_id ),
      false => $this->_get_current_tab()->get_name( $course_id, $lesson_id )
    ];

    echo wpep_breadcrumb( $bread_crumbs );

    $table_information = $this->_get_current_tab()->get_table_data(
      $course_id,
      $lesson_id,
      $filter_by
    );

    $table_navigation = [];

    foreach( $this->table_tabs as $table_tab ) {
      if( method_exists( $table_tab, 'is_public' ) && $table_tab->is_public() === false ) {
        if( $this->_current_tab != $table_tab->get_alias() )
          continue;
      }

      $key = ( $this->_current_tab != $table_tab->get_alias()
                ? wpep_admin_url_statistics( [ 'tab' => 'courses', 'course_id' => $course_id, 'view' => $table_tab->get_alias() ] )
                : false
             );

      $table_navigation[ $key ] = $table_tab->get_name( $course_id, $lesson_id );
    }

    wpep_admin_display_table(
      $table_navigation,
      $table_information['head'],
      $table_information['body'],
      true
    );
  }

  public function download( $course_id = false, $filter_by = [] ) {
    $lesson_id = isset( $_GET['lesson_id'] ) ? intval( $_GET[ 'lesson_id' ] ) : false;

    $table_information = $this->_get_current_tab()->get_table_data(
      $course_id,
      $lesson_id,
      $filter_by
    );

    do_action(
      'wpep_admin_download_table',
      $this->_get_current_tab()->get_file_alias(
        $course_id,
        $lesson_id,
        $filter_by
      ),
      $table_information['head'],
      $table_information['body']
    );
  }

}