<?php

namespace WPEP\Admin\Coordinator\Statistics\Courses\Single;

use WPEP\Controller as WPEP_Controller;

class UserList {

  public function get_alias() {
    return 'user_list';
  }

  public function get_name() {
    return __( "User List", "wpep" );
  }

  public function get_file_alias( $course_id, $lesson_id = false, $filter_by = [] ) {
    return 'course-' . get_the_title( $course_id ) . '-' .
            ( isset( $filter_by['user_role'] ) && $filter_by['user_role'] !== false ? $filter_by['user_role'] . '-' : '' ) .
            'user-list-' . wpep_format_date();
  }

  public function get_table_data( $course_id, $lesson_id = false, $filter_by = [] ) {
    $table_information = [
      'head'          => [],
      'body'          => []
    ];

    $user_map = WPEP_Controller::instance()->courseActivity->get_course_started_users( $course_id );

    if( empty( $user_map ) )
      return $table_information;

    $table_information['head'] = [
      'name'                    => __( "User", 'wpep' ),
      'student_identification'  => __( "SID", 'wpep' ),
      'status'                  => __( "Status", 'wpep' ),
      'progress'                => [
        'type'  => 'digits',
        'value' => __( "Progress", 'wpep' )
      ],
      'course_started'          => [
        'type'  => 'digits',
        'value' => __( "Course Started", 'wpep' )
      ],
      'last_lesson_completed'   => [
        'type'  => 'digits',
        'value' => __( "Last Lesson Completed", 'wpep' )
      ],
      'view_submissions'        => [
        'type'  => 'link-item',
        'value' => ''
      ],
    ];

    if( !wpep_has_sid_support() )
      unset( $table_information['head']['student_identification'] );

    foreach( $user_map as $user_id => $started_at ) {
      $user_information = get_user_by( 'ID', $user_id );

      if( $user_information == NULL )
        continue;

      if( !wpep_user_filter_requirement_met( $user_information, $filter_by ) )
        continue;

      $last_lesson_completed = WPEP_Controller::instance()->userActivity->get_course_lesson_completed_date( $course_id, $user_id, 'DESC' );

      $body = [
        'name'                    => [
          'type'      => 'link',
          'value'     => '<a href="' . wpep_admin_url_statistics( [ 'tab' => 'courses', 'user_id' => $user_id ] ) . '">' . $user_information->user_nicename . '</a>',
          'value_raw' => wpep_content_user_profile_format_name( $user_information )
        ],
        'status'                  => [
          'class'   => "status",
          'value'   => __( WPEP_Controller::instance()->userActivity->has_completed_course( $course_id, $user_id ) ? 'Completed' : 'Progressing' , 'wpep' )
        ],
        'progress'                   => [
          'type'  => 'digits',
          'value' => wpep_get_percentage( WPEP_Controller::instance()->userActivity->course_progress_percentage( $course_id, $user_id, false ) )
        ],
        'course_started'             => [
          'type'  => 'digits',
          'value' => wpep_format_date( $started_at )
        ],
        'last_lesson_completed'          => [
          'type'  => 'digits',
          'value' => ( $last_lesson_completed == 0 ? '-' : wpep_format_date( $last_lesson_completed ) )
        ],
        'view_submissions'        => [
          'type'  => 'link-item',
          'value' => '<a class="button button-primary" style="width: 100px;" href="' . wpep_admin_url_statistics( [ 'tab' => 'courses', 'course_id' => $course_id, 'user_id' => $user_id ] ) . '">' . __( "View Report", "wpep" )    . '</a>'
        ],
      ];

      if( wpep_has_sid_support() )
        $body['student_identification'] = get_user_meta( $user_information->ID, WPEP_USER_META_STUDENT_IDENTIFICATION, true );

      $table_information['body'][] = $body;
    }

    if( empty( $table_information['body'] ) )
      $table_information['head'] = [];

    return $table_information;
  }

}