<?php

namespace WPEP\Admin\Coordinator\Statistics\Courses;

use WPEP\Controller as WPEP_Controller;

class User {

  public function display( $user_id = false, $filter_by = [] ) {
    $user_id          = ( $user_id === false ? $_GET['user_id'] : $user_id );
    $user_information = get_user_by( 'ID', $user_id );

    if( $user_information == null )
      return;

    $table_information = $this->_get_table_data( $user_id, $filter_by );

    echo wpep_breadcrumb( [
      wpep_admin_url_statistics( [ 'tab' => 'courses' ] ) => __( "Course List", "wpep" ),
      false => sprintf( __( "%ss' Courses", "wpep" ), ucfirst( $user_information->user_nicename ) )
    ] );

    wpep_admin_display_table(
      __( "User Course List", "wpep" ),
      $table_information['head'],
      $table_information['body'],
      true
    );
  }

  public function download( $user_id = false, $filter_by = [] ) {
    $user_id          = ( $user_id === false ? $_GET['user_id'] : $user_id );
    $user_information = get_user_by( 'ID', $user_id );

    if( $user_information == null )
      return;

    $table_information = $this->_get_table_data( $user_id, $filter_by );

    do_action(
      'wpep_admin_download_table',
      'courses-' . $user_information->user_nicename . '-' . wpep_format_date(),
      $table_information['head'],
      $table_information['body']
    );
  }

  private function _get_table_data( $user_id, $filter_by = [] ) {
    $table_information = [
      'head'          => [],
      'body'          => []
    ];

    $list_statistics = WPEP_Controller::instance()->userActivity->statistics->get_started_course_list( $user_id );

    if( empty( $list_statistics ) )
      return $table_information;

    $table_information['head'] = [
      'name'                    => __( "Course Name", 'wpep' ),
      'status'                  => __( "Status", 'wpep' ),
      'progress'                => [
        'type'  => 'digits',
        'value' => __( "Progress", 'wpep' )
      ],
      'started_at'  => [
        'type'  => 'digits',
        'value' => __( "Started At", 'wpep' )
      ],
      'view_submissions'        => [
        'type'  => 'link-item',
        'value' => ''
      ]
    ];

    foreach( $list_statistics as $course_id => $started_at ) {
      if( !wpep_user_has_statistics_access_for_post( $course_id ) )
        continue;

      $title = get_the_title( $course_id );

      $table_information['body'][] = [
        'name'                    => [
          'type'      => 'link',
          'value'     => '<a href="' . wpep_admin_url_statistics( [ 'tab' => 'courses', 'course_id' => $course_id ] ) . '">' . $title    . '</a>',
          'value_raw' => $title
        ],
        'status'                  => __( WPEP_Controller::instance()->userActivity->get_course_user_data_value( WPEP_USER_COURSE_COMPLETED, $course_id, false, false, $user_id ) ? 'Completed' : 'Progressing' , 'wpep' ),
        'progress'                => [
          'type'  => 'digits',
          'value' => wpep_get_percentage( WPEP_Controller::instance()->userActivity->course_progress_percentage( $course_id, $user_id, false ) )
        ],
        'started_at'                => [
          'type'  => 'digits',
          'value' => wpep_format_date( $started_at )
        ],
        'view_submissions'        => [
          'type'  => 'link-item',
          'value' => '<a class="button button-primary" style="width: 100px;" href="' . wpep_admin_url_statistics( [ 'tab' => 'courses', 'course_id' => $course_id, 'user_id' => $user_id ] ) . '">' . __( "View Report", "wpep" )    . '</a>'
        ],
      ];

    }

    if( empty( $table_information['body'] ) )
      $table_information['head'] = [];

    return $table_information;
  }

}