<?php

namespace WPEP\Admin\Coordinator\Statistics\Courses;

use WPEP\Controller as WPEP_Controller;

class UserSingle {

  public function display( $course_id, $user_id ) {
    $user_information = get_user_by( 'ID', $user_id );
    $course_list      = get_posts( [ 'post_type' => WPEP_POST_TYPE_COURSE, 'nopaging' => true, 'orderby' => 'title', 'author' => wpep_user_statistics_query_author_id() ] );

    echo wpep_breadcrumb( [
      wpep_admin_url_statistics( [ 'tab' => 'courses' ] ) => __( "Course List", "wpep" ),
      wpep_admin_url_statistics( [ 'tab' => 'courses', 'user_id' => $user_id ] ) => sprintf( __( "%ss' Course List", "wpep" ), ucfirst( $user_information->user_nicename ) ),
      false => sprintf( __( "%ss' Course Report", "wpep" ), ucfirst( $user_information->user_nicename ) )
    ] );

    echo '<div class="wpep-user-profile-container">';

    echo    '<div class="wpep-user-profile">';
    echo        get_avatar( $user_information->ID );
    echo        '<p class="wpep-user-profile-nicename">' . $user_information->user_nicename . '</p>';
    echo        '<p class="wpep-user-profile-email">' . $user_information->user_email . '</p>';

    if( wpep_has_sid_support() )
      echo        '<p class="wpep-user-profile-sid">' . get_user_meta( $user_information->ID, WPEP_USER_META_STUDENT_IDENTIFICATION, true ) . '</p>';

    echo    '</div>';

    echo     '<div class="wpep-user-course-history">';

    echo        '<h3>';
    echo          '<select id="wpep-course-navigation-helper" data-base-url="edit.php?post_type=' . WPEP_POST_TYPE_COURSE . '&page=wpep-statistics&tab=courses&user_id=' . $user_information->ID . '&course_id=">';

    foreach( $course_list as $course )
      echo  '<option value="' . $course->ID . '" ' . selected( $course_id, $course->ID, false ) . '>' . $course->post_title . '</option>';

    echo          '</select>';
    echo        '</h3>';
    echo        '<hr/>';

    $progress_percent = WPEP_Controller::instance()->userActivity->course_progress_percentage( $course_id, $user_id );

    echo        '<div class="wpep-progress-bar-container">';
    echo            '<p class="wpep-progress-bar-value">' . wpep_get_percentage( $progress_percent ). '</p>';
    echo            '<div class="wpep-progress-bar">';
    echo              '<div class="wpep-progress-bar-fill" role="progressbar" aria-valuenow="' . $progress_percent . '" aria-valuemin="0" aria-valuemax="100" style="width: ' . $progress_percent . '%"></div>';
    echo            '</div>';
    echo        '</div>';

    foreach( WPEP_Controller::instance()->userActivity->course_map( $course_id, $user_information->ID ) as $section_information ) {
      $user_section_learning_path = ( isset( $section_information->user_data->learning_path ) ? $section_information->user_data->learning_path : -1 );

      echo '<div class="section">';
      echo    '<h4>' . $section_information->title;
      echo      ( $user_section_learning_path != -1
        ? ( isset( $section_information->options->question_answers[$user_section_learning_path] ) ? ' ( ' . $section_information->options->question_answers[$user_section_learning_path] . ' ) ' : ' ( ' . __( "User went trough a learning path that has been deleted.", "wpep" ) . ' ) ' )
        : ''
      );
      echo    '</h4>';

      foreach( $section_information->lessons as $lesson ) {
        if( isset( $lesson->options->question_answer ) )
          if( $lesson->options->question_answer != -1 && $lesson->options->question_answer != $user_section_learning_path )
            continue;

        echo  '<p><input type="checkbox" disabled="disabled" ' . checked( $lesson->is_completed, true, false ). '/>' . $lesson->title . '</p>';
      }

      echo '</div>';
    }

    echo '<hr/>';

    echo '<h2>' . __( "Execution Tracking - order in which user executed the course.") . '</h2>';

    echo '<hr/>';

    echo '<table class="wp-list-table widefat fixed striped">';
    echo    '<thead>';
    echo      '<tr>';
    echo        '<th>' . __( "Date", "wpep") . '</th>';
    echo        '<th>' . __( "Section Name", "wpep") . '</th>';
    echo        '<th>' . __( "Lesson Name", "wpep") . '</th>';
    echo      '</tr>';
    echo    '</thead>';
    echo    '<tbody>';

    foreach( WPEP_Controller::instance()->userActivity->course_lesson_completion_map( $course_id, $user_information->ID ) as $lesson_information ) {
      echo '<tr>';
      echo    '<td>' . $lesson_information->date . '</td>';
      echo    '<td>' . $lesson_information->section_title . '</td>';
      echo    '<td>' . $lesson_information->lesson_title . '</td>';
      echo '</tr>';
    }

    echo    '</tbody>';
    echo '</table>';



    echo   '</div>';

    echo '</div>';

    echo '<script type="text/javascript">';
    echo    'jQuery(document).ready(function () { ';
    echo      'jQuery("#wpep-course-navigation-helper").bind( "change", function() {';
    echo        'window.location.replace( jQuery(this).attr("data-base-url") + jQuery(this).val() );';
    echo      '}); ';
    echo    '});';
    echo '</script>';
  }

}