<?php

namespace WPEP\Admin\Coordinator\Statistics;

use WPEP\Controller as WPEP_Controller;
use WP_User_Query;

class GAReport extends Skeleton {

  public $tabs_year   = [];
  public $tabs_month  = [];
  public $current_year  = 0;
  public $current_month = 0;

  public function get_name() {
    return __( 'GA Report', "wpep" );
  }

  public function get_alias() {
    return 'irs_report';
  }

  private function _setup_data() {
    $this->tabs_year = [];

    for( $i = date("Y"); $i >= WPEP_STATISTICS_YEAR_FROM; $i-- )
      $this->tabs_year[ $i ] = $i;

    $this->current_year = ( isset( $_GET['year'] ) ? $_GET['year'] : '' );

    if( !isset( $this->tabs_year[ $this->current_year ] ) )
      $this->current_year = key( $this->tabs_year );

    $this->tabs_month = array_reverse( wpep_get_month_list_within_year( $this->current_year ), true );

    $this->current_month = ( isset( $_GET['month'] ) ? $_GET['month'] : date('m') );

    if( !isset( $this->tabs_month[ $this->current_month ] ) )
      $this->current_month = key( $this->tabs_month );
  }

  public function display( $filter_by = [] ) {
    $this->_setup_data();

    $this->_display_tabs();

    $table_information = $this->_get_table_data( $filter_by );

    wpep_admin_display_table(
      __( "Student List", "wpep" ),
      $table_information['head'],
      $table_information['body'],
      true
    );
  }

  private function _display_tabs() {
    echo '<h3 class="nav-tab-wrapper">';

    foreach( $this->tabs_year  as $tab_year => $tab_name ){
      $class = ( $tab_year == $this->current_year ) ? 'nav-tab-active' : '';
      echo '<a class="nav-tab ' . $class . '" href="' . wpep_admin_url_statistics( [ 'tab' => $this->get_alias(), 'year' => $tab_year ] ) . '">' . $tab_name . '</a>';
    }

    echo '</h3>';

    echo '<h3 class="nav-tab-wrapper">';

    foreach( $this->tabs_month  as $tab_month => $tab_name ){
      $class = ( $tab_month == $this->current_month ) ? 'nav-tab-active' : '';
      echo '<a class="nav-tab ' . $class . '" 
               href="' . wpep_admin_url_statistics( [ 'tab' => $this->get_alias(), 'year' => $this->current_year, 'month' => $tab_month ] ) . '">' . $tab_name . '</a>';
    }

    echo '</h3>';
  }

  public function download( $filter_by = [] ) {
    $this->_setup_data();

    $table_information = $this->_get_table_data( $filter_by );

    do_action(
      'wpep_admin_download_table',
      'irs-report-' . ( isset( $filter_by['user_role'] ) && $filter_by['user_role'] != false ? $filter_by['user_role'] . '-' : '' ) .
           $this->current_year . ( $this->current_month != '' ? '-' . $this->current_month : '' ) . '-created-' . wpep_format_date(),
      $table_information['head'],
      $table_information['body']
    );
  }

  private function _get_table_data( $filter_by = [] ) {
    $table_information = [
      'head'          => [],
      'body'          => []
    ];

    $date_from  = $this->current_year . '-' . $this->current_month . '-' . '01';
    $date_to    = (
      $this->current_month == 12
      ? ( $this->current_year + 1 ) . '-01-01'
      : $this->current_year . '-' . ( $this->current_month + 1 ) . '-' . '01'
    );

    $completed_course_list = WPEP_Controller::instance()->courseActivity->get_period_completed_courses_list( $date_from, $date_to );

    if( empty( $completed_course_list ) )
      return $table_information;

    $table_information['head'] = [
      'first_name'     => __( "First Name", 'wpep' ),
      'last_name'      => __( "Last Name", 'wpep' ),
      'student_identifier' => __( "Student Identifier", 'wpep' ),
      'course_identifier'  => __( "Course Identifier", 'wpep' ),
      'completed_at'       => __( "Completed at", 'wpep' ),
      'email_address'  => __( "Email Address", 'wpep' ),
    ];

    foreach( $completed_course_list as $completed_course ) {
      $user = get_user_by( 'id', $completed_course->user_id );

      if( !wpep_user_filter_requirement_met( $user, $filter_by ) )
        continue;

      $table_information['body'][$user->ID] = [
        'first_name'        => $user->first_name,
        'last_name'         => $user->last_name,
        'student_identifier'=> get_user_meta( $user->ID, WPEP_USER_META_STUDENT_IDENTIFICATION, true ),
        'course_identifier' => get_post_meta( $completed_course->course_id, 'wpep_course_identifier', true ),
        'completed_at'      => wpep_format_date( $completed_course->completed_at ),
        'email_address'     => $user->user_email
      ];
    }

    return $table_information;
  }

  

}
