<?php

namespace WPEP\Admin\Coordinator\Statistics;

use WPEP\Controller as WPEP_Controller;
use WP_User_Query;

class UserNotifications extends Skeleton {

  public function get_name() {
    return __( 'Student Notifications', "wpep" );
  }

  public function get_alias() {
    return 'user-notifications';
  }

  public function download() {
    $table_information = $this->_get_table_data( 1000000, 1 );

    do_action(
      'wpep_admin_download_table',
      'student-timing-notifications-' . wpep_format_date(),
      $table_information['head'],
      $table_information['body']
    );
  }

  public function display( $filter_by = [] ) {
    $entries_per_page = 30;
    $current_page     = max( 1, ( isset( $_GET['pagination'] ) ? intval( $_GET['pagination'] ) : 1 ) );

    $table_information = $this->_get_table_data( $entries_per_page, $current_page );

    if( isset( $filter_by['user_role'] ) && $filter_by['user_role'] != false ) {
      echo '<br/>';
      echo '<div class="wpep-notification wpep-notification-info">' . __( "This Statistics Page cannot be limited to your User Role Filter, you'll see all entries.", "wpep" ). '</div>';
    }

    wpep_admin_display_table(
      __( "Timing Notify - Notifications", "wpep" ),
      $table_information['head'],
      $table_information['body'],
      true,
      $table_information['page_current'],
      $table_information['page_total']
    );
  }

  private function _get_table_data( $entries_per_page, $current_page ) {
    $table_information = [
      'head'          => [],
      'body'          => [],
      'page_current'  => null,
      'page_total'    => null
    ];

    $entries_query_information = WPEP_Controller::instance()->courseActivity->get_time_action_notify_map(
      $entries_per_page, ( $current_page - 1 ) * $entries_per_page, true
    );

    if( empty( $entries_query_information['entries'] ) )
      return $table_information;

    $table_information['head'] = [
      'user_name'              => __( "Student Name", 'wpep' ),
      'course_identification'  => __( "Course Identification", "wpep" ),
      'student_identification' => __( "SID", 'wpep' ),
      'course_title'           => __( "Course Title", 'wpep' ),
      'entity_title'           => __( "Title", 'wpep' ),
      'overdue'                => __( "Overdue", "wpep" )
    ];

    if( !wpep_has_sid_support() )
      unset( $table_information['head']['student_identification'] );

    foreach( $entries_query_information['entries'] as $entry ) {
      $user_information = get_user_by( 'ID', $entry->user_id );

      if( $user_information == null )
        continue;

      $course_title = get_the_title( $entry->post_id );

      $body = [
        'user_name' => [
          'type'      => 'link',
          'value'     => '<a href="' . wpep_admin_url_statistics( [ 'tab' => 'courses', 'course_id' => $entry->post_id, 'user_id' => $user_information->ID ] ) . '">' . $user_information->user_nicename . '</a>',
          'value_raw' => wpep_content_user_profile_format_name( $user_information )
        ],
        'course_title' => [
          'type'      => 'link',
          'value'     => '<a href="' . wpep_admin_url_statistics( [ 'tab' => 'courses', 'course_id' => $entry->post_id, 'user_id' => $user_information->ID ] ) . '">' . $course_title . '</a>',
          'value_raw' => $course_title
        ],
        'entity_title' => wpep_alias_to_label( $entry->type ) . ' : ' . $entry->title,
        'overdue'      => human_time_diff( time(), $entry->user_started_at + $entry->time_action_notify ) . __( " ago", "wpep" )
      ];

      if( wpep_has_sid_support() ) {
        $body['student_identification'] = get_user_meta( $user_information->ID, WPEP_USER_META_STUDENT_IDENTIFICATION, true );
        $body['course_identification']  = get_post_meta( $entry->post_id, 'wpep_course_identifier', true );
      }

      $table_information['body'][] = $body;
    }

    $table_information['page_current'] = $current_page;
    $table_information['page_total']   = ceil( $entries_query_information['entries_total'] / $entries_per_page );

    return $table_information;
  }

}
