<?php

namespace WPEP\Admin\Coordinator\Statistics;

use WPEP\Controller as WPEP_Controller;
use WP_User_Query;

class Users extends Skeleton {

  public function get_name() {
    return __( 'User List', "wpep" );
  }

  public function get_alias() {
    return 'users';
  }

  public function display( $filter_by = [] ) {
    $user_list_per_page = 30;
    $current_page       = max( 1, ( isset( $_GET['pagination'] ) ? intval( $_GET['pagination'] ) : 0 ) );

    $table_information = $this->_get_table_data(
      $user_list_per_page,
      $current_page,
      $filter_by
    );

    wpep_admin_display_table(
      __( "User List", "wpep" ),
      $table_information['head'],
      $table_information['body'],
      true,
      $table_information['page_current'],
      $table_information['page_total']
    );
  }

  public function download( $filter_by = [] ) {
    $table_information = $this->_get_table_data(
      1000000,
      1,
      $filter_by
    );

    do_action(
      'wpep_admin_download_table',
      'user-list-' . ( isset( $filter_by['user_role'] ) && $filter_by['user_role'] != false ? $filter_by['user_role'] . '-' : '' ) . wpep_format_date(),
      $table_information['head'],
      $table_information['body']
    );
  }

  private function _get_table_data( $user_list_per_page, $current_page, $filter_by = [] ) {
    $table_information = [
      'head'          => [],
      'body'          => [],
      'page_current'  => null,
      'page_total'    => null
    ];

    $user_list_query_args = [
      'number'    => $user_list_per_page,
      'paged'     => $current_page
    ];

    $user_list_query_args = wpep_user_filter_query_args( $user_list_query_args, $filter_by );

    $user_list_query = new WP_User_Query( $user_list_query_args );

    $user_list = $user_list_query->get_results();

    if( empty( $user_list ) )
      return $table_information;

    $table_information['head'] = [
      'name'                   => __( "Name", 'wpep' ),
      'student_identification' => __( "SID", 'wpep' ),
      'courses_started' => [
        'type'  => "digits",
        'value' =>  __( "Courses Started", 'wpep' )
      ],
      'courses_completed' => [
        'type'  => "digits",
        'value' =>  __( "Courses Completed", 'wpep' )
      ],
      'lessons_completed' => [
        'type'  => "digits",
        'value' =>  __( "Lessons Completed", 'wpep' )
      ],
      'assessments_pending_review' => [
        'type'  => "digits",
        'value' =>  __( "Assessments Pending Review", 'wpep' )
      ],
      'assessments_completed' => [
        'type'  => "digits",
        'value' =>  __( "Assessments Completed", 'wpep' )
      ],
      'assessments_failed' => [
        'type'  => "digits",
        'value' =>  __( "Assessments Failed", 'wpep' )
      ],
      'course_progress' => [
        'type'  => 'link-item',
        'style' => 'width:160px;text-align: center;',
        'value' => __( "Course Progress", "wpep" )
      ],
      'assessments_view' => [
        'type'  => 'link-item',
        'style' => 'width:160px;text-align: center;',
        'value' => __( "Assessments", "wpep" )
      ],
    ];

    if( !wpep_get_setting_flag( 'enable-course-assessments' ) )
      unset(
        $table_information['head']['assessments_pending_review'],
        $table_information['head']['assessments_completed'],
        $table_information['head']['assessments_failed'],
        $table_information['head']['assessments_view']
      );

    if( !wpep_has_sid_support() )
      unset( $table_information['head']['student_identification'] );

    foreach( $user_list as $user ) {
      $table_information['body'][$user->ID] = [
        'name'              => wpep_content_user_profile_format_name( $user ),
        'courses_started'   => [
          'type'  => 'digits',
          'value' => WPEP_Controller::instance()->userActivity->statistics->get_started_courses_count( $user->ID )
        ],
        'courses_completed' => [
          'type'  => 'digits',
          'value' => WPEP_Controller::instance()->userActivity->statistics->get_completed_course_count( $user->ID )
        ],
        'lessons_completed' => [
          'type'  => 'digits',
          'value' => WPEP_Controller::instance()->userActivity->statistics->get_completed_lesson_count( $user->ID )
        ],
        'course_progress' => [
          'type'  => 'link-item',
          'style' => 'width:120px;',
          'value' => '<a class="button button-primary" href="' . wpep_admin_url_statistics( [ 'tab' => 'courses', 'user_id' => $user->ID ] ) . '">' . __( "View", "wpep" ) . '</a>'
        ],
      ];

      if( wpep_has_sid_support() )
        $table_information['body'][$user->ID]['student_identification'] = get_user_meta( $user->ID, WPEP_USER_META_STUDENT_IDENTIFICATION, true );

      if( !wpep_get_setting_flag( 'enable-course-assessments' ) )
        continue;

      $assessment_data = WPEP_Controller::instance()->assessmentActivity->get_assessment_status_count( false, $user->ID );

      $table_information['body'][$user->ID] += [
        'assessments_pending_review' => [
          'type'  => "digits",
          'value' =>  $assessment_data[ WPEP_USER_ASSESSMENT_STATUS_PENDING_REVIEW ]
        ],
        'assessments_completed' => [
          'type'  => "digits",
          'value' =>  $assessment_data[ WPEP_USER_ASSESSMENT_STATUS_COMPLETED ]
        ],
        'assessments_failed' => [
          'type'  => "digits",
          'value' =>  $assessment_data[ WPEP_USER_ASSESSMENT_STATUS_FAILED ]
        ],
        'assessments_view' => [
          'type'  => 'link-item',
          'style' => 'width:120px;',
          'value' => '<a class="button button-primary" href="' . wpep_admin_url_statistics( [ 'tab' => 'assessments', 'user_id' => $user->ID ] ) . '">' . __( "View", "wpep" ) . '</a>'
        ]
      ];
    }

    $table_information['page_current'] = $current_page;
    $table_information['page_total']   = ceil( $user_list_query->get_total() / $user_list_per_page );

    return $table_information;
  }

  

}
