<?php

namespace WPEP;

/**
 * declare Controller class.
 *
 * @since        1.0
 */
class Controller {

  const SHOP_PRODUCT_ID = 35;

  protected static $instance;

  /**
   * @var Coordinator\AddOns
   */
  public $addOns;

  /**
   * @var Coordinator\Ajax
   */
  public $ajax;

  /**
   * @var Coordinator\Capabilities
   */
  public $capabilities;

  /**
   * @var Coordinator\ContentLibrary
   */
  public $contentLibrary;

  /**
   * @var Coordinator\Hooks
   */
  public $hooks;

  /**
   * @var Coordinator\Template
   */
  public $template;

  /**
   * @var Coordinator\TemplateStyle
   */
  public $templateStyle;

  /**
   * @var Coordinator\Assets
   */
  public $assets;

  /**
   * @var Coordinator\Email
   */
  public $email;

  /**
   * @var Coordinator\Migration
   */
  public $migration;

  /**
   * @var Coordinator\Notifications
   */
  public $notifications;

  /**
   * @var Coordinator\PlatformHealth
   */
  public $platformHealth;

  /**
   * @var Coordinator\PostComments
   */
  public $postComments;

  /**
   * @var Coordinator\Content
   */
  public $content;

  /**
   * @var Coordinator\PreAuth
   */
  public $preAuth;

  /**
   * @var Coordinator\Request
   */
  public $request;

  /**
   * @var Coordinator\Search
   */
  public $search;

  /**
   * @var Coordinator\ShortCode
   */
  public $shortCode;

  /**
   * @var Coordinator\UserActivity
   */
  public $userActivity;

  /**
   * @var Coordinator\AssessmentActivity
   */
  public $assessmentActivity;

  /**
   * @var Coordinator\CourseActivity
   */
  public $courseActivity;

  /**
   * @var Coordinator\UserEngagement
   */
  public $userEngagement;

  /**
   * @var Admin\Controller
   */
  public $adminController;

  public static function instance() {
    if (!isset(self::$instance))
      self::$instance = new self();

    return self::$instance;

  }

  public function __construct() {
    require_once(WPEP_BASE_PATH . "/app/_functions/general.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/utilities.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/administration.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/auth.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/assessments.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/course.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/post-comments.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/post.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/primary.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/user.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/user-profile.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/search.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/shortcode-builder.php" );
  }

  public function setup() {
    $this->register_coordinators();

    // Migration Process need to be early.
    $this->migration->setup();

    do_action( "wpep_migration_setup" );

    add_action( 'plugins_loaded', [ $this, '_setup' ] );
  }

  public function _setup() {
    $this->setup_coordinators();

    do_action( 'wpep_register_addon', $this->addOns );

    do_action( 'wpep_init_addon_before', $this->addOns );

    $this->addOns->_init_add_ons();

    do_action( 'wpep_init_addon_after', $this->addOns );

    do_action( 'wpep_loaded' );

    $response = load_plugin_textdomain( 'wpep', false, basename( WPEP_BASE_PATH ) . '/lib/languages/' );
  }

  public function register_coordinators() {
    $this->migration      = new Coordinator\Migration();
    $this->notifications  = new Coordinator\Notifications();
    $this->platformHealth = new Coordinator\PlatformHealth();
    $this->postComments   = new Coordinator\PostComments();

    $this->addOns    = new Coordinator\AddOns();

    $this->capabilities   = new Coordinator\Capabilities();
    $this->contentLibrary = new Coordinator\ContentLibrary();
    $this->ajax           = new Coordinator\Ajax();
    $this->hooks          = new Coordinator\Hooks();
    $this->template       = new Coordinator\Template();
    $this->templateStyle  = new Coordinator\TemplateStyle();
    $this->assets         = new Coordinator\Assets();
    $this->content        = new Coordinator\Content();
    $this->request        = new Coordinator\Request();
    $this->preAuth        = new Coordinator\PreAuth();
    $this->email          = new Coordinator\Email();
    $this->search         = new Coordinator\Search();
    $this->shortCode      = new Coordinator\ShortCode();
    $this->assessmentActivity = new Coordinator\AssessmentActivity();
    $this->userActivity   = new Coordinator\UserActivity();
    $this->userEngagement = new Coordinator\UserEngagement();
    $this->courseActivity = new Coordinator\CourseActivity();

    $this->adminController   = new Admin\Controller();
  }

  public function setup_coordinators() {
    $this->notifications->setup();
    $this->postComments->setup();
    $this->ajax->setup();
    $this->hooks->setup();
    $this->template->setup();
    $this->templateStyle->setup();
    $this->assets->setup();
    $this->content->setup();
    $this->preAuth->setup();
    $this->shortCode->setup();
    $this->assessmentActivity->setup();
    $this->userActivity->setup();
    $this->courseActivity->setup();

    $this->adminController->dispatch();
  }

}
