<?php
namespace WPEP\Coordinator\Ajax;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;

class Admin {

  public function __construct() {
    add_action('wp_ajax_wpep_verify_license', [ $this, 'verify_license_key' ] );
    add_action('wp_ajax_wpep_admin_save_post', [ $this, 'wpep_admin_save_post_ajax_coordinator' ] );
    add_action('wp_ajax_wpep_admin_user_icon_list', [ $this, 'user_icon_list' ] );
    add_action('wp_ajax_wpep_admin_walk_trough_end', [ $this, 'walk_trough_end'] );

    add_action('wp_ajax_wpep_admin_plugin_get_plugin_status', [ $this, 'get_plugin_status' ] );
    add_action('wp_ajax_wpep_admin_plugin_action_install', [ $this, 'install_store_product' ] );
    add_action('wp_ajax_wpep_admin_plugin_action_activate', [ $this, 'activate_plugin' ] );
    add_action('wp_ajax_wpep_admin_plugin_action_deactivate', [ $this, 'deactivate_plugin' ] );

    if( isset( $_GET['wpep-editor-display'] ) )
      add_action( 'admin_init', [ $this, 'wpep_admin_editor'], 100 );
  }

  public function wpep_admin_save_post_ajax_coordinator() {
    if( !current_user_can( WPEP_CAP_EDIT_COURSE )
        && !current_user_can( WPEP_CAP_EDIT_ASSESSMENT ) )
      exit( json_encode( [
          'status'  => 'error',
          'error'   => __( "Current user cannot manage_options.", "wpep" )
      ] ) );

    if( !isset( $_POST['post_id'] ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "Post ID has not been provided.", "wpep" )
      ] ) );

    $post_id = intval( $_POST['post_id'] );

    $entity_id_map = ( isset( $_POST['wpep_entity_id_map'] ) ? $_POST['wpep_entity_id_map'] : [] );
    $entity_id_map = apply_filters( 'wpep_admin_save_post_ajax', $entity_id_map, $post_id );

    exit( json_encode( [
      'status'          => 'ok',
      'entity_id_map'   => $entity_id_map
    ] ) );
  }

  public function wpep_admin_editor() {
    $content = ( isset( $_POST['content'] ) ? stripslashes( $_POST['content'] ) : '' );

    if( $content == ''
        && (
        current_user_can( WPEP_CAP_EDIT_COURSE )
        || current_user_can( WPEP_CAP_EDIT_ASSESSMENT )
        || current_user_can( WPEP_CAP_MANAGE_SETTINGS )
      ) ) {
      global $wpdb;

      if( isset( $_POST['assessment_id'] ) ) {
        $assessment_type = ( isset( $_POST['assessment_type'] ) ? sanitize_title( $_POST['assessment_type'] ) : 'success' );

        $content = ( $assessment_type == 'success'
          ? get_post_meta( intval( $_POST['assessment_id'] ), 'wpep_success_message', true )
          : get_post_meta( intval( $_POST['assessment_id'] ), 'wpep_failed_message', true )
        );

        if( $content == '' && $assessment_type == 'success' )
          $content = wpep_get_setting_translated( 'assessment-success-message' );

        if( $content == '' && $assessment_type == 'failed' )
          $content = wpep_get_setting_translated( 'assessment-status-failed' );

      } else if( isset( $_POST['lesson_id'] ) ) {
        $content = $wpdb->get_col('SELECT `content` FROM ' . $wpdb->prefix . 'wpep_section_lesson WHERE id = ' . intval( $_POST['lesson_id'] ) );
      } else if( isset( $_POST['section_id'] ) ) {
        $content = $wpdb->get_col('SELECT `content` FROM ' . $wpdb->prefix . 'wpep_section WHERE id = ' . intval( $_POST['section_id'] ) );
      }

      if( is_array( $content ) )
        $content = array_shift( $content );

      if( !is_string( $content ) )
        $content = '';
    }

    $editor_args = [];

    if( isset( $_POST['editor_class'] ) )
      $editor_args['editor_class'] = sanitize_text_field( $_POST['editor_class'] );

    wpep_editor( $content, $_POST['id'], $_POST['name'], $editor_args );

    exit;
  }

  public function verify_license_key() {

    $license = trim($_POST['license-key']);
    $nonce   = $_POST['nonce'];

    if (!wp_verify_nonce($nonce, 'wpep_license_activate_nonce'))
      return;

    $is_valid_license = WPEP_Controller::instance()->adminController->license->activate_license( $license );

    if( $is_valid_license ) {
      echo '<div class="updated"><p>' . __( "Congratulations! You Successfully validated your WPEP Plugin.", "wpep" ) . '</p></div>';
      
      echo '<p>';
      echo    '<a href="' . admin_url('edit.php?post_type=' . WPEP_POST_TYPE_COURSE . '&page=wpep-settings') . '" class="button button-primary">' . __( "Manage Settings", "wpep" ) . '</a> ';
      echo    '<a href="' . admin_url('edit.php?post_type=' . WPEP_POST_TYPE_COURSE) . '" class="button button-primary">' . __( "Manage Courses", "wpep" ) . '</a> ';
      echo    '<a href="' . admin_url('post-new.php?post_type=' . WPEP_POST_TYPE_COURSE) . '" class="button button-primary">' . __( "Create Course", "wpep" ) . '</a>';
      echo '</p>';
    } else {
      echo '<div class="error"><p>' . __( "Warning! Your license key is not valid. Please contact our support.", "wpep" ) . '</p></div>';
    }

    exit;
  }

  public function user_icon_list() {
    echo json_encode( wpep_get_admin_icon_selection_list() );
    exit;
  }

  public function install_store_product() {
    $store_product_id = isset( $_POST['store_product_id'] ) ? $_POST['store_product_id'] : false;

    if ( $store_product_id == false
        || ! isset( $_POST['nonce'] )
        || ! wp_verify_nonce( $_POST['nonce'], 'wpep-install-plugin_' . $store_product_id )
        || !current_user_can( 'install_plugins'))
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "An unexpected error has happened, please refresh the page and try again", "wpep" )
      ] ) );

    $addon_list = wpep_controller()->adminController->addOnsSection->get_available_addon_list();

    if( !isset( $addon_list[ $store_product_id ] ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "The provided store_id doesn't match any addon.", "wpep" )
      ] ) );

    if( !in_array( $store_product_id, wpep_controller()->adminController->license->get_license_bundled_products() ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "The Bundle License cannot be used to download this addon / plugin, please download it from the codestore.", "wpep" )
      ] ) );

    $addon_information = $addon_list[ $store_product_id ];

    unset( $addon_list ); // Cleanup Memory

    set_current_screen();

    $license_and_updates = new \WPEP_Integration_License_And_Updates(
      $addon_information->title,
      $addon_information->id,
      false,
      '0.0.0',
      false
    );

    $license_data = $license_and_updates->api->activate_license();

    if( !isset( $license_data->license ) || $license_data->license != 'valid' )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "The Bundle License cannot be validated, please download the plugin from the My Account section.", "wpep" )
      ] ) );

    $request_params = $license_and_updates->api->get_remote_request_params();
    $request_params['body'] = $license_and_updates->api->get_request_params( 'get_version' );

    $request = wp_remote_post( WPEP_STORE_LINK, $request_params );

    if ( is_wp_error( $request ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "Plugin Information cannot be fetched from server, please download the plugin from the My Account section.", "wpep" )
      ] ) );

    $request = json_decode( wp_remote_retrieve_body( $request ) );

    if( !isset( $request->package ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "Plugin Package cannot be fetched from server, please download the plugin from the My Account section.", "wpep" )
      ] ) );

    if( !class_exists( '\Plugin_Upgrader' ) )
      require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';

    if( !class_exists( '\WPEP_Silent_Upgrader_Skin') )
      require_once WPEP_BASE_PATH . '/lib/_internal/Silent_Upgrader.php';

    try {
      ob_start();
      $installer = new \Plugin_Upgrader( new \WPEP_Silent_Upgrader_Skin() );
      $installation_response = $installer->install( $request->package );
      ob_end_clean();
      if( is_wp_error( $installation_response ) )
        exit( json_encode( [
          'status'  => 'error',
          'error'   => $installation_response->get_error_message()
        ] ) );

      wp_cache_flush();
    } catch ( \Exception $exception ) {
      exit( json_encode( [
        'status'  => 'error',
        'error'   => $exception->getMessage()
      ] ) );
    }

    // Flush the cache and return the newly installed plugin basename.
    if ( !$installer->plugin_info() )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "The installation has failed, please install the addon manually as we cannot do it for you at the moment.", "wpep" )
      ] ) );

    exit( json_encode( [
      'status'              => 'ok'
    ] ) );
  }

  public function get_plugin_status() {
    if ( ! isset( $_POST[ 'plugin_alias' ] ) || !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "An unexpected error has happened, please refresh the page and try again", "wpep" )
      ] ) );

    $addon = wpep_controller()->addOns->get_add_on( $_POST[ 'plugin_alias' ] );

    $response = [];

    $response['available_operation_html'] = wpep_controller()->adminController->addOnsSection->get_addon_status_operations(
      sanitize_text_field( $_POST[ 'plugin_alias' ] ),
      $addon
    );

    $response['status_html'] = wpep_controller()->adminController->addOnsSection->get_addon_status_text( $addon );

    exit( json_encode( $response ) );
  }

  public function activate_plugin() {
    $plugin_identifier = isset( $_POST['plugin'] ) ? $_POST['plugin'] : false;

    if ( $plugin_identifier == false
          || ! isset( $_POST['nonce'] )
          || ! wp_verify_nonce( $_POST['nonce'], 'activate-plugin_' . $plugin_identifier )
          || !current_user_can( 'activate_plugins') )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "An unexpected error has happened, please refresh the page and try again", "wpep" )
      ] ) );

    $activate = activate_plugin( $plugin_identifier );

    if ( is_wp_error( $activate ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => $activate->get_error_message()
      ] ) );

    exit( json_encode( [
      'status'              => 'ok'
    ] ) );
  }

  public function deactivate_plugin() {
    $plugin_identifier = isset( $_POST['plugin'] ) ? $_POST['plugin'] : false;

    if ( $plugin_identifier == false
        || ! isset( $_POST['nonce'] )
        || ! wp_verify_nonce( $_POST['nonce'], 'deactivate-plugin_' . $plugin_identifier )
        || !current_user_can( 'activate_plugins') )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "An unexpected error has happened, please refresh the page and try again", "wpep" )
      ] ) );

    $deactivate = deactivate_plugins( $plugin_identifier );

    if ( is_wp_error( $deactivate ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => $deactivate->get_error_message()
      ] ) );

    exit( json_encode( [
      'status'              => 'ok'
    ] ) );
  }

  public function walk_trough_end() {
    if( is_user_logged_in() )
      delete_user_meta( get_current_user_id(), WPEP_Controller::instance()->adminController->learningCenter->walkTrough->user_meta_identifier );

    exit( json_encode( [
      'status'              => 'ok'
    ] ) );
  }

}