<?php
namespace WPEP\Coordinator\Ajax;

class UserProfile {

  public function __construct() {
    add_action('wp_ajax_wpep_update_user_profile', [ $this, 'update_user_profile' ] );

    add_action('wp_ajax_wpep_update_user_profile_cover_picture', [ $this, 'update_user_profile_cover_picture' ] );
    add_action('wp_ajax_wpep_update_user_profile_picture', [ $this, 'update_user_profile_picture' ] );
  }

  public function update_user_profile() {
    if( !isset( $_POST['information'] ) || !isset( $_POST['information'] ) )
      exit ( json_encode( [
        'status'  => 'error'
      ] ) );

    $information = $_POST['information'];

    $user_id = ( isset( $information['wpep_user_profile_id'] ) ? intval( $information['wpep_user_profile_id'] ) : get_current_user_id() );

    if( $user_id != get_current_user_id() && !current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      exit ( json_encode( [
        'status'  => 'error'
      ] ) );

    $user = get_user_by( 'ID', $user_id );

    if( $user === false )
      exit ( json_encode( [
        'status'  => 'error'
      ] ) );

    $wp_user_update = [];

    if( isset( $information['wpep_user_url'] ) && $information['wpep_user_url'] != $user->user_url ) {
      $user->user_url = sanitize_text_field( $information['wpep_user_url'] );

      $wp_user_update['user_url'] = $user->user_url;
    }

    if( isset( $information['wpep_user_first_name'] ) && $information['wpep_user_first_name'] != $user->first_name ) {
      $user->first_name = sanitize_text_field( $information['wpep_user_first_name'] );

      $wp_user_update['first_name'] = $user->first_name;
    }

    if( isset( $information['wpep_user_last_name'] ) && $information['wpep_user_last_name'] != $user->last_name ) {
      $user->last_name = sanitize_text_field( $information['wpep_user_last_name'] );

      $wp_user_update['last_name'] = $user->last_name;
    }

    if( isset( $information['wpep_user_description'] ) && $information['wpep_user_description'] != $user->description ) {
      $user->description = sanitize_textarea_field( $information['wpep_user_description'] );

      $wp_user_update['description'] = $user->description;
    }

    if( isset( $information['wpep_user_student_identification'] ) && $information['wpep_user_student_identification'] != get_user_meta( $user_id, WPEP_USER_META_STUDENT_IDENTIFICATION, true ) ) {
      update_user_meta( $user_id, WPEP_USER_META_STUDENT_IDENTIFICATION, sanitize_text_field( $information['wpep_user_student_identification'] ) );
    }

    if( isset( $information['wpep_user_location'] ) && $information['wpep_user_location'] != get_user_meta( $user_id, WPEP_USER_LOCATION_META, true ) ) {
      update_user_meta( $user_id, WPEP_USER_LOCATION_META, sanitize_text_field( $information['wpep_user_location'] ) );
    }

    if( isset( $information['wpep_user_social_network'] ) ) {
      $current_social_networks = get_user_meta( $user_id, WPEP_USER_META_SOCIAL_NETWORK_PROFILES, true );

      if( !is_array( $current_social_networks ) )
        $current_social_networks = [];

      foreach( wpep_user_profile_social_network_platform_list_active() as $social_network_key => $social_network_information ) {
        if( !isset( $information['wpep_user_social_network'][$social_network_key] ) )
          continue;

        $current_val = trim( $information['wpep_user_social_network'][$social_network_key] );

        if( $current_val === '' ) {
          unset( $current_social_networks[$social_network_key] );
          continue;
        }

        $current_social_networks[$social_network_key] = (
          isset( $social_network_information['type'] ) && $social_network_information['type'] == 'link'
            ? esc_url( $current_val )
            : sanitize_text_field( $current_val )
        );
      }

      update_user_meta( $user_id, WPEP_USER_META_SOCIAL_NETWORK_PROFILES, $current_social_networks );
    }

    if( isset( $information['wpep_user_weekly_recap'] ) && $information['wpep_user_weekly_recap'] != get_user_meta( $user_id, 'wpep_disable_email_weekly_recap', true ) ) {
      update_user_meta( $user_id, 'wpep_disable_email_weekly_recap', intval( $information['wpep_user_weekly_recap'] ) );
    }

    do_action( 'wpep_user_profile_information_save', $information, $user );

    if( !empty( $wp_user_update ) )
      wp_update_user( [ 'ID' => $user_id ] + $wp_user_update );

    exit ( json_encode( [
      'status'        => 'ok',
      'display_name'  => wpep_content_user_profile_format_name( $user ),
      'description'   => wpautop( make_clickable( $user->description ) )
    ] ) );
  }

  public function update_user_profile_cover_picture() {
    if( !isset( $_POST['information'] ) || !isset( $_POST['information'] ) )
      exit ( json_encode( [
        'status'  => 'error'
      ] ) );

    $information = $_POST['information'];

    $user_id = ( isset( $information['wpep_user_id'] ) ? intval( $information['wpep_user_id'] ) : 0 );

    if( $user_id == false )
      $user_id = ( isset( $information['user_id'] ) ? intval( $information['user_id'] ) : get_current_user_id() );

    if( $user_id != get_current_user_id() && !current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      exit ( json_encode( [
        'status'  => 'error'
      ] ) );

    $user = get_user_by( 'ID', $user_id );

    if( $user === false )
      exit ( json_encode( [
        'status'  => 'error'
      ] ) );

    if( wpep_link_is_404( $information['picture'] ) )
      exit ( json_encode( [
        'status'  => 'error'
      ] ) );

    update_user_meta( $user_id, WPEP_USER_META_COVER_PICTURE, $information['picture'] );

    exit ( json_encode( [
      'status'        => 'ok'
    ] ) );
  }

  public function update_user_profile_picture() {
    if( !isset( $_POST['information'] ) || !isset( $_POST['information'] ) )
      exit ( json_encode( [
        'status'  => 'error'
      ] ) );

    $information = $_POST['information'];

    $user_id = ( isset( $information['wpep_user_id'] ) ? intval( $information['wpep_user_id'] ) : 0 );

    if( $user_id == false )
      $user_id = ( isset( $information['user_id'] ) ? intval( $information['user_id'] ) : get_current_user_id() );

    if( $user_id != get_current_user_id() && !current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      exit ( json_encode( [
        'status'  => 'error'
      ] ) );

    $user = get_user_by( 'ID', $user_id );

    if( $user === false )
      exit ( json_encode( [
        'status'  => 'error'
      ] ) );

    if( wpep_link_is_404( $information['picture'] ) )
      exit ( json_encode( [
        'status'  => 'error'
      ] ) );

    update_user_meta( $user_id, WPEP_USER_META_PROFILE_PICTURE, $information['picture'] );

    exit ( json_encode( [
      'status'        => 'ok'
    ] ) );
  }

}