<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;

class Assets {

  public $has_enqueue_media_support = false;

  public function setup() {
    add_action( 'wp_enqueue_scripts', [ $this, 'frontend_assets' ], 5 );
  }

  public function frontend_assets() {
    $current_theme_information = WPEP_Controller::instance()->templateStyle->get_current_theme();
    $custom_style_file_dependency = [ 'wpep-front-style' ];

    $front_css_file       = apply_filters('wpep_frontend_style_file', $current_theme_information['style_file'] );
    $front_css_expiration = apply_filters('wpep_frontend_style_tag', 'primary-' . WPEP_VERSION);

    wp_enqueue_style('wpep-front-style', $front_css_file, [ 'dashicons' ], $front_css_expiration);

    if( wpep_is_rtl() ) {
      $front_css_rtl_file       = apply_filters( 'wpep_frontend_style_rtl_file', $current_theme_information['style_rtl_file'] );
      $front_css_rtl_expiration = apply_filters( 'wpep_frontend_style_rtl_tag', 'primary-rtl-' . WPEP_VERSION );

      wp_enqueue_style('wpep-front-rtl-style', $front_css_rtl_file, [ 'wpep-front-style' ], $front_css_rtl_expiration);

      $custom_style_file_dependency[] = 'wpep-front-rtl-style';
    }

    $primary_script_dependencies = [ 'jquery' ];
    $primary_script_variables    = [
      'post_comments_refresh_ms'  => intval( wpep_get_setting( 'comments-refresh-ms-interval' ) ) * 1000,
      'ajax_url'                  => admin_url( 'admin-ajax.php' ),
      'resize_wistia_videos'      => wpep_get_setting( 'course-resize-wistia-embeds' )
    ];

    if( is_single() && get_post_type() == WPEP_POST_TYPE_ASSESSMENT )
      $this->has_enqueue_media_support = true;

    if( wpep_has_engagement_embed_support() ) {
      wp_enqueue_script('clipboard', WPEP_BASE_URL_PATH . 'assets/clipboard.min.js', [], '1.5.16' , true);
      $primary_script_dependencies[] = 'clipboard';

      $primary_script_variables['external_embed_helper_script'] = WPEP_BASE_URL_PATH . 'assets/front/embed-support-min.js';
    }

    if( $this->has_enqueue_media_support )
      $this->enqueue_media_support();

    /* enqueue Script. */
    wp_register_script( 'wpep-front-primary', WPEP_BASE_URL_PATH . 'assets/front/primary-min.js', $primary_script_dependencies, WPEP_VERSION, true);
    wp_localize_script( 'wpep-front-primary', 'wpep', $primary_script_variables );

    wp_enqueue_script( 'wpep-front-primary');

    $custom_style_file = ( wpep_get_setting_flag( 'wpep-custom-css-by-file' ) ? wpep_get_custom_style_file() : false );

    if( $custom_style_file != false ) {
      wp_enqueue_style(
        'wpep-front-custom-style',
        $custom_style_file,
        $custom_style_file_dependency,
        filemtime( wpep_get_custom_style_file( 'basedir' ) )
      );

      WPEP_Controller::instance()->templateStyle->custom_style_delivered = true;
    }
  }

  public function enqueue_media_support() {
    if ( did_action( 'wp_enqueue_media' ) )
      return;

    wp_enqueue_media();
    wp_enqueue_script( 'media-grid', "", [], true );
    wp_enqueue_script( 'media', "", [], true );
    wp_enqueue_style( 'media' );
    wp_enqueue_script( 'media-upload', "", [], true );
    wp_enqueue_script( 'thickbox', "", [], true );
  }

}