<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;

class Capabilities {

  public function get_role_list() {
    $role_list_capabilities_map = $this->get_role_list_capabilities_map();

    if( !is_array( $role_list_capabilities_map ) )
      return [];

    $response = [];

    foreach( array_keys( $role_list_capabilities_map ) as $role_key )
      $response[ $role_key ] = ( get_role( $role_key ) !== NULL ) ? 'ok' : 'error';

    return $response;
  }

  public function get_role_list_capabilities_map() {
    $role_list_capabilities_map = [
      WPEP_USER_ROLE_AUDIENCE_MANAGER => [
        'core'  => [
          WPEP_CAP_MANAGE_STATISTICS,
          WPEP_CAP_MANAGE_MEMBERS,
          WPEP_CAP_LEARNING_CENTER
        ]
      ],
      WPEP_USER_ROLE_INDIVIDUAL_CONTENT_MANAGER => [
        'core'  => [
          WPEP_CAP_LEARNING_CENTER
        ],
        WPEP_POST_CAP_COURSE      => $this->get_individual_post_type_manager_capabilities( WPEP_POST_CAP_COURSE ),
        WPEP_POST_CAP_VIDEO       => $this->get_individual_post_type_manager_capabilities( WPEP_POST_CAP_VIDEO ),
        WPEP_POST_CAP_E_BOOK      => $this->get_individual_post_type_manager_capabilities( WPEP_POST_CAP_E_BOOK ),
        WPEP_POST_CAP_OFFER       => $this->get_individual_post_type_manager_capabilities( WPEP_POST_CAP_OFFER ),
        WPEP_POST_CAP_ASSESSMENT  => $this->get_individual_post_type_manager_capabilities( WPEP_POST_CAP_ASSESSMENT ),
      ],
      WPEP_USER_ROLE_INDIVIDUAL_COURSES_MANAGER => [
        'core'  => [
          WPEP_CAP_LEARNING_CENTER
        ],
        WPEP_POST_CAP_COURSE      => $this->get_individual_post_type_manager_capabilities( WPEP_POST_CAP_COURSE ),
        WPEP_POST_CAP_ASSESSMENT  => $this->get_individual_post_type_manager_capabilities( WPEP_POST_CAP_ASSESSMENT ),
      ],
      WPEP_USER_ROLE_CONTENT_MANAGER  => [
        'core'  => [
          WPEP_CAP_LEARNING_CENTER
        ],
        WPEP_POST_CAP_COURSE      => true,
        WPEP_POST_CAP_VIDEO       => true,
        WPEP_POST_CAP_E_BOOK      => true,
        WPEP_POST_CAP_OFFER       => true,
        WPEP_POST_CAP_ASSESSMENT  => true
      ],
      WPEP_USER_ROLE_PLATFORM_MANAGER => true,
      'administrator'                 => true
    ];

    $role_list_capabilities_map = apply_filters( 'wpep_assign_user_role_capabilities_map', $role_list_capabilities_map );

    return $role_list_capabilities_map;
  }

  public function register_caps_and_roles() {
    global $wp_roles;

    if ( ! class_exists( '\WP_Roles' ) ) {
      return false;
    }

    if ( ! isset( $wp_roles ) ) {
      $wp_roles = new \WP_Roles();
    }

    $this->_remove_roles();
    $this->_add_base_roles_and_capabilities();

    return true;
  }

  private function _remove_roles() {
    remove_role( WPEP_USER_ROLE_INDIVIDUAL_CONTENT_MANAGER );
    remove_role( WPEP_USER_ROLE_INDIVIDUAL_COURSES_MANAGER );

    remove_role( WPEP_USER_ROLE_AUDIENCE_MANAGER );
    remove_role( WPEP_USER_ROLE_CONTENT_MANAGER );
    remove_role( WPEP_USER_ROLE_PLATFORM_MANAGER );
  }

  public function get_individual_post_type_manager_capabilities( $capability_type ) {
    return [
      "view_{$capability_type}",
      "edit_{$capability_type}",
      "read_{$capability_type}",
      "delete_{$capability_type}",
      "edit_{$capability_type}s",
      "publish_{$capability_type}s",
      "delete_{$capability_type}s",
      "delete_private_{$capability_type}s",
      "delete_published_{$capability_type}s",
      "edit_private_{$capability_type}s",
      "edit_published_{$capability_type}s",
      "assign_{$capability_type}_terms",
    ];
  }

  private function _add_base_roles_and_capabilities() {
    do_action( 'wpep_add_user_roles_before' );

    $this->_add_roles_and_capabilities_in_system();
    $this->_add_custom_caps_to_roles();

    do_action( 'wpep_add_user_roles_after' );
  }

  public function _add_custom_caps_to_roles() {
    global $wp_roles;

    $capabilities_grouped_list  = $this->get_core_capabilities();
    $role_list_capabilities_map = $this->get_role_list_capabilities_map();

    foreach( $role_list_capabilities_map as $role_key => $role_capabilities_map ) {
      $role_capabilities = [];

      if( $role_capabilities_map === true ) {
        foreach( $capabilities_grouped_list as $capabilities_grouped => $capabilities_group_attributes ) {
          foreach( $capabilities_group_attributes as $capabilities_group_attribute )
            $role_capabilities[] = $capabilities_group_attribute;
        }
      } else {
        foreach( $role_capabilities_map as $capability_group => $capability_options ) {
          if( $capability_options === true && isset( $capabilities_grouped_list[$capability_group] ) ) {

            foreach( $capabilities_grouped_list[$capability_group] as $capability_option )
              $role_capabilities[] = $capability_option;

          } elseif( is_array( $capability_options ) ) {
            foreach( $capability_options as $capability_option )
              $role_capabilities[] = $capability_option;
          }
        }
      }

      foreach( $role_capabilities as $role_capability )
        $wp_roles->add_cap( $role_key, $role_capability );
    }
  }

  /**
   * @return array
   */
  private function get_core_capabilities() {
    $capabilities = [
      'core' => [
        WPEP_CAP_MANAGE_MEMBERS,
        WPEP_CAP_MANAGE_SETTINGS,
        WPEP_CAP_MANAGE_STATISTICS,
        WPEP_CAP_LEARNING_CENTER
      ]
    ];

    $capability_types = [
      WPEP_POST_CAP_COURSE,
      WPEP_POST_CAP_VIDEO,
      WPEP_POST_CAP_E_BOOK,
      WPEP_POST_CAP_OFFER,
      WPEP_POST_CAP_ASSESSMENT
    ];

    foreach ( $capability_types as $capability_type ) {

      $capabilities[ $capability_type ] = [
        // Post type
        "edit_{$capability_type}",
        "read_{$capability_type}",
        "delete_{$capability_type}",
        "edit_{$capability_type}s",
        "edit_others_{$capability_type}s",
        "publish_{$capability_type}s",
        "read_private_{$capability_type}s",
        "delete_{$capability_type}s",
        "delete_private_{$capability_type}s",
        "delete_published_{$capability_type}s",
        "delete_others_{$capability_type}s",
        "edit_private_{$capability_type}s",
        "edit_published_{$capability_type}s",

        // Terms
        "manage_{$capability_type}_terms",
        "edit_{$capability_type}_terms",
        "delete_{$capability_type}_terms",
        "assign_{$capability_type}_terms",
      ];
    }

    return $capabilities;
  }

  private function _add_roles_and_capabilities_in_system() {
    add_role( WPEP_USER_ROLE_INDIVIDUAL_COURSES_MANAGER, __( 'WPEP Individual Courses Manager', 'wpep' ), [
      'read'                   => true,
      'unfiltered_html'        => false,
      'upload_files'           => true,
    ] );
    add_role( WPEP_USER_ROLE_INDIVIDUAL_CONTENT_MANAGER, __( 'WPEP Individual Content Manager', 'wpep' ), [
      'read'                   => true,
      'unfiltered_html'        => false,
      'upload_files'           => true,
    ] );

    add_role( WPEP_USER_ROLE_AUDIENCE_MANAGER, __( 'WPEP Audience Manager', 'wpep' ), [
      'read'                   => true,
      'read_private_pages'     => true,
      'read_private_posts'     => true,
      'moderate_comments'      => true,
      'upload_files'           => true,
    ] );

    add_role( WPEP_USER_ROLE_CONTENT_MANAGER, __( 'WPEP Content Manager', 'wpep' ), [
      'read'                   => true,
      'read_private_pages'     => true,
      'read_private_posts'     => true,
      'edit_posts'             => true,
      'edit_pages'             => true,
      'edit_published_posts'   => true,
      'edit_published_pages'   => true,
      'edit_private_pages'     => true,
      'edit_private_posts'     => true,
      'edit_others_posts'      => true,
      'edit_others_pages'      => true,
      'publish_posts'          => true,
      'publish_pages'          => true,
      'delete_posts'           => true,
      'delete_pages'           => true,
      'delete_private_pages'   => true,
      'delete_private_posts'   => true,
      'delete_published_pages' => true,
      'delete_published_posts' => true,
      'delete_others_posts'    => true,
      'delete_others_pages'    => true,
      'manage_categories'      => true,
      'manage_links'           => true,
      'moderate_comments'      => true,
      'unfiltered_html'        => true,
      'upload_files'           => true,
    ] );

    add_role( WPEP_USER_ROLE_PLATFORM_MANAGER, __( 'WPEP Platform Manager', 'wpep' ), [
      'level_9'                => true,
      'level_8'                => true,
      'level_7'                => true,
      'level_6'                => true,
      'level_5'                => true,
      'level_4'                => true,
      'level_3'                => true,
      'level_2'                => true,
      'level_1'                => true,
      'level_0'                => true,
      'read'                   => true,
      'read_private_pages'     => true,
      'read_private_posts'     => true,
      'edit_users'             => true,
      'edit_posts'             => true,
      'edit_pages'             => true,
      'edit_published_posts'   => true,
      'edit_published_pages'   => true,
      'edit_private_pages'     => true,
      'edit_private_posts'     => true,
      'edit_others_posts'      => true,
      'edit_others_pages'      => true,
      'publish_posts'          => true,
      'publish_pages'          => true,
      'delete_posts'           => true,
      'delete_pages'           => true,
      'delete_private_pages'   => true,
      'delete_private_posts'   => true,
      'delete_published_pages' => true,
      'delete_published_posts' => true,
      'delete_others_posts'    => true,
      'delete_others_pages'    => true,
      'manage_categories'      => true,
      'manage_links'           => true,
      'moderate_comments'      => true,
      'unfiltered_html'        => true,
      'upload_files'           => true,
      'export'                 => true,
      'import'                 => true,
      'list_users'             => true,
    ] );
  }

}