<?php

namespace WPEP\Coordinator\Content\Single;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings   as WPEP_Settings;
use WPEP\Entity\Course as WPEP_Entity_Course;

class Basic extends Skeleton {

  public $has_next_suggestions = true;
  public $has_next_suggestions_after_completion = false;

  public function __construct( $current_course_id ) {
    $this->_skeleton_init( $current_course_id );
  }

  public function init() {
    if( !is_user_logged_in() )
      return;

    if( isset( $_GET['complete-course'] ) ) {
      if( !wp_verify_nonce( $_GET['complete-course'], 'complete-course' ) )
        wp_nonce_ays( 'complete-course' );

      $course_lessons = WPEP_Entity_Course::instance()->get_lessons( $this->course_id, 'post_id' );

      foreach( $course_lessons as $course_lesson ) {
        WPEP_Controller::instance()->userActivity->set_lesson_status(
          $course_lesson->id, 1, get_current_user_id()
        );
      }

    } else if( isset( $_GET['incomplete-course'] ) ) {
      if( !wp_verify_nonce( $_GET['incomplete-course'], 'incomplete-course' ) )
        wp_nonce_ays( 'incomplete-course' );

      $course_lessons = WPEP_Entity_Course::instance()->get_lessons( $this->course_id, 'post_id' );

      foreach( $course_lessons as $course_lesson ) {
        WPEP_Controller::instance()->userActivity->set_lesson_status(
          $course_lesson->id, 0, get_current_user_id()
        );
      }
    }

  }

  public function content() {
    $course_video             = get_post_meta( get_the_ID(), 'wpep_course_video', true );
    $course_video_jump_links  = get_post_meta( get_the_ID(), '_wpep_course_video_jump_links', true );
    $course_downloads_list    = get_post_meta( get_the_ID(), '_wpep_course_downloads', true );
    $course_video_description = ( wpep_get_setting_flag( 'course-basic-enable-video-description' )
      ? get_post_meta( get_the_ID(), 'wpep_course_video_description', true )
      : ''
    );

    if( !is_string( $course_video ) )
      $course_video = '';

    if( !is_array( $course_video_jump_links ) )
      $course_video_jump_links = [];

    if( !is_array( $course_downloads_list ) )
      $course_downloads_list = [];

    $description = get_post_meta( $this->course_id , 'wpep_course_description', true );

    $args = [
      'course_id'        => $this->course_id,
      'course_read_only' => ( is_user_logged_in() ? $this->read_only : true ),
      'course_completed' => WPEP_Controller::instance()->userActivity->has_completed_course( $this->course_id, get_current_user_id(),false ),
      'title'            => get_the_title( $this->course_id ),
      'description'      => $description,
      'course_video'     => trim( $course_video ),
      'course_video_description'=> $course_video_description,
      'course_video_jump_links' => $course_video_jump_links,
      'course_downloads_list'   => $course_downloads_list
    ];

    if( wpep_assessments_enabled() )
      $args['final_assessment_id'] = wpep_user_assessment_id_by_course_id( $this->course_id, 0, get_current_user_id() );

    $args = apply_filters( 'wpep_course_single_content_basic_args', $args );

    return WPEP_Controller::instance()->template->get_template( 'course-single/basic-content.php', $args );
  }

  public function get_alias() {
    return WPEP_COURSE_TYPE_ONE_OFF;
  }


}