<?php

namespace WPEP\Coordinator\Content\Single;

use WPEP\Controller as WPEP_Controller;

class ExecutionPlan extends Skeleton {

  public $show_quick_continue;
  public $course_sidebar_option;

  public $has_next_suggestions = true;
  public $has_next_suggestions_after_completion = false;

  public function __construct( $current_course_id ) {
    $this->_skeleton_init( $current_course_id );

    $this->show_quick_continue = wpep_get_setting_flag( 'show-lesson-continue-button' );
  }

  public function init() {
    add_filter( 'wpep_course_single_content_lesson_content', [ $this, 'lesson_content_filter' ], 10, 2 );
  }

  public function remove_init() {
    remove_filter( 'wpep_course_single_content_lesson_content', [ $this, 'lesson_content_filter' ], 10 );
  }

  public function content() {
    $response = '';

    if( apply_filters('wpep_course_single_content_include_header', true ) )
      $response .= $this->get_header();

    if( apply_filters('wpep_course_single_content_include_primary_content', true ) )
      $response .= $this->get_form();

    return $response;
  }

  public function get_alias() {
    return WPEP_COURSE_TYPE_EXECUTION_PLAN;
  }

  public function get_form() {
    $args = $this->get_form_args();

    return apply_filters(
      'wpep_single_content_form_html',
      WPEP_Controller::instance()->template->get_template( 'course-single/form.php', $args ),
      $args
    );
  }

  public function get_form_args() {
    $course_sections = WPEP_Controller::instance()->userActivity->get_course_sections( $this->course_id, get_current_user_id() );

    $args = [
      'course_id'        => $this->course_id,
      'course_read_only' => $this->read_only,
      'course_sections'  => $course_sections,
      'progress_percent' => WPEP_Controller::instance()->userActivity->course_progress_percentage( $this->course_id, get_current_user_id() ),
    ];

    if( wpep_assessments_enabled() ) {
      $args['final_assessment_id']  = wpep_user_assessment_id_by_course_id( $this->course_id, 0, get_current_user_id() );
      $args['assessments_required'] = wpep_course_assessments_required( $this->course_id );
    }

    $args = apply_filters( 'wpep_course_single_content_form_args', $args );

    return $args;
  }

  public function lesson_content_filter( $content, $lesson_information ) {
    $content = wpautop( $content );

    if( isset($lesson_information['options']) && isset( $lesson_information['options']['downloads_list'] ) && !empty( $lesson_information['options']['downloads_list'] ) ) {
      $content .= WPEP_Controller::instance()->template->get_template( 'utilities/downloads-list.php', [
        'downloads_list'  => $lesson_information['options']['downloads_list']
      ] );
    }

    if( !wpep_get_setting_flag( 'course-use-the-content-filter' ) )
      $content = do_shortcode( $content );

    if( !$this->show_quick_continue || $content === '' )
      return $content;

    if( strpos( $content, 'class="lesson-continue-container"' ) !== false )
      return $content;

    $attributes = 'class="lesson-continue"';
    $alignment  = wpep_get_setting( 'lesson-continue-button-position' );

    if( $alignment != 'default' )
      $attributes .= ' style="' . ( $alignment == 'center' ? 'float:none;width:200px;text-align:center;margin:0 auto;display:block;' : ( 'float:' . $alignment . ';'  ) ) . '"';

    return $content . '<p class="lesson-continue-container"><span ' . $attributes . '>' . wpep_get_setting_translated( 'lesson-quick-continue-button-text' ) . '</span></p>';
  }

}