<?php

namespace WPEP\Coordinator\Content\UserProfile;

use WPEP\Controller as WPEP_Controller;

class PublicProfile {

  public function __construct() {
    add_action( 'init', [ $this, '_add_rewrite_rules' ] );
    add_filter( 'query_vars', [ $this, '_add_query_vars' ] );
    add_filter( 'status_header', [ $this, '_status_header' ], 50 );

    add_filter( 'wpep_current_request_template', [ $this, '_wpep_current_request_template' ] );
  }

  public function _add_query_vars( $query_vars ) {
    $query_vars[] = $this->_profile_coordinator()->user_identifier;

    return $query_vars;
  }

  public function _add_rewrite_rules() {
    add_rewrite_rule(
      $this->_profile_coordinator()->user_identifier . '/([^/]+)/?',
      'index.php?pagename=' . $this->_profile_coordinator()->pagename_identifier . '&' . $this->_profile_coordinator()->user_identifier . '=$matches[1]',
      'top'
    );
    add_rewrite_rule(
      $this->_profile_coordinator()->user_identifier,
      'index.php?pagename=' . $this->_profile_coordinator()->pagename_identifier,
      'top'
    );
  }

  public function _status_header( $response ) {
    if( get_query_var( 'pagename' ) != $this->_profile_coordinator()->pagename_identifier )
      return $response;

    return 'HTTP/1.1 200 OK';
  }

  private function _profile_coordinator() {
    return wpep_controller()->content->userProfileCoordinator;
  }

  public function _wpep_current_request_template( $template ) {
    if( get_query_var( 'pagename' ) != $this->_profile_coordinator()->pagename_identifier )
      return $template;

    add_filter( 'wp_title', [ $this, 'wp_title' ], 50 );
    add_filter( 'pre_get_document_title', [ $this, 'wp_title' ], 50 );
    add_action( 'wp_head', [ $this, 'html_head_tags' ], 5 );

    if( isset( $_GET[ 'wpep-login' ] ) ) {
      add_filter( 'wpep_user_profile_authentication_form_view', [ $this, '_authentication_form_view' ], 200 );
    }

    // Remove Yoast
    if( class_exists( '\WPSEO_Frontend' ) ) {
      $wp_thing = \WPSEO_Frontend::get_instance();
      remove_action('wp_head',array($wp_thing,'head'),1);
    }

    return $template;
  }

  public function wp_title( $title ) {
    return $this->_get_title();
  }

  private function _get_title() {
    return sprintf( wpep_get_setting_translated( 'user-profile-heading' ), wpep_content_user_profile_format_name( $this->_profile_coordinator()->get_display_user() ) );
  }

  public function _authentication_form_view() {
    return 'login';
  }

  public function html_head_tags() {
    $current_user = $this->_profile_coordinator()->get_display_user();

    $description = wpep_user_profile_description_excerpt( $current_user->ID );

    echo '<meta name="description" content="' . $description . '">' . "\n";
    echo '<meta property="og:url"         content="' . wpep_user_profile_public_link( $current_user ) . '" />' . "\n";
    echo '<meta property="og:type"        content="profile" />' . "\n";
    echo '<meta property="og:title"       content="' . $this->_get_title() . '" />' . "\n";
    echo '<meta property="og:description" content="' . $description . '" />' . "\n";
    echo '<meta property="og:image"       content="' . wpep_user_profile_picture_url( $current_user->ID ). '" />' . "\n";
  }

}