<?php

namespace WPEP\Coordinator;

class ContentLibrary {

  protected $_integrations = [];

  /**
   * @param \WPEP_Content_Library_Integration $content_library_integration
   */
  public function register( $content_library_integration ) {
    $this->_integrations[ $content_library_integration->prefix ] = $content_library_integration;
  }

  public function get_integrations(){
    return $this->_integrations;
  }

  /**
   * @param $post_id
   * @return bool
   */
  public function has_access_to_post( $post_id ) {
    $has_access = true;

    foreach( $this->_integrations as $integration ) {
      if( $integration->has_access( $post_id ) )
        continue;

      $has_access = false;
      break;
    }

    return $has_access;
  }

  public function access_to_post_map( $post_id ) {
    $response = [
      'has_access'     => true,
      'purchase_text'  => '',
      'purchase_link'  => ''
    ];

    foreach( $this->_integrations as $integration ) {
      if( $integration->has_access( $post_id ) )
        continue;

      $response['has_access']     = false;
      $response['purchase_text']  = $integration->get_sell_text( $post_id );
      $response['purchase_link']  = $integration->get_sell_link( $post_id );

      return $response;
    }

    return $response;
  }

}