<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;
use WPEP\Container\Migration as WPEP_Migration_Container;

class Migration {

  use WPEP_Migration_Container;

  public function setup() {
    $this->trait_setup( [
      'has_admin'                   => true,
      'namespace'                   => 'wpep',
      'plugin_file_path'            => WPEP_BASE_FILE_PATH,
      'option_name'                 => 'wpep_migration_version',
      'option_progress_name'        => 'wpep_migration_progress',
      'current_version_available'   => '1.8.0',
      'plugin_migration_folder_path'=> WPEP_BASE_PATH . '/lib/migrations/',
      'migration_list'              => [
        '1.0.0' => 'version-1.0.0.php',
        '1.0.1' => 'version-1.0.1.php',
        '1.0.2' => 'version-1.0.2.php',
        '1.1.0' => 'version-1.1.0.php',
        '1.2.0' => 'version-1.2.0.php',
        '1.3.0' => 'version-1.3.0.php',
        '1.3.1' => 'version-1.3.1.php',
        '1.3.2' => 'version-1.3.2.php',
        '1.3.3' => 'version-1.3.3.php',
        '1.3.4' => 'version-1.3.4.php',
        '1.4.0' => 'version-1.4.0.php',
        '1.4.1' => 'version-1.4.1.php',
        '1.4.2' => 'version-1.4.2.php',
        '1.4.3' => 'version-1.4.3.php',
        '1.5.0' => 'version-1.5.0.php',
        '1.5.1' => 'version-1.5.1.php',
        '1.5.2' => 'version-1.5.2.php',
        '1.5.3' => 'version-1.5.3.php',
        '1.6.0' => 'version-1.6.0.php',
        '1.6.1' => 'version-1.6.1.php',
        '1.6.2' => 'version-1.6.2.php',
        '1.6.3' => 'version-1.6.3.php',
        '1.7.1' => 'version-1.7.1.php',
        '1.7.2' => 'version-1.7.2.php',
        '1.8.0' => 'version-1.8.0.php',
      ],
    ] );
  }

  public function wpmu_drop_tables( $tables ) {
    global $wpdb;

    $tables[] = $wpdb->prefix . 'wpep_course_user_data';
    $tables[] = $wpdb->prefix . 'wpep_section';
    $tables[] = $wpdb->prefix . 'wpep_section_lesson';
    $tables[] = $wpdb->prefix . 'wpep_assessment_question';
    $tables[] = $wpdb->prefix . 'wpep_assessment_question_answer';
    $tables[] = $wpdb->prefix . 'wpep_assessment_user_data';

    return $tables;
  }

  public function _after_activation() {
    WPEP_Settings::instance()->update_settings( [
      'setup-rewrite-rules-flushed'  => 0,
    ], true );
  }

  public function _after_background_migration() {
    WPEP_Settings::instance()->update_settings( [
      'show-lesson-hover-color'      => '1',
      'wpep-theme'                   => WPEP_Controller::instance()->templateStyle->default_theme
    ], true );
  }

  public function handle_migration_in_background() {
    if( $this->current_version != '0.0.0' )
      return false;

    if( count( get_posts( [
        'post_type'   => WPEP_POST_TYPE_COURSE,
        'numberposts' => -1,
        'fields'      => 'ids',
        'meta_key'    => '_cic_cb-course-desc' // Under 1.2.4.4
    ] ) ) > 0 )
      return false;

    return true;
  }

}