<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\Course as Entity_Course;

class PreAuth {

  private $_cookie_completed_lessons     = 'wpep_pacl';
  private $_cookie_completed_lessons_map = 'wpep_paclm';
  /**
   * @var array lesson_id => course_id
   */
  public $completed_lessons     = [];

  public $completed_lessons_map = [];

  public function setup() {
    if( isset( $_COOKIE[ $this->_cookie_completed_lessons ] ) )
      $this->completed_lessons = json_decode( $_COOKIE[ $this->_cookie_completed_lessons ], true );

    if( isset( $_COOKIE[ $this->_cookie_completed_lessons_map ] ) )
      $this->completed_lessons_map = json_decode( $_COOKIE[ $this->_cookie_completed_lessons_map ], true );

    add_action( "wpep_user_activity_lesson_status", [ $this, 'lesson_status' ], 5, 3 );

    add_filter( "wpep_user_activity_lesson_is_completed", [ $this, 'lesson_is_completed' ], 5, 4 );
    add_filter( "wpep_user_activity_course_progress_percentage", [ $this, 'course_progress_percentage' ], 5, 4 );

    add_action( 'user_register', [ $this, 'user_register' ] );
  }

  public function lesson_status( $lesson_id, $is_completed, $user_id ) {
    if( $user_id != 0 )
      return;

    $course_id = WPEP_Controller::instance()->userActivity->get_course_lesson( intval( $lesson_id ) )->course_id;

    if( $is_completed == 0 ) {
      if( !isset( $this->completed_lessons_map[$course_id] ) )
        $this->completed_lessons_map[$course_id] = [];
      else
        unset( $this->completed_lessons_map[$course_id][$lesson_id] );

      unset( $this->completed_lessons[ $lesson_id ] );

    } else {
      $this->completed_lessons[$lesson_id] = time();

      if( !isset( $this->completed_lessons_map[$course_id] ) )
        $this->completed_lessons_map[$course_id] = [];

      $this->completed_lessons_map[$course_id][$lesson_id] = time();
    }

    setcookie( $this->_cookie_completed_lessons, json_encode( $this->completed_lessons ), time() + MONTH_IN_SECONDS , '/' );
    setcookie( $this->_cookie_completed_lessons_map, json_encode( $this->completed_lessons_map ), time() + MONTH_IN_SECONDS , '/' );
  }

  public function lesson_is_completed( $is_completed, $lesson_id, $user_id, $lesson_information ) {
    if( $user_id != 0 )
      return $is_completed;

    return ( isset( $this->completed_lessons[ $lesson_id ] ) );
  }

  public function course_progress_percentage( $percentage, $course_id, $user_id, $course_progress ) {
    if( $user_id != 0 )
      return $percentage;

    if( !isset( $this->completed_lessons_map[$course_id] ) )
      return 0;

    $completed_lessons = count( $this->completed_lessons_map[$course_id] );
    $lesson_count      = Entity_Course::instance()->get_lesson_count( $course_id );

    if( $completed_lessons == 0 || $lesson_count == 0)
      return 0;

    return floatval( $completed_lessons / $lesson_count * 100 );
  }

  public function user_register( $user_id ) {
    // Admin added an user, we want to avoid mistakes.
    if( current_user_can( 'edit_users' ) )
      return;

    if( empty( $this->completed_lessons ) )
      return;

    foreach( $this->completed_lessons as $lesson_id => $timestamp )
      WPEP_Controller::instance()->userActivity->set_lesson_status( intval( $lesson_id ), 1, $user_id, gmdate( 'Y-m-d H:i:s', intval( $timestamp ) ) );

    $this->completed_lessons     = [];
    $this->completed_lessons_map = [];

    setcookie( $this->_cookie_completed_lessons, json_encode( $this->completed_lessons ), time() , '/' );
    setcookie( $this->_cookie_completed_lessons_map, json_encode( $this->completed_lessons_map ), time() , '/' );
  }
  
}