<?php
namespace WPEP\Coordinator\ShortCode;

use WPEP\Controller as WPEP_Controller;

class Misc {

  public function __construct() {
    add_shortcode( 'wpep_post_content', [ $this, 'post_content' ] );
    add_shortcode( 'wpep_center', [ $this, 'center_section' ] );
    add_shortcode( 'wpep_notification', [ $this, 'notification' ] );
    add_shortcode( 'wpep_link', [ $this, 'link' ] );
    add_shortcode( 'wpep_video', [ $this, 'video' ] );
    add_shortcode( 'wpep_hero_section', [ $this, 'hero_section' ] );

    add_shortcode( 'wpep_search', [ $this, 'search' ] );
  }

  public function post_content($attributes, $post_id) {
    $post_content_post = get_post( intval( $post_id ) );

    if( ! ( $post_content_post instanceof \WP_Post ) ) {
      $response  = '<div class="wpep-shortcode-post-content">';
      $response .=    __( "Post ID Provided is not valid. ( " . $post_id . ' ) ' );
      $response .= '</div>';

      return $response;
    }

    global $post;
    $previous_post = $post;

    $post = $post_content_post;

    setup_postdata( $post_content_post );

    $response  = '<div class="wpep-shortcode-post-content">';
    $response .=    apply_filters( 'the_content', $post->post_content );
    $response .= '</div>';

    $post = $previous_post;
    setup_postdata( $post );

    return $response;
  }

  public function center_section($attributes, $content) {
    $response  = '<div class="wpep-center-section">';
    $response .=    do_shortcode( $content );
    $response .= '</div>';

    return $response;
  }

  public function notification( $attributes, $content ) {
    $type = isset( $attributes['type'] ) ? $attributes['type'] : 'standard';

    return '<div class="wpep-notification wpep-notification-' . $type . '">' . $content . '</div>';
  }

  public function link($attributes, $link = null) {
    return wpep_frontend_button( $link, $attributes );
  }

  public function video( $attributes, $content ) {
    $content = strip_tags( $content );
    $content = trim( $content );

    return wpep_get_video_source_output( $content );
  }

  public function hero_section( $attributes, $content ) {
    return '<div class="wpep-hero-section">' . do_shortcode( $content ) . '</div>';
  }

  public function search( $attributes ) {
    return WPEP_Controller::instance()->template->get_template( "search/form.php", [
      'post_type_list'     => ( isset( $attributes['post_type'] )          ? $attributes['post_type']          : '' ),
      'include_categories' => ( isset( $attributes['include_categories'] ) ? $attributes['include_categories'] : '' ),
      'exclude_categories' => ( isset( $attributes['exclude_categories'] ) ? $attributes['exclude_categories'] : '' )
    ] );
  }

}