<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;

class Template {

  public function setup() {
    add_action( 'init', [ $this, 'init' ], 50 );
  }

  public function init() {
    $priority = ( get_option( 'template' ) == 'optimizePressTheme' ? 11 : 1 );

    add_filter( 'template_include', [ $this, 'load' ], $priority);
  }

  public function load($template_path) {
    $current_request_template = $this->_get_current_request_template();

    if( $current_request_template == false ) {
      if( wpep_has_embed_support() && isset( $_GET['wpep-embed'] ) ) {
        if( get_post_type() == 'page' )
          return $this->locate_template( '_embed/page-post-types.php' );

        return $this->locate_template( '_embed/external-post-types.php' );
      }
      return $template_path;
    }

    if( apply_filters( 'wpep_current_request_verify_auth_rules', true ) ) {

      if( !is_user_logged_in() && in_array( $current_request_template, wpep_get_auth_status_page_restrictions() ) ) {

        if( !is_single() || ( is_single() && !wpep_post_has_member_magnet_option( get_the_ID() ) ) )
          $current_request_template = 'authentication.php';
      }

      if( !is_user_logged_in() && $current_request_template == 'content-assessment-single.php' )
        $current_request_template = 'authentication.php';

    }

    $this->_setup_request_template( $current_request_template );

    return $this->locate_template( $current_request_template );
  }

  private function _get_current_request_template() {
    $template_name = false;

    if ( wpep_get_setting( 'opt-course-index' ) != '' && wpep_get_setting( 'opt-course-index' ) != 0 && get_the_ID() == wpep_get_setting('opt-course-index')  ) {
      $template_name = 'primary.php';
    } elseif (  is_single() && get_post_type() == WPEP_POST_TYPE_COURSE  ) {
      global $post;

      $template_name = (
        wpep_get_setting_flag( 'course-layout-content-editor' )
          && (
            strpos( $post->post_content, '[wpep_the_course]' ) !== false
            || strpos( $post->post_content, '[WPEP_the_course]' ) !== false
          )
        ? $template_name
        : 'content-course-single.php'
      );
    } elseif ( is_single() && get_post_type() == WPEP_POST_TYPE_VIDEO ) {
      $template_name = 'content-video-single.php';
    } elseif ( is_single() && get_post_type() == WPEP_POST_TYPE_ASSESSMENT ) {
      $template_name = 'content-assessment-single.php';
    } elseif ( is_archive()
                && in_array(
                 get_post_type(),
                 [ WPEP_POST_TYPE_COURSE, WPEP_POST_TYPE_E_BOOK, WPEP_POST_TYPE_VIDEO, WPEP_POST_TYPE_OFFER, WPEP_POST_TYPE_ASSESSMENT ]
                )
    ) {
      $template_name = 'primary-category.php';
    } elseif( get_query_var( 'pagename' ) == WPEP_Controller::instance()->content->userProfileCoordinator->pagename_identifier
                && wpep_public_profiles_enabled() ) {
      $template_name = 'user-profile.php';
    }

    if( isset( $_GET['wpep-login'] ) && $_GET['wpep-login'] == 1 && !is_user_logged_in() ) {
      $template_name = 'authentication.php';
    }

    return apply_filters( 'wpep_current_request_template', $template_name );
  }

  private function _setup_request_template( $template_name ) {
    if( $template_name == 'primary-category.php' ) {
      $current_category_object = get_queried_object();

      add_filter( 'wpep_template_category_list_items_per_row', function( $current_categories_per_row, $base_categories_per_row = '' ) {
        return ( !empty( $base_categories_per_row ) ? $base_categories_per_row : $current_categories_per_row );
        }, 15, 2 );

      add_filter( 'wpep_primary_content_grid_categories', function() use ( $current_category_object ) { return [ $current_category_object ]; } );
    }

    if( $template_name == 'content-video-single.php' ) {
      $current_categories = wp_get_object_terms( get_the_ID(), 'course_category', [ 'parent' => 0 ] );

      if( empty( $current_categories ) ) {
        $current_categories = [];

        foreach( wp_get_object_terms( get_the_ID(), 'course_category' ) as $child_taxonomy ) {
          if( !isset( $current_categories[ $child_taxonomy->parent ] ) )
            $current_categories[ $child_taxonomy->parent ] = get_term( $child_taxonomy->parent );
        }
      }

      add_filter( 'wpep_primary_content_display_header', function() { return false; } );
      add_filter( 'wpep_content_primary_has_categories', function() { return false; } );
      add_filter( 'wpep_primary_content_grid_categories', function() use ( $current_categories ) { return $current_categories; } );
    }

    if( $template_name == 'user-profile.php' ) {
      WPEP_Controller::instance()->assets->has_enqueue_media_support = true;
    }
  }

  public function get_template( $template_name, $args = [], $template_path = '', $default_path = '' ) {
    ob_start();

    $this->load_template( $template_name, $args, $template_path, $default_path );

    $ret = ob_get_contents();

    ob_end_clean();

    return $ret;
  }

  public function load_template( $template_name, $args = [], $template_path = '', $default_path = '' ) {
    if ( $args && is_array( $args ) ) {
      extract( $args );
    }

    $located = $this->locate_template( $template_name, $template_path, $default_path );

    if ( ! file_exists( $located ) ) {
      _doing_it_wrong( __FUNCTION__, sprintf( '<code>%s</code> does not exist.', $located ), '1.0.0' );
      return;
    }

    // Allow 3rd party plugin filter template file from their plugin
    $located = apply_filters( 'wpep_get_template', $located, $template_name, $args, $template_path, $default_path );

    do_action( 'wpep_before_template_part', $template_name, $template_path, $located, $args );

    include( $located );

    do_action( 'wpep_after_template_part', $template_name, $template_path, $located, $args );
  }

  public function locate_template( $template_name, $template_path = '', $default_path = '' ) {
    if ( ! $template_path )
      $template_path = $this->template_path();

    if ( ! $default_path )
      $default_path = WPEP_BASE_PATH . '/templates/';

    // Look within passed path within the theme - this is priority
    $template = locate_template( [
        trailingslashit( $template_path ) . $template_name,
        $template_name
    ] );

    // Get default template
    if ( ! $template )
      $template = $default_path . $template_name;

    // Return what we found
    return apply_filters( 'wpep_locate_template', $template, $template_name, $template_path );
  }

  public function template_path() {
    return apply_filters( 'wpep_template_path', 'wpep/' );
  }

  public function get_header() {
    if( !$this->_is_wpep_embed() ) {
      get_header();
      return;
    }

    show_admin_bar(false);

    $this->load_template( '_embed/header.php' );
  }

  public function get_footer() {
    if( !$this->_is_wpep_embed() ) {
      get_footer();
      return;
    }

    $this->load_template( '_embed/footer.php' );
  }

  private function _is_wpep_embed() {
    if( !wpep_has_embed_support() )
      return false;

    if( isset( $_GET[ 'wpep-embed'] ) && $_GET[ 'wpep-embed'] == true )
      return true;

    return false;
  }

}
