<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\Course as WPEP_Course;

class UserActivity {

  /**
   * @var UserActivity\Assessment
   */
  public $assessment;


  /**
   * @var UserActivity\Statistics
   */
  public $statistics;

  public function setup() {
    $this->assessment = new UserActivity\Assessment();
    $this->statistics = new UserActivity\Statistics();
  }

  public function get_course_bookmark_status( $course_id, $user_id = false ) {
    return $this->get_course_user_data_value(
      WPEP_USER_COURSE_IS_BOOKMARKED,
      $course_id,
      false,
      false,
      $user_id,
      true
    );
  }


  public function set_course_bookmark_status( $course_id, $is_bookmarked, $user_id = false ) {
    return $this->set_course_user_data(
      WPEP_USER_COURSE_IS_BOOKMARKED,
      intval( $is_bookmarked ),
      $course_id,
      false,
      false,
      $user_id
    );
  }

  public function set_lesson_status( $lesson_id, $is_completed = true, $user_id = false, $created_at = false, $set_progress_flags = true ) {
    $user_id = ( $user_id === false ? get_current_user_id() : $user_id );

    if( $user_id == 0 ) {
      do_action( "wpep_user_activity_lesson_status", $lesson_id, $is_completed, $user_id );
      return;
    }

    $lesson_id    = intval( $lesson_id );
    $is_completed = intval( $is_completed );
    $course_id    = WPEP_Course::instance()->get_course_id_by_lesson_id( $lesson_id );

    global $wpdb;

    $wpdb->delete( $wpdb->prefix . 'wpep_section_lesson_completed', [
      'user_id'   => $user_id,
      'lesson_id' => $lesson_id
    ] );

    if( $is_completed ) {
      $wpdb->insert( $wpdb->prefix . 'wpep_section_lesson_completed', [
        'user_id'    => $user_id,
        'lesson_id'  => $lesson_id,
        'created_at' => ( $created_at == false ? current_time( 'mysql' ) : $created_at )
      ] );
    }

    if( $set_progress_flags ) {
      $this->set_course_user_data(
        WPEP_USER_COURSE_PROGRESS,
        $this->course_progress_percentage_raw( $course_id, $user_id ),
        $course_id,
        0,
        0,
        $user_id
      );

      if( $this->course_progress_percentage( $course_id, $user_id ) == 100 ) {
        $this->set_course_user_data(
          WPEP_USER_COURSE_COMPLETED,
          1,
          $course_id,
          0,
          0,
          $user_id
        );

        $this->set_course_user_data(
          WPEP_USER_COURSE_COMPLETED_AT,
          time(),
          $course_id, 0, 0, $user_id
        );
      } else {
        $this->set_course_user_data(
          WPEP_USER_COURSE_COMPLETED,
          0,
          $course_id,
          0,
          0,
          $user_id
        );

        $this->remove_course_user_data(
          WPEP_USER_COURSE_COMPLETED_AT,
          null,
          $course_id, 0, 0, $user_id
        );
      }

    }
    
    $started_at = $this->get_course_user_data_value( WPEP_USER_COURSE_STARTED, $course_id, 0, 0, $user_id, true );

    if( $started_at === null )
      $this->set_course_user_data( WPEP_USER_COURSE_STARTED, time(), $course_id, 0, 0, $user_id );

    $this->set_course_user_data( WPEP_USER_COURSE_LATEST_INTERACTION, time(), $course_id, 0, 0, $user_id );

    do_action( "wpep_user_activity_lesson_status", $lesson_id, $is_completed, $user_id );
  }

  public function set_section_learning_path( $course_id, $section_id, $learning_path_index, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return;

    $this->set_course_user_data( 'learning_path', $learning_path_index, $course_id, $section_id, 0, $user_id );
    $this->set_course_user_data( WPEP_USER_COURSE_LATEST_INTERACTION, time(), $course_id, 0, 0, $user_id );

    do_action( "wpep_user_set_section_learning_path", $course_id, $section_id, $learning_path_index, $user_id );
  }

  public function has_completed_course( $course_id = false, $user_id = false, $including_assessments = false ) {
    $course_id = ( $course_id == false ? get_the_ID()          : $course_id );
    $user_id   = ( $user_id   == false ? get_current_user_id() : $user_id );

    if( $course_id == 0 || $user_id == 0 )
      return false;

    $is_completed = $this->get_course_user_data_value( WPEP_USER_COURSE_COMPLETED, $course_id, 0, 0, $user_id, true );

    if( ( $is_completed === null ? false : intval( $is_completed ) ) == false )
      return false;

    if( $including_assessments == false )
      return true;

    $remaining_assessments = wpep_get_assessment_requirements_for_course( $course_id, $user_id );

    if( empty( $remaining_assessments ) )
      return true;

    return false;
  }

  public function get_course_user_data_value( $key = false, $course_id = false, $section_id = false, $lesson_id = false, $user_id = false, $single_row = true ) {
    if( $single_row ) {
      $response = null;

      $user_data = $this->get_course_user_data( $key, $course_id, $section_id, $lesson_id, $user_id, true );

      if( $user_data != NULL )
        return $user_data->value;

      return $response;
    }

    $response = [];

    foreach( $this->get_course_user_data( $key, $course_id, $section_id, $lesson_id, $user_id, false ) as $user_data )
      $response[$user_data->id] = $user_data->value;

    return $response;
  }

  public function get_course_user_data( $key = false, $course_id = false, $section_id = false, $lesson_id = false, $user_id = false, $single_row = true, $value = false ) {
    if ( $single_row ) {
      $user_id = ( $user_id === false ? get_current_user_id() : $user_id );

      if ($user_id == 0 )
        return null;
    }

    global $wpdb;

    $sql = 'SELECT * FROM ' . $wpdb->prefix . 'wpep_course_user_data cud WHERE 1 = 1 ';

    if( $user_id != 0 )
      $sql .= ' AND cud.user_id = ' . intval( $user_id );

    if( $key !== false )
      $sql .= ' AND cud.key = "' . htmlentities( $key ) . '"';

    if( $course_id !== false )
      $sql .= ' AND cud.course_id  = ' . intval( $course_id );

    if( $section_id !== false )
      $sql .= ' AND cud.section_id = ' . intval( $section_id );

    if( $lesson_id !== false )
      $sql .= ' AND cud.lesson_id  = ' . intval( $lesson_id );

    if( $value !== false ) {
      if( is_array( $value ) && isset( $value[0] ) && isset( $value[1] )  ) {
        $sql .= ' AND cud.value  >= ' . intval( $value[0] );
        $sql .= ' AND cud.value  <= ' . intval( $value[1] );
      } else {
        $sql .= ' AND cud.value  = ' . intval( $value );
      }
    }

    return ( $single_row ? $wpdb->get_row( $sql ) : $wpdb->get_results( $sql ) );

  }

  public function set_course_user_data( $key, $value, $course_id, $section_id, $lesson_id, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return false;

    $course_id  = intval( $course_id );
    $section_id = intval( $section_id );
    $lesson_id  = intval( $lesson_id );

    $existent_entry = $this->get_course_user_data( $key, $course_id, $section_id, $lesson_id, $user_id, true );

    $value_updated = true;

    if( $existent_entry === null )
      $this->add_course_user_data( $key, $value, $course_id, $section_id, $lesson_id, $user_id );
    else if( $existent_entry->value != $value ) {
      global $wpdb;

      $wpdb->update(
        $wpdb->prefix . 'wpep_course_user_data',
        [ 'value' => $value ],
        [ 'id'    => $existent_entry->id ]
      );
    } else {
      $value_updated = false;
    }

    if( !$value_updated )
      return true;

    do_action( "wpep_user_set_course_data", $key, $value, $course_id, $section_id, $lesson_id, $user_id, $existent_entry );

    return true;
  }

  public function add_course_user_data( $key, $value, $course_id, $section_id = 0, $lesson_id = 0, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return;

    $course_id  = intval( $course_id );
    $section_id = intval( $section_id );
    $lesson_id  = intval( $lesson_id );

    global $wpdb;

    $wpdb->insert( $wpdb->prefix . 'wpep_course_user_data', [
      'user_id'    => $user_id,
      'course_id'  => $course_id,
      'section_id' => $section_id,
      'lesson_id'  => $lesson_id,
      'key'        => $key,
      'value'      => $value,
    ] );

    do_action( "wpep_user_add_course_data", $key, $value, $course_id, $section_id, $lesson_id, $user_id );
  }

  public function remove_course_user_data( $key = null, $value = null, $course_id = null, $section_id = null, $lesson_id = null, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return null;

    $delete_where = [ 'user_id' => $user_id ];

    if( $key !== null )
      $delete_where['key'] = htmlentities( $key );

    if( $value !== null )
      $delete_where['value'] = htmlentities( $value );

    if( $course_id !== null )
      $delete_where['course_id'] = intval( $course_id );

    if( $section_id !== null )
      $delete_where['section_id'] = intval( $section_id );

    if( $lesson_id !== null )
      $delete_where['lesson_id'] = intval( $lesson_id );

    global $wpdb;

    $wpdb->delete(
      $wpdb->prefix . 'wpep_course_user_data',
      $delete_where
    );

    do_action( "wpep_user_remove_course_data", $key, $value, $course_id, $section_id, $lesson_id, $user_id );
  }

  /**
   * @param $section_id
   * @param bool $user_id
   * @return array
   */
  public function get_course_section( $section_id, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    $section_completed = true;
    $user_section = apply_filters( 'wpep_user_activity_course_section', $this->get_section_map( $section_id, $user_id ) );
    $response     = [
      'id'           => $section_id,
      'title'        => $user_section->title,
      'content'      => $user_section->content,
    ];

    foreach( $user_section->lessons as $lesson_key => $lesson_information ) {

      $is_completed = apply_filters( 'wpep_user_activity_lesson_is_completed', $lesson_information->is_completed, $lesson_information->id, $user_id, $lesson_information );

      $response['lessons'][$lesson_information->id] = [
        'title'        => $lesson_information->title,
        'content'      => $lesson_information->content,
        'is_completed' => $is_completed
      ];

      if( $is_completed == false )
        $section_completed = false;

    }

    $response['is_completed'] = $section_completed;

    return $response;
  }

  /**
   * This function will return content as needed based on the Timing Options if enabled.
   * @param $course_id
   * @param bool $user_id
   * @param bool|null $handle_timing
   * @return array
   */
  public function get_course_sections( $course_id, $user_id = false, $handle_timing = null ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $handle_timing == null )
      $handle_timing = wpep_user_handle_timing_options( $handle_timing );

    $user_sections = apply_filters( 'wpep_user_activity_course_sections', $this->course_map( $course_id, $user_id, true, $handle_timing ) );
    $response      = [];

    foreach( $user_sections as $user_section ) {
      $section_completed = true;

      $response[$user_section->id] = [
        'title'        => $user_section->title,
        'content'      => $user_section->content,
        'lessons'      => [],
        'options'      => ( isset( $user_section->options )   ? ( is_object( $user_section->options ) ? get_object_vars( $user_section->options ) : $user_section->options ) : [] ),
        'user_data'    => ( isset( $user_section->user_data ) ? ( is_object( $user_section->user_data ) ? get_object_vars( $user_section->user_data ) : $user_section->user_data ) : [] ),
      ];

      if( wpep_assessments_enabled() )
        $response[$user_section->id]['assessment_id'] = wpep_user_assessment_id_by_course_id( $course_id, $user_section->id, get_current_user_id() );

      foreach( $user_section->lessons as $lesson_key => $lesson_information ) {

        $is_completed = apply_filters( 'wpep_user_activity_lesson_is_completed', $lesson_information->is_completed, $lesson_information->id, $user_id, $lesson_information );

        $response[$user_section->id]['lessons'][$lesson_information->id] = [
          'title'        => $lesson_information->title,
          'content'      => $lesson_information->content,
          'is_completed' => $is_completed,
          'options'      => ( isset( $lesson_information->options ) ? ( is_object( $lesson_information->options ) ? get_object_vars( $lesson_information->options ) : $lesson_information->options ) : [] ),
        ];

        if( $is_completed == false )
          $section_completed = false;

      }

      $response[$user_section->id]['is_completed'] = $section_completed;

    }

    return $response;
  }

  /**
   * @param $course_id
   * @param bool $user_id
   * @return array|null|object
   */
  public function course_lesson_completion_map( $course_id, $user_id = false ) {
    global $wpdb;

    if( $user_id === false )
      $user_id = get_current_user_id();

    $sql = 'SELECT lc.created_at as `date`,
                   l.id          as lesson_id,
                   l.title       as lesson_title,
                   s.id          as section_id,
                   s.title       as section_title
             FROM ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
             LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson l ON l.id = lc.lesson_id
             LEFT JOIN ' . $wpdb->prefix . 'wpep_section s ON s.id = l.section_id
             WHERE lc.user_id = ' . intval( $user_id ) . '
               AND s.post_id  = ' . intval( $course_id ) . '
             ORDER BY lc.created_at ASC';

    return $wpdb->get_results($sql);
  }

  /**
   * This function will return content as needed based on the Timing Options if enabled.
   * @param $course_id
   * @param bool $user_id
   * @param bool $include_lessons
   * @param bool|null $handle_timing
   * @return array
   */
  public function course_map( $course_id, $user_id = false, $include_lessons = true, $handle_timing = null ) {
    global $wpdb;

    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $handle_timing == null )
      $handle_timing = wpep_user_handle_timing_options( $handle_timing );

    $response = [];

    $sql = 'SELECT * FROM ' . $wpdb->prefix . 'wpep_section WHERE post_id = ' . $course_id;

    if( $handle_timing ) {

      $seconds_spent = wpep_user_course_started_at_seconds( $course_id, $user_id );

      $sql .= ' AND ( time_action_display_from = 0 OR time_action_display_from < ' . $seconds_spent . ' ) ';
      $sql .= ' AND ( time_action_display_until = 0 OR time_action_display_until > ' . $seconds_spent . ' ) ';
    }

    $sql .= ' ORDER BY `order` ASC';

    $course_sections = $wpdb->get_results($sql);

    foreach( $course_sections as $course_section ) {
      $response[$course_section->id] = $course_section;
      $response[$course_section->id]->user_data = wpep_utility_map_to_object_assoc ( $this->get_course_user_data( false, $course_id, $course_section->id, false, $user_id, false ), 'key', 'value' );
      $response[$course_section->id]->options = ( $course_section->options == '' ? new \stdClass() : json_decode( $course_section->options ) );

      if( $include_lessons )
        $response[$course_section->id]->lessons = $this->get_section_lessons_map( $course_section->id, $user_id, $handle_timing, $course_id );
    }

    return $response;
  }

  /**
   * This function will return content as needed based on the Timing Options if enabled.
   * @param $section_id
   * @param bool $user_id
   * @param bool|null $handle_timing
   * @param int|null $course_id
   * @return array|null|object
   */
  public function get_section_map( $section_id, $user_id = false, $handle_timing = null, $course_id = null ) {
    global $wpdb;

    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $handle_timing == null )
      $handle_timing = wpep_user_handle_timing_options( $handle_timing );

    $sql = 'SELECT * FROM ' . $wpdb->prefix . 'wpep_section WHERE id = ' . $section_id . ' ORDER BY `order` ASC';

    $course_section = $wpdb->get_row($sql);

    if( is_null( $course_section ) )
      return null;

    if( $handle_timing ) {
      $course_id = ( $course_id == null ? WPEP_Course::instance()->get_course_id_by_section_id( $section_id ) : $course_id );
      $seconds_spent = wpep_user_course_started_at_seconds( $course_id, $user_id );

      if( $course_section->time_action_display_from != 0 && $course_section->time_action_display_from < $seconds_spent )
        return null;

      if( $course_section->time_action_display_until != 0 && $course_section->time_action_display_until > $seconds_spent )
        return null;

    }

    $course_section->lessons = $this->get_section_lessons_map( $section_id, $user_id, $handle_timing, $course_section->post_id );

    return $course_section;
  }

  /**
   * @param $lesson_id
   * @param bool $user_id
   * @param bool $fetch_next_previous
   * @param bool|null $handle_timing
   * @return array|null|object
   */
  public function get_course_lesson( $lesson_id, $user_id = false, $fetch_next_previous = true, $handle_timing = null ) {
    global $wpdb;

    if( $user_id === false )
      $user_id = get_current_user_id();

    $sql = 'SELECT l.*,
                   s.post_id as course_id,
                   s.id      as section_id,
                   s.title   as section_title,
                   IFNULL( s.time_action_display_from, 0 ) as section_time_action_display_from,
                   IFNULL( s.time_action_display_until, 0 ) as section_time_action_display_until,
                   IF(lc.id IS NULL, 0, 1) as is_completed 
              FROM ' . $wpdb->prefix . 'wpep_section_lesson l
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section s ON s.id = l.section_id ';

    $sql .=  'WHERE l.id = ' . $lesson_id . ' LIMIT 1';

    $course_lesson = $wpdb->get_row($sql);

    if( $course_lesson == null )
      return null;

    if( $handle_timing == null )
      $handle_timing = wpep_user_handle_timing_options( $handle_timing );

    if( $handle_timing ) {
      $seconds_spent = wpep_user_course_started_at_seconds( $course_lesson->post_id, $user_id );

      if( $course_lesson->time_action_display_from != 0 && $course_lesson->time_action_display_from > $seconds_spent )
        return null;

      if( $course_lesson->time_action_display_until != 0 && $course_lesson->time_action_display_until < $seconds_spent )
        return null;

      if( $course_lesson->section_time_action_display_from != 0 && $course_lesson->section_time_action_display_from > $seconds_spent )
        return null;

      if( $course_lesson->section_time_action_display_until != 0 && $course_lesson->section_time_action_display_until < $seconds_spent )
        return null;
    }

    if( $fetch_next_previous ) {
      $raw_query = ' SELECT inner_lesson.* FROM ' . $wpdb->prefix . 'wpep_section_lesson inner_lesson
                         LEFT JOIN ' . $wpdb->prefix . 'wpep_section inner_section
                                ON inner_lesson.section_id = inner_section.id
                             WHERE inner_lesson.post_id = ' . $course_lesson->post_id;

      if( $handle_timing ) {
        $raw_query .= ' AND ( inner_section.time_action_display_from = 0 OR inner_section.time_action_display_from < ' . $seconds_spent . ' ) 
                        AND ( inner_section.time_action_display_until = 0 OR inner_section.time_action_display_until > ' . $seconds_spent . ' )
                        AND ( inner_lesson.time_action_display_from = 0 OR inner_lesson.time_action_display_from < ' . $seconds_spent . ' )
                        AND ( inner_lesson.time_action_display_until = 0 OR inner_lesson.time_action_display_until > ' . $seconds_spent . ' )
                      ';
      }

      $previous_lesson = $wpdb->get_row( $raw_query . ' AND inner_lesson.order < ' . $course_lesson->order . ' ORDER BY inner_lesson.order DESC' );
      $next_lesson = $wpdb->get_row( $raw_query . ' AND inner_lesson.order > ' . $course_lesson->order . ' ORDER BY inner_lesson.order ASC' );

      $course_lesson->previous_lesson_id     = ( $previous_lesson == null ? null : $previous_lesson->id );
      $course_lesson->previous_lesson_title  = ( $previous_lesson == null ? null : $previous_lesson->title );
      $course_lesson->next_lesson_id         = ( $next_lesson == null ? null : $next_lesson->id );
      $course_lesson->next_lesson_title      = ( $next_lesson == null ? null : $next_lesson->title );
      $course_lesson->next_lesson_section_id = ( $next_lesson == null ? null : $next_lesson->section_id );
    }



    $course_lesson->is_completed = apply_filters( 'wpep_user_activity_lesson_is_completed', $course_lesson->is_completed, $course_lesson->id, $user_id, $course_lesson );

    return $course_lesson;
  }

  /**
   * @param $lesson_id
   * @param bool $user_id
   * @param bool|null $handle_timing
   * @return array|null|object
   */
  public function get_course_next_lesson_id( $lesson_id, $user_id = false, $handle_timing = null ) {
    global $wpdb;

    if( $user_id === false )
      $user_id = get_current_user_id();

    $sql = 'SELECT l.*,
                   IFNULL( s.time_action_display_from, 0 ) as section_time_action_display_from,
                   IFNULL( s.time_action_display_until, 0 ) as section_time_action_display_until
              FROM ' . $wpdb->prefix . 'wpep_section_lesson l
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section s ON s.id = l.section_id ';

    $sql .=  'WHERE l.id = ' . $lesson_id . ' LIMIT 1';

    $course_lesson = $wpdb->get_row($sql);

    if( $course_lesson == null )
      return null;

    if( $handle_timing == null )
      $handle_timing = wpep_user_handle_timing_options( $handle_timing );

    if( $handle_timing ) {
      $seconds_spent = wpep_user_course_started_at_seconds( $course_lesson->post_id, $user_id );
    }

    $raw_query = ' SELECT inner_lesson.* FROM ' . $wpdb->prefix . 'wpep_section_lesson inner_lesson
                       LEFT JOIN ' . $wpdb->prefix . 'wpep_section inner_section
                              ON inner_lesson.section_id = inner_section.id
                           WHERE inner_lesson.post_id = ' . $course_lesson->post_id;

    if( $handle_timing ) {
      $raw_query .= ' AND ( inner_section.time_action_display_from = 0 OR inner_section.time_action_display_from < ' . $seconds_spent . ' ) 
                      AND ( inner_section.time_action_display_until = 0 OR inner_section.time_action_display_until > ' . $seconds_spent . ' )
                      AND ( inner_lesson.time_action_display_from = 0 OR inner_lesson.time_action_display_from < ' . $seconds_spent . ' )
                      AND ( inner_lesson.time_action_display_until = 0 OR inner_lesson.time_action_display_until > ' . $seconds_spent . ' )
                    ';
    }

    $next_lesson = $wpdb->get_row( $raw_query . ' AND inner_lesson.order > ' . $course_lesson->order . ' ORDER BY inner_lesson.order ASC' );

    return ( $next_lesson == null ? 0 : $next_lesson->id );
  }

  /**
   * @param $course_id
   * @param bool $user_id
   * @param null $handle_timing
   * @param null $course_id
   * @return array
   */
  public function get_delivered_course_lesson_id( $course_id, $user_id = false, $handle_timing = null ) {
    global $wpdb;

    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $handle_timing == null )
      $handle_timing = wpep_user_handle_timing_options( $handle_timing );

    if( $handle_timing )
      $seconds_spent = wpep_user_course_started_at_seconds( $course_id, $user_id );

    $sql = 'SELECT l.id
                  FROM ' . $wpdb->prefix . 'wpep_section_lesson l
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section s
                     ON s.id = l.section_id
              WHERE lc.id IS NULL
                AND l.post_id = ' . intval( $course_id );

    if( $handle_timing ) {
      $sql .= ' AND ( s.time_action_display_from = 0 OR s.time_action_display_from < ' . $seconds_spent . ' ) 
                AND ( s.time_action_display_until = 0 OR s.time_action_display_until > ' . $seconds_spent . ' )
                AND ( l.time_action_display_from = 0 OR l.time_action_display_from < ' . $seconds_spent . ' )
                AND ( l.time_action_display_until = 0 OR l.time_action_display_until > ' . $seconds_spent . ' )
              ';
    }

    $sql .= ' ORDER BY l.order ASC';


    $resp = $wpdb->get_col($sql);
    $resp = is_array( $resp ) ? array_shift( $resp ) : $resp;

    if( is_null( $resp ) ) {
      $sql = 'SELECT l.id
                  FROM ' . $wpdb->prefix . 'wpep_section_lesson l
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section s
                     ON l.section_id = s.id
              WHERE l.post_id = ' . intval( $course_id );

      if( $handle_timing ) {
        $sql .= ' AND ( s.time_action_display_from = 0 OR s.time_action_display_from < ' . $seconds_spent . ' ) 
                  AND ( s.time_action_display_until = 0 OR s.time_action_display_until > ' . $seconds_spent . ' )
                  AND ( l.time_action_display_from = 0 OR l.time_action_display_from < ' . $seconds_spent . ' )
                  AND ( l.time_action_display_until = 0 OR l.time_action_display_until > ' . $seconds_spent . ' )
                ';
      }

      $sql .= ' ORDER BY l.order ASC';

      $resp = $wpdb->get_col($sql);
      $resp = is_array( $resp ) ? array_shift( $resp ) : $resp;
    }

    return $resp;
  }

  /**
   * This function will return content as needed based on the Timing Options if enabled.
   * @param $section_id
   * @param bool $user_id
   * @param bool|null $handle_timing
   * @param int|null $course_id
   * @return array|null|object
   */
  public function get_section_lessons_map( $section_id, $user_id = false, $handle_timing = null, $course_id = null ) {
    global $wpdb;

    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $handle_timing == null )
      $handle_timing = wpep_user_handle_timing_options( $handle_timing );

    $section_id = intval( $section_id );

    $sql = 'SELECT l.*, IF(lc.id IS NULL, 0, 1) as is_completed
                FROM ' . $wpdb->prefix . 'wpep_section_lesson l
               LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
                    ON l.id       = lc.lesson_id
                   AND lc.user_id = ' . intval( $user_id ) . '
               WHERE l.section_id = ' . $section_id;

    if( $handle_timing ) {
      $course_id = ( $course_id == null ? WPEP_Course::instance()->get_course_id_by_section_id( $section_id ) : $course_id );

      $seconds_spent = wpep_user_course_started_at_seconds( $course_id, $user_id );

      $sql .= ' AND ( l.time_action_display_from = 0 OR l.time_action_display_from < ' . $seconds_spent . ' ) ';
      $sql .= ' AND ( l.time_action_display_until = 0 OR l.time_action_display_until > ' . $seconds_spent . ' ) ';
    }

    $sql .= ' ORDER BY l.`order` ASC';

    $response = $wpdb->get_results($sql);

    foreach( $response as $response_key => $response_value )
      $response[$response_key]->options = ( $response_value->options == '' ? new \stdClass() : json_decode( $response_value->options ) );

    return $response;
  }

  public function get_course_lesson_completed_date( $course_id, $user_id = false, $order = 'ASC' ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    global $wpdb;

    $sql = 'SELECT lc.created_at
                  FROM ' . $wpdb->prefix . 'wpep_section_lesson l
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              WHERE l.post_id = ' . intval( $course_id ) . '      
              ORDER BY lc.created_at ' . ( $order == 'ASC' ? 'ASC' : 'DESC' );

    $date = $wpdb->get_var( $sql );

    return ( is_string( $date ) ? strtotime( $date ) : 0 );
  }

  public function course_progress_percentage( $course_id, $user_id = false, $option_section_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    $course_progress = $this->get_course_user_data( WPEP_USER_COURSE_PROGRESS, $course_id, false, false, $user_id );

    if( $course_progress !== null && isset( $course_progress->value ) && $option_section_id == false ) {
      $course_progress = floatval( $course_progress->value );

    } else if( get_post_meta( $course_id, 'wpep_learning_path_enabled', true ) ) {
      $course_progress = floatval( 0 );
    } else {
      $course_progress = $this->course_progress_percentage_raw( $course_id, $user_id, $option_section_id );
    }


    return apply_filters( 'wpep_user_activity_course_progress_percentage', $course_progress, $course_id, $user_id, $course_progress );
  }

  public function course_progress_percentage_raw( $course_id, $user_id = false, $option_section_id = false ) {
    global $wpdb;
    $section_id_query = 'IN (
        SELECT id FROM ' . $wpdb->prefix . 'wpep_section WHERE post_id = ' . intval( $course_id ) . '
      )';

    if( is_numeric( $option_section_id ) && $option_section_id != false )
      $section_id_query = '= ' . $option_section_id;
    else if( is_array( $option_section_id ) )
      $section_id_query = 'IN ( ' . implode( ',', $option_section_id ) . ' )';

    $sql = 'SELECT ( SUM(IF(lc.id IS NULL, 0, 1)) / COUNT(l.id) * 100 ) as completed_percent, 
                   COUNT(l.id) as lesson_count
              FROM ' . $wpdb->prefix . 'wpep_section_lesson l
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              WHERE section_id ' . $section_id_query . '
              ORDER BY `order` ASC';

    return floatval( $wpdb->get_var( $sql ) );
  }

  public function course_progress_reset( $course_id, $user_id = false, $option_section_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return false;

    global $wpdb;

    $section_id_query = 'IN (
      SELECT id FROM ' . $wpdb->prefix . 'wpep_section WHERE post_id = ' . intval( $course_id ) . '
    )';

    if( is_numeric( $option_section_id ) && $option_section_id != false )
      $section_id_query = '= ' . $option_section_id;
    else if( is_array( $option_section_id ) )
      $section_id_query = 'IN ( ' . implode( ',', $option_section_id ) . ' )';


    $sql = 'DELETE lc FROM ' . $wpdb->prefix . 'wpep_section_lesson l
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              WHERE section_id ' . $section_id_query;

    $this->remove_course_user_data( WPEP_USER_COURSE_COMPLETED_AT, null, $course_id, null, null, $user_id );
    $this->remove_course_user_data( WPEP_USER_COURSE_COMPLETED, null, $course_id, null, null, $user_id );
    $this->remove_course_user_data( WPEP_USER_COURSE_PROGRESS, null, $course_id, null, null, $user_id );
    $this->remove_course_user_data( WPEP_USER_COURSE_STARTED, null, $course_id, null, null, $user_id );

    return $wpdb->query( $sql );
  }

}