<?php

namespace WPEP\Entity;

class Course {

  protected static $_instance;

  public static function instance() {
    if( self::$_instance == null )
      self::$_instance = new self();

    return self::$_instance;
  }

  /**
   * @var \wpdb;
   */
  public $wpdb;
  public $table_section = 'wpep_section';
  public $table_section_lesson = 'wpep_section_lesson';

  public function __construct() {
    $this->_setup();
  }

  private function _setup() {
    global $wpdb;

    $this->wpdb = $wpdb;

    $this->table_section = $this->wpdb->prefix . $this->table_section;
    $this->table_section_lesson = $this->wpdb->prefix . $this->table_section_lesson;
  }

  public function get_section( $section_id ) {
    $sql = 'SELECT *
              FROM ' . $this->table_section . '
             WHERE id = ' . $section_id;

    $section = $this->wpdb->get_row($sql);

    if( $section === null )
      return null;

    $section->options = ( $section->options == '' ? new \stdClass() : json_decode( $section->options ) );

    return $section;
  }

  public function get_section_by_lesson_id( $lesson_id ) {
    $sql = 'SELECT * FROM ' . $this->table_section . ' 
                     WHERE id = ( SELECT section_id FROM ' . $this->table_section_lesson . ' WHERE id = ' . intval( $lesson_id ) . ' )';

    $section = $this->wpdb->get_row($sql);

    $section->options = ( $section->options == '' ? new \stdClass() : json_decode( $section->options ) );

    return $section;
  }

  public function get_sections_by_course_id( $course_id ) {
    $sql = 'SELECT *
              FROM ' . $this->table_section . '
             WHERE post_id = ' . $course_id . ' ORDER BY `order` ASC';

    $sections = $this->wpdb->get_results($sql);

    foreach( $sections as $key => $section )
      $sections[$key]->options = ( $section->options == '' ? new \stdClass() : json_decode( $section->options ) );

    return $sections;
  }

  public function get_with_lessons( $id = false ) {
    $course_sections = $this->get_list( $id );

    foreach( $course_sections as $k => $course_section )
      $course_sections[$k]->lessons = $this->get_lessons( $course_section->id );

    return $course_sections;
  }

  public function get_list( $id = false ) {
    if( $id == false )
      $id = get_the_ID();

    return $this->get_sections_by_course_id( $id );
  }

  public function get_course_id_by_lesson_id( $lesson_id ) {
    $sql = 'SELECT post_id FROM ' . $this->table_section_lesson . ' WHERE id = ' . intval( $lesson_id );

    return $this->wpdb->get_var($sql);
  }

  public function get_course_id_by_section_id( $section_id ) {
    $sql = 'SELECT post_id FROM ' . $this->table_section . ' WHERE id = ' . intval( $section_id ) . ' )';

    return $this->wpdb->get_var($sql);
  }

  public function get_lesson( $lesson_id ) {
    $sql = 'SELECT *
              FROM ' . $this->table_section_lesson . '
             WHERE id = ' . $lesson_id;

    $lesson = $this->wpdb->get_row($sql);

    if( $lesson === null )
      return null;

    $lesson->options = ( $lesson->options == '' ? new \stdClass() : json_decode( $lesson->options ) );

    return $lesson;
  }

  public function get_lesson_title_map( $post_id ) {
    $sql = 'SELECT id, title
                FROM ' . $this->table_section_lesson . '
               WHERE post_id = ' . $post_id . '
               ORDER BY `order` ASC';

    $lessons = $this->wpdb->get_results($sql);

    if( empty( $lessons ) )
      return [];

    return wpep_utility_map_to_array_assoc( $lessons, 'id', 'title' );
  }

  public function get_lessons( $id, $field = 'section_id' ) {
    $sql = 'SELECT *
                FROM ' . $this->table_section_lesson . '
               WHERE ' . ( $field == 'section_id' ? 'section_id' : 'post_id' ) . ' = ' . $id . '
               ORDER BY `order` ASC';

    $lessons = $this->wpdb->get_results($sql);

    foreach( $lessons as $lesson_key => $lesson )
      $lessons[$lesson_key]->options = ( $lesson->options == '' ? new \stdClass() : json_decode( $lesson->options ) );

    return $lessons;
  }

  public function get_lesson_count( $course_id ) {
    $sql = 'SELECT COUNT(*)
                FROM ' . $this->table_section_lesson . '
               WHERE post_id = ' . $course_id;

    return intval( $this->wpdb->get_var($sql) );
  }

}