<?php

/**
 * @param $url
 * @return int
 */
function wpep_get_post_id_by_url( $url ) {
  return wpep_controller()->request->get_post_id_by_url( $url );
}

/**
 * @return string
 */
function wpep_get_current_url() {
  return wpep_controller()->request->get_current_url();
}

/**
 * @return int
 */
function wpep_get_current_post_id() {
  return wpep_controller()->request->get_current_post_id();
}

/**
 * @return int
 */
function wpep_get_current_post_type() {
  return wpep_controller()->request->get_current_post_type();
}

/**
 * @param $course_id
 * @param @user_id
 * @return int
 */
function wpep_post_has_member_magnet_option( $course_id, $user_id = null ) {
  if( $user_id == null || $user_id == 0 ) {
    $has_member_magnet_option = wpep_get_setting_flag( 'enable-post-member-magnet-option' );

    if( $has_member_magnet_option ) {
      $post_meta = get_post_meta( $course_id, 'wpep_member_magnet_option_status', true );

      $has_member_magnet_option = ( is_numeric( $post_meta ) ? $post_meta : $has_member_magnet_option );
    }
  } else {
    $has_member_magnet_option = false;
  }

  return apply_filters( 'wpep_single_post_has_member_magnet_option', $has_member_magnet_option );
}

/**
 * @param $post_id
 * @return string
 */
function wpep_post_member_magnet_link( $post_id ) {
  $response = get_the_permalink( $post_id );

  if( wpep_has_embed_support() && isset( $_GET['wpep-embed'] ) )
    $response = add_query_arg( 'wpep-embed', 1, $response );

  return add_query_arg( 'wpep-login', 1, $response );
}

function wpep_current_request_setup_member_magnet() {
  add_filter( 'wpep_user_profile_authentication_form_has_register', function(){ return true; }, 50 );
}

if( !function_exists( 'wpep_post_read_only' ) ) {

  /**
   * @param $course_id
   * @return int
   */
  function wpep_post_read_only( $course_id ) {
    return get_post_meta( $course_id, 'wpep_read_only', true );
  }

}

/**
 * @param $course_id
 * @return int
 */
function wpep_post_coming_soon( $course_id ) {
  return get_post_meta( $course_id, 'wpep_coming_soon', true );
}

/**
 * This is not by default used on the frontend, and the option Enforce Ordering is required to be checked.
 * @param $post_list
 * @param $order = 'ASC' | 'DESC'
 * @param $order_field = wpep_order
 * @param $verify_integrity = true
 * @return mixed
 */
function wpep_post_list_ordering( $post_list, $order = 'ASC', $order_field = 'wpep_order', $verify_integrity = true ) {
  if( !is_array( $post_list ) )
    return $post_list;

  if( empty( $post_list ) )
    return $post_list;

  if( $verify_integrity ) {
    foreach( $post_list as $post_index => $post )
      if( !isset( $post_list[$post_index]->$order_field ) )
        $post_list[$post_index]->$order_field = intval( get_post_meta( $post->ID, $order_field, true ) );
  }

  usort( $post_list, function( $a, $b ) use ( $order, $order_field ) {
    if ( $a->$order_field == $b->$order_field )
      return 0;

    return ( $order == 'ASC'
      ? ( ( $a->$order_field < $b->$order_field ) ? -1 : 1 )
      : ( ( $a->$order_field > $b->$order_field ) ? -1 : 1 )
    );
  } );

  return $post_list;
}

if( !function_exists( 'wpep_get_post_list_assoc' ) ) {

  function wpep_get_post_list_assoc( $post_type = WPEP_POST_TYPE_COURSE, $key_param = 'ID', $value_param = 'post_title', $orderby = 'post_title' ) {
    return wpep_utility_map_to_array_assoc(
      get_posts( [
        'post_type'   => $post_type,
        'numberposts' => -1,
        'orderby'     => $orderby,
        'order'       => 'ASC'
      ] ),
      $key_param,
      $value_param
    );
  }

}

function wpep_entity_post_splinter_id( $post_id, $user_id = false ) {
  WPEP\Entity\PostSplinter::instance()->get_splinter_id( $post_id, $user_id );
}

function wpep_entity_post_splinter_load( $post = false, $user_id = false ) {
  WPEP\Entity\PostSplinter::instance()->load( $post, $user_id );
}

function wpep_entity_post_splinter_reset() {
  WPEP\Entity\PostSplinter::instance()->reset();
}