<?php

use WPEP\Entity\Course as Entity_Course;

/**
 * @param $tag
 * @param array $options
 * @param string $label
 * @param bool $button
 * @return string
 */
function wpep_admin_shortcode_builder( $tag, $options = [], $label = 'Configure ShortCode', $button = true) {
  add_thickbox();

  return '<a href="' . wpep_admin_shortcode_builder_url( $tag, $options ) . '" 
             class="wpep-thickbox thickbox' . ( $button ? ' button button-primary' : '' ) . '">' .
            $label .
         '</a>';
}

/**
 * @param $tag
 * @param array $options
 * @return string
 */
function wpep_admin_shortcode_builder_url(
  $tag,
  $options = []
) {
  $url = add_query_arg( 'wpep-shortcode-builder', $tag, admin_url() );

  if( !empty( $options ) ) {
    $url = add_query_arg( 'wpep-sb-options', urlencode( json_encode( $options ) ), $url );
  }
  return $url;
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_index_options() {
  $category_list = wpep_utility_map_to_array_assoc(
    get_terms( [ 'taxonomy'   => 'course_category', 'hide_empty' => false, 'parent' => 0 ] ),
    'term_id',
    'name'
  );

  $per_row_options = [ '' => __( "Default", "wpep" ) ] + wpep_content_primary_row_options() + [ 0 => __( "Do not display", "wpep" ) ];
  $courses_per_row_options = [ '' => __( "Default", "wpep" ) ] + wpep_content_primary_row_options( WPEP_POST_TYPE_COURSE ) + [ 0 => __( "Do not display", "wpep" ) ];

  return [
    'categories' => [
      'type'        => 'multiple_select',
      'options'     => $category_list,
      'description' => __( "Display only certain categories within this Index Page", "wpep" )
    ],
    'categories_per_row' => [
      'type'    => 'select',
      'options' => [ '' => __( "Default", "wpep" ) ] + wpep_content_categories_row_options()
    ],
    'all_categories_filter' => [
      'type'    => 'select',
      'options' => [
        ''  => sprintf( __( "Default : %s" ), wpep_get_setting_flag( 'index-page-enable-filter-all-categories' ) ? 'Enabled' : 'Disabled' )
      ] + wpep_flag_setting_field_options()
    ],
    'single_category_view' => [
      'type'    => 'select',
      'options' => [
        ''  => sprintf( __( "Default : Show %s category navigation" ), wpep_get_setting_flag( 'index-page-single-category-primary-view' ) ? 'primary' : 'secondary' ),
        0   => __( "Show secondary category navigation" ),
        1   => __( "Show primary category navigation" )
      ]
    ],
    'display_header' => [
      'type'    => 'select',
      'options' => [
        '1' => 'Yes',
        '0' => 'No'
      ]
    ],
    'order' => [
      'type'    => 'select',
      'options' => wpep_content_primary_order_options()
    ],
    'order_by' => [
      'type'    => 'select',
      'options' => wpep_content_primary_order_by_options()
    ],
    'courses_per_row' => [
      'type'    => 'select',
      'options' => $courses_per_row_options
    ],
    'courses'         => [
      'type'    => 'multiple_select',
      'options' => wpep_get_post_list_assoc( WPEP_POST_TYPE_COURSE ),
      'description' => __( "If you want to Display specific Course Tabs only leave the Categories field empty and disable Category Navigation.", "wpep" )
    ],
    'course_progress_bars' => [
      'type'    => 'select',
      'options' => [
        ''  => __( "Default", "wpep")
      ] + wpep_flag_setting_field_options(),
    ],
    'videos_per_row' => [
      'type'    => 'select',
      'options' => [
        ''  => __( "Default", "wpep" ),
        1   => __( "One per row", "wpep" ),
        0   => __( "Do not display", "wpep" )
      ]
    ],
    'ebooks'         => [
      'type'    => 'multiple_select',
      'options' => wpep_get_post_list_assoc( WPEP_POST_TYPE_E_BOOK ),
      'description' => __( "If you want to Display specific E-Book Tabs only leave the Categories field empty and disable Category Navigation.", "wpep" )
    ],
    'ebooks_per_row' => [
      'type'    => 'select',
      'options' => $per_row_options
    ],
    'offers'         => [
      'type'    => 'multiple_select',
      'options' => wpep_get_post_list_assoc( WPEP_POST_TYPE_OFFER ),
      'description' => __( "If you want to Display specific Offers Tabs only leave the Categories field empty and disable Category Navigation.", "wpep" )
    ],
    'offers_per_row' => [
      'type'    => 'select',
      'options' => $per_row_options
    ],
    'posts_per_page' => [
      'type'    => 'select',
      'options' => [ '' => __( "Show Everything" ) ] + wpep_get_indexed_number_list( 1, 12, "%value% of each type of content." ),
      'description' => __( "Advanced Option : Limit the number of Presentation Cards to include on the Index Page for each Content Type, within each Category View-Port.", "wpep" )
    ],
  ];
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_course_grid_options() {
  return [
    'courses'         => [
      'type'    => 'multiple_select',
      'options' => wpep_utility_map_to_array_assoc(
        get_posts( [
          'post_type'   => WPEP_POST_TYPE_COURSE,
          'numberposts' => -1,
          'orderby'     => 'post_title',
          'order'       => 'ASC'
        ] ),
        'ID',
        'post_title'
      ),
      'description' => __( "Leaving this empty will display all the courses.", "wpep" )
    ],
    'container_class' => [
      'name'    => __('Container', "wpep"),
      'type'    => 'select',
      'options' => [
        ''                                    => 'Default',
        'wpep-3d-carousel-container-shadowed' => 'Box Shadow on Sides'
      ]
    ]
  ];
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_course_options() {
  $options = [];

  foreach( wpep_course_layout_types( [
            WPEP_COURSE_TYPE_EXECUTION_PLAN,
            WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED,
            WPEP_COURSE_TYPE_LEARN_MAP,
            WPEP_COURSE_TYPE_ONE_OFF
          ] ) as $course_type => $course_type_label ) {
    $group_options = wpep_utility_map_to_array_assoc(
      get_posts( [
        'post_type'   => WPEP_POST_TYPE_COURSE,
        'numberposts' => -1,
        'orderby'     => 'post_title',
        'order'       => 'ASC',
        'meta_query'  => [
          [
            'key'   => 'course_type',
            'value' => $course_type,
          ]
        ]
      ] ),
      'ID',
      'post_title'
    );

    if( empty( $group_options ) )
      continue;

    $options[] = [
      'group_title'   => $course_type_label,
      'group_options' => $group_options
    ]; ;
  }

  return [
    'id'          => [
      'name'    => __('Course', "wpep"),
      'type'    => 'searchable_select',
      'options' => $options
    ],
    'show_header' => [
      'type'        => 'checkbox',
      'is_checked'  => 1
    ],
    'course_type' => [
      'name'    => __('Course Type', "wpep"),
      'type'    => 'select',
      'options' => [
          "" => __( "Option Selected in Course Settings", "wpep" )
        ] + wpep_course_layout_types( [
          WPEP_COURSE_TYPE_EXECUTION_PLAN,
          WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED,
          WPEP_COURSE_TYPE_LEARN_MAP,
          WPEP_COURSE_TYPE_ONE_OFF
        ] )
    ]
  ];
}

function wpep_admin_shortcode_builder_wpep_course_preview_options( $builder_base_options = [] ) {
  if( !isset( $builder_base_options[ 'id' ] ) )
    return [];

  $course_map = Entity_Course::instance()->get_with_lessons( $builder_base_options['id'] );
  $available_lessons_options = [];

  foreach( $course_map as $section_information ) {
    $available_lessons_options[] = [
      'group_title'   => $section_information->title,
      'group_options' => wpep_utility_map_to_array_assoc( $section_information->lessons, 'id', 'title' )
    ]; ;
  }

  return [
    'show_header' => [
      'type'        => 'checkbox',
      'is_checked'  => 1
    ],
    'course_lessons' => [
      'name'    => __('Available Lessons', "wpep"),
      'type'    => 'multiple_select',
      'options' => $available_lessons_options,
      'desc'    => __( "Select each lesson that will be available under this Course Preview", "wpep" )
    ],
    'course_type'       => [
      'name'    => __('Course Type', "wpep"),
      'type'    => 'select',
      'options' => wpep_course_layout_types( [
          WPEP_COURSE_TYPE_EXECUTION_PLAN,
          WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED,
          WPEP_COURSE_TYPE_LEARN_MAP
      ] )
    ]
  ];
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_link_options() {
  return [
    'content' => [
      'name'        => __( "Link" ),
      'type'        => 'shortcode_content'
    ],
    'text'  => [
      'name'    => __('Button Text', "wpep"),
      'type'    => 'text'
    ],
    'background_color' => [
      'name'    => __('Button Color', "wpep"),
      'type'    => 'color'
    ],
    'color' => [
      'name'    => __('Text Color', "wpep"),
      'type'    => 'color'
    ],
    'position' => [
      'name'    => __( "Alignment", "wpep" ),
      'type'    => 'select',
      'options' => [
        'left'    => __( "Left", "wpep" ),
        'center'  => __( "Center", "wpep" ),
        'right'   => __( "Right", "wpep" )
      ]
    ]
  ];
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_notification_options() {
  return [
    'content' => [
      'name'        => __( "Content" ),
      'type'        => 'shortcode_content'
    ],
    'type' => [
      'name'    => __('Type', "wpep"),
      'type'    => 'select',
      'options' => [
        "standard" => __( "Standard", "wpep" ),
        "info"     => __( "Informative", "wpep" ),
        "success"  => __( "Success", "wpep" ),
        "warning"  => __( "Warning", "wpep" ),
        "danger"   => __( "Danger", "wpep" ),
      ]
    ]
  ];
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_course_ids_selection_options() {
  return [
    'content' => [
      'name'        => __( "Content" ),
      'type'        => 'shortcode_content',
      'default'     => 'You can add any content here.'
    ],
    'id'          => [
      'name'    => __('Course', "wpep"),
      'type'    => 'multiple_select',
      'options' => wpep_utility_map_to_array_assoc(
        get_posts( [
          'post_type'   => WPEP_POST_TYPE_COURSE,
          'numberposts' => -1,
          'orderby'     => 'post_title',
          'order'       => 'ASC'
        ] ),
        'ID',
        'post_title'
      )
    ]
  ];
}

function wpep_admin_shortcode_builder_wpep_search_advanced_options() {
  $category_list = wpep_utility_map_to_array_assoc(
    get_terms( [ 'taxonomy'   => 'course_category', 'hide_empty' => true, 'parent' => 0 ] ),
    'term_id',
    'name'
  );

  return [
    'post_type'         => [
      'type'        => 'multiple_select',
      'options'     => [
        WPEP_POST_TYPE_COURSE   => __( "Courses", "wpep" ),
        WPEP_POST_TYPE_E_BOOK   => __( "E-Books", "wpep" ),
        WPEP_POST_TYPE_VIDEO    => __( "Videos", "wpep" ),
        WPEP_POST_TYPE_OFFER    => __( "Offers", "wpep" )
      ],
      'description' => __( "Display only certain Post Types", "wpep" )
    ],
    'include_categories' => [
      'type'        => 'multiple_select',
      'options'     => $category_list,
      'description' => __( "Include only these Categories in this search", "wpep" )
    ],
    'exclude_categories' => [
      'type'        => 'multiple_select',
      'options'     => $category_list,
      'description' => __( "Exclude these Categories in this search", "wpep" )
    ],

  ];
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_lesson_content_options() {
  $options = [];
  $courses = wpep_utility_map_to_array_assoc(
    get_posts( [
      'post_type'   => WPEP_POST_TYPE_COURSE,
      'numberposts' => -1,
      'orderby'     => 'post_title',
      'order'       => 'ASC'
    ] ),
    'ID',
    'post_title'
  );

  foreach( $courses as $course_id => $course_title ) {
    if( $course_title === '' )
      continue;

    $options[ $course_id ] = [
      'group_title'   => $course_title,
      'group_options' => Entity_Course::instance()->get_lesson_title_map( $course_id )
    ];
  }

  return [
    'lesson' => [
      'type'    => 'searchable_select',
      'options' => $options
    ]
  ];
}