<?php

use WPEP\Entity\Course as EntityCourse;

/**
 * @param $course_id
 * @param bool $user_id
 * @return int
 */
function wpep_user_course_started_at( $course_id, $user_id = false ) {
  $user_id = ( $user_id == false ? get_current_user_id() : $user_id );

  return intval( wpep_controller()->userActivity->get_course_user_data_value( WPEP_USER_COURSE_STARTED, $course_id, 0, 0, $user_id, true ) );
}

/**
 * @param $course_id
 * @param bool $user_id
 * @return int
 */
function wpep_user_course_started_at_seconds( $course_id, $user_id = false ) {
  $started_at = wpep_user_course_started_at( $course_id, $user_id );

  return intval( $started_at == 0 ? 0 : ( time() - $started_at ) );
}

function wpep_user_get_roles( $user_id ) {
  if( isset( $user_id->roles ) )
    return $user_id->roles;

  $user = get_userdata( $user_id );

  return empty( $user ) ? [] : $user->roles;
}

/**
 * @param $role
 * @param int|WP_User $user_id
 * @return bool
 */
function wpep_user_has_role( $role, $user_id ) {
  if( $role === false )
    return true;

  return in_array( $role, wpep_user_get_roles( $user_id ) );
}

/**
 * @param $course_id
 * @param bool $user_id
 * @return int
 */
function wpep_user_course_progress_percentage( $course_id, $user_id = false ) {
  return wpep_controller()->userActivity->course_progress_percentage( $course_id, $user_id );
}

/**
 * @param $course_id
 * @param bool $user_id
 * @return int|bool
 */
function wpep_user_course_is_bookmarked( $course_id, $user_id = false ) {
  return wpep_controller()->userActivity->get_course_bookmark_status( $course_id, $user_id );
}

/**
 * @param $course_id
 * @param bool $user_id
 * @return int
 */
function wpep_user_course_pending_display_lessons( $course_id, $user_id = false ) {
  if( !wpep_user_handle_timing_options( $user_id ) )
    return 0;

  $user_id   = intval( $user_id == false ? get_current_user_id() : $user_id );
  $course_id = intval( $course_id );

  global $wpdb;

  if( $user_id === false )
    $user_id = get_current_user_id();

  $seconds_spent = wpep_user_course_started_at_seconds( $course_id, $user_id );

  $sql = 'SELECT COUNT( l.id ) 
            FROM ' . $wpdb->prefix . 'wpep_section_lesson l
            LEFT JOIN ' . $wpdb->prefix . 'wpep_section s ON s.id = l.section_id
           WHERE l.post_id = ' . $course_id . '
             AND ( l.time_action_display_from > ' . $seconds_spent . ' OR s.time_action_display_from > ' . $seconds_spent . ' ) ';

  return intval( $wpdb->get_var($sql) );
}

function wpep_user_has_statistics_access_for_post( $post_id ) {
  if( current_user_can( WPEP_CAP_MANAGE_STATISTICS ) )
    return true;

  $course = get_post( $post_id );

  if( !isset( $course->post_author ) || $course->post_author != get_current_user_id() )
    return false;

  return true;
}

function wpep_user_statistics_query_author_id() {
  return ( current_user_can( WPEP_CAP_MANAGE_STATISTICS ) ? 0 : get_current_user_id() );
}

/**
 * Timing Options are not done if :
 *    - it's within the WP administration
 *    - it's an admin user
 *    - the logged in user is an administrator
 * @param bool $user_id
 * @return bool
 */
function wpep_user_handle_timing_options( $user_id = false ) {
  if( wpep_get_setting( 'enable-course-timing-option', 0 ) == false )
    return false;

  if( is_admin() )
    return false;

  $user_id = ( $user_id == false ? get_current_user_id() : $user_id );

  if( user_can( $user_id, WPEP_CAP_EDIT_COURSE ) )
    return false;

  if( current_user_can( WPEP_CAP_EDIT_COURSE ) )
    return false;

  return true;
}

/**
 * @param $assessment_id
 * @param int $user_id
 * @return string
 */
function wpep_user_assessment_status( $assessment_id, $user_id = 0 ) {
  $user_id = ( $user_id == false ? get_current_user_id() : $user_id );

  if( $user_id == 0 )
    return WPEP_USER_ASSESSMENT_STATUS_PENDING;

  $user_status = wpep_controller()->userActivity->assessment->get_user_data(
    WPEP_USER_ASSESSMENT_STATUS, $assessment_id, 0, 0, get_current_user_id(), true
  );
  $user_status = ( $user_status === null ? WPEP_USER_ASSESSMENT_STATUS_PENDING : $user_status->value );

  return $user_status;
}

/**
 * @param $course_id
 * @param int $section_id
 * @param int $user_id
 * @return int
 */
function wpep_user_assessment_id_by_course_id( $course_id, $section_id = 0, $user_id = 0 ) {
  $user_id         = ( $user_id == false ? get_current_user_id() : $user_id );

  if( $user_id == 0 )
    return 0;

  $assessment_status = [ 'publish' ];

  if( user_can( $user_id, WPEP_CAP_EDIT_ASSESSMENT ) )
    $assessment_status[] = 'private';

  return wpep_get_course_assessment_id( $course_id, $section_id, $assessment_status );
}

/**
 * @param $lesson_id
 * @param bool $user_id
 * @param array $assessment_user_status
 * @return array
 */
function wpep_user_assessment_requirements_for_lesson( $lesson_id, $user_id = false, $assessment_user_status = [ WPEP_USER_ASSESSMENT_STATUS_PENDING, WPEP_USER_ASSESSMENT_STATUS_FAILED ] ) {
  $user_id        = ( $user_id == false ? get_current_user_id() : $user_id );
  $lesson_section = EntityCourse::instance()->get_section_by_lesson_id( $lesson_id );

  if( $lesson_section === null )
    return [];

  return wpep_user_assessment_requirements_for_section( $lesson_section, $user_id, $assessment_user_status );
}

function wpep_user_assessment_requirements_for_section( $lesson_section, $user_id = false, $assessment_user_status = [ WPEP_USER_ASSESSMENT_STATUS_PENDING, WPEP_USER_ASSESSMENT_STATUS_FAILED ] ) {
  $user_id        = ( $user_id == false ? get_current_user_id() : $user_id );

  if( is_numeric( $lesson_section ) )
    $lesson_section = EntityCourse::instance()->get_section( $lesson_section );

  // The User is not required to finish any assessments prior to getting access to this Lesson.
  if( !wpep_course_assessments_required( $lesson_section->post_id ) )
    return [];

  $response = [];

  $section_list = EntityCourse::instance()->get_sections_by_course_id( $lesson_section->post_id );

  foreach( $section_list as $section ) {
    if( $section->order >= $lesson_section->order )
      break;

    $assessment_id = wpep_user_assessment_id_by_course_id( $lesson_section->post_id, $section->id, $user_id );

    if( $assessment_id == 0 )
      continue;

    if( in_array( wpep_user_assessment_status( $assessment_id, $user_id ), $assessment_user_status ) )
      $response[] = $assessment_id;
  }

  return $response;
}

/**
 * @param $course_id
 * @param bool $user_id
 * @param array $assessment_user_status
 * @return array
 */
function wpep_get_assessment_requirements_for_course( $course_id, $user_id = false, $assessment_user_status = [ WPEP_USER_ASSESSMENT_STATUS_PENDING, WPEP_USER_ASSESSMENT_STATUS_FAILED ] ) {
  $user_id        = ( $user_id == false ? get_current_user_id() : $user_id );

  $response = [];

  $section_list = EntityCourse::instance()->get_sections_by_course_id( $course_id );

  foreach( $section_list as $section ) {
    $assessment_id = wpep_user_assessment_id_by_course_id( $course_id, $section->id, $user_id );

    if( $assessment_id == 0 )
      continue;

    if( in_array( wpep_user_assessment_status( $assessment_id, $user_id ), $assessment_user_status ) )
      $response[] = $assessment_id;
  }

  // Final Assessment
  $assessment_id = wpep_user_assessment_id_by_course_id( $course_id, 0, $user_id );

  if( $assessment_id == 0 )
    return $response;

  if( in_array( wpep_user_assessment_status( $assessment_id, $user_id ), $assessment_user_status ) )
    $response[] = $assessment_id;

  return $response;
}

function wpep_user_filter_query_args( $user_list_query_args, $filter_by = [] ) {
  if( isset( $filter_by['user_role'] ) && $filter_by['user_role'] != false )
    $user_list_query_args[ 'role' ] = $filter_by['user_role'];

  return $user_list_query_args;
}

/**
 * @param $user_id
 * @param array $filter_by
 * @return bool
 */
function wpep_user_filter_requirement_met( $user_id, $filter_by = [] ) {
  if( isset( $filter_by['user_role'] )
    && !wpep_user_has_role( $filter_by['user_role'], $user_id ) )
    return false;

  return true;
}

/**
 * @param $data
 * @param array $filter_by
 * @return mixed
 */
function wpep_user_filter_map_entries( $data, $filter_by = [] ) {
  if( !isset( $filter_by['user_role'] ) || $filter_by['user_role'] === false )
    return $data;

  foreach( $data as $data_user_id => $data_value )
    if( !wpep_user_filter_requirement_met( $data_user_id, $filter_by ) )
      unset( $data[ $data_user_id ] );

  return $data;
}

function wpep_user_course_started_event( $course_id, $user_id = false ) {
  $user_id = ( $user_id == false ? get_current_user_id() : $user_id );

  $started_at = wpep_user_course_started_at( $course_id, $user_id );

  if( $started_at == 0 ) {
    $started_at = time();

    wpep_controller()->userActivity->set_course_user_data(
      WPEP_USER_COURSE_STARTED,
      $started_at,
      $course_id,
      0,
      0,
      $user_id
    );
  }

  return $started_at;
}
