<?php

abstract class WPEP_AddOn_Settings_Integration {

  public function __construct() {
    $this->_setup_integration();
  }

  public function _setup_integration() {
    if( $this->get_plugin_basename() != false )
      add_filter( 'plugin_action_links_' . $this->get_plugin_basename() , [ $this, 'plugin_action_links' ] );
  }

  abstract public function get_name();

  public function get_plugin_basename() {
    return false;
  }

  public function get_page_link() {
    return esc_url( get_admin_url(null, 'edit.php?post_type=' . WPEP_POST_TYPE_COURSE . '&page=wpep-settings&tab=addon&addon=' . $this->get_alias() ) );
  }

  public function plugin_action_links( $links ) {
    $links[] = '<a href="'. $this->get_page_link() .'">' . __( "Settings", 'wpep' )  . '</a>';
    return $links;
  }

  public function get_alias() {
    return sanitize_title( $this->get_name() );
  }

  public function get_option_prefix() {
    return 'wpep_addon_' . $this->get_alias() . '_';
  }

  public function get_option( $key, $default = '' ) {
    return get_option( $this->get_option_prefix() . $key, $default );
  }

  /**
   * AddOn Settings Pages don't always require a controller, this should return an WPEP_AddOn_Integration instance.
   * @return bool|WPEP_AddOn_Integration
   */
  public function get_controller() {
    return false;
  }

  /**
   *
   * @return array(
      'field_alias' => array(
        'name'    => __( "Template", 'wpep_custom_ui' ),
        'type'    => 'text|textarea|colorpicker|select|html',
        'value'   => '',

        // Required for select inputs
        'values'  => array(
          'default' => 'Default'
        )
        // HTML content
        'html' => 'your custom content'
      )
     )
   */
  abstract public function get_field_list();

  /**
   * Tip : Implement your own method for saving, if you're not implementing a addon under the WPEP AddOn Structure.
   * @param $information
   * @return bool
   */
  public function save_information( $information ) {
    foreach( $information as $information_key => $information_value ) {
      if( $information_key === 'license'
          && $this->get_controller() != false
          && property_exists( $this->get_controller(), 'licenseAndUpdatesIntegration' ) ) {
        $this->get_controller()->licenseAndUpdatesIntegration->activate_license( $information_value );

        continue;
      }

      if( !is_array( $information_value ) )
        $information_value = sanitize_text_field( $information_value );

      update_option( $this->get_option_prefix() . $information_key, $information_value );
    }

    return true;
  }

}