<?php

require_once( WPEP_BASE_PATH . "/lib/integrations/Email_Marketing/Frontend.php" );
require_once( WPEP_BASE_PATH . "/lib/integrations/Email_Marketing/Administration.php" );

abstract class WPEP_Integration_Email_Marketing {

  protected $_tag_list = NULL;

  public $entity_name             = "Tag";
  public $input_placeholder       = "Start typing to autocomplete...";
  public $input_class             = 'regular-text wpep-tag-cloud-input';
  public $has_statistics_log      = true;

  public $post_meta_prefix        = '';
  public $post_meta_started       = '';
  public $post_meta_completed     = '';

  public $section_option_name     = '';
  public $lesson_option_name      = '';

  public $log_option_name = '';

  public $administrationCoordinator;
  public $frontendCoordinator;

  public function __construct() {
    $this->_setup_vars();

    if( $this->is_configured() ) {
      $this->frontendCoordinator = new WPEP_Integration_Email_Marketing_Coordinator_Frontend( $this );
      $this->frontendCoordinator->init();
    }

    if( is_admin() ) {
      $this->administrationCoordinator = new WPEP_Integration_Email_Marketing_Coordinator_Administration( $this );

      $this->administrationCoordinator->init();
    }
  }

  abstract public function get_name();

  abstract public function get_alias();

  abstract public function get_slug();

  abstract public function is_configured();

  abstract public function get_tag_list();

  public function get_tag_list_cached( $from_transient = true, $force_refresh = false ) {
    if( !$force_refresh ) {
      if( $this->_tag_list != NULL )
        return $this->_tag_list;

      if( $from_transient ) {
        $this->_tag_list = get_transient( 'wpep_email_marketing_' . $this->get_alias() . '_tags' );

        if( is_array( $this->_tag_list ) )
          return $this->_tag_list;
      }
    }

    $this->_tag_list = $this->get_tag_list();

    set_transient( 'wpep_email_marketing_' . $this->get_alias() . '_tags', $this->_tag_list, time() + ( 3600 * 24 ) );

    return $this->_tag_list;
  }

  abstract public function add_tag( $tag, $user_id );
  abstract public function remove_tag( $tag, $user_id );

  private function _setup_vars() {
    $this->section_option_name = $this->get_alias() . '_tag';
    $this->lesson_option_name  = $this->get_alias() . '_tag';

    $this->post_meta_prefix    = 'wpep_' . $this->get_alias() . '_';

    $this->post_meta_started   = $this->post_meta_prefix . 'tag_started';
    $this->post_meta_completed = $this->post_meta_prefix . 'tag';

    if( $this->log_option_name == '' )
      $this->log_option_name     = 'wpep_addon_' . $this->get_alias() . '_log';
  }

  public function get_tag_input_metabox_default() {
    $storage_index_data = [
      'storage-index' => $this->get_alias() . "_tags"
    ];

    return [
      'type'        => 'text',
      'class'       => $this->input_class,
      'data'        => ( strpos( $this->input_class, 'wpep-tag-cloud-input' ) === false ? [] : $storage_index_data ),
      'placeholder' => $this->input_placeholder
    ];
  }

  public function get_tag_input_html( $input_name, $value = '' ) {
    return   '<input type="text" 
                     class="' . $this->input_class . '" 
                     ' . ( strpos( $this->input_class, 'wpep-tag-cloud-input' ) === false ? '' : 'data-storage-index="' . $this->get_alias() . '_tags"' ) . '
                     style="min-width:400px;"
                     placeholder="' . $this->input_placeholder . '"
                     name="' . $input_name . '" 
                     value="' . $value . '">';
  }

}