<?php

class WPEP_Integration_Email_Marketing_Coordinator_Administration {

  /**
   * @var $controller WPEP_Integration_Email_Marketing
   */
  protected $_controller;

  /**
   * @var $controller WPEP_Integration_Email_Marketing
   */
  public function __construct( WPEP_Integration_Email_Marketing $controller ) {
    $this->_controller = $controller;
  }

  public function init() {
    if( $this->_controller->has_statistics_log && current_user_can( WPEP_CAP_MANAGE_SETTINGS ) ) {
      add_filter( 'wpep_administration_platform_health_pages', [ $this, 'wpep_platform_health_pages' ] );
    }

    if( $this->_controller->is_configured() != true )
      return;

    add_filter( 'wpep_admin_script_variables', [ $this, '_admin_script_variables' ], 10, 2 );

    add_action( 'wpep_administration_section_option_field', [ $this, 'section_option_field' ], 10, 2 );
    add_action( 'wpep_administration_lesson_option_field',  [ $this, 'lesson_option_field' ], 10, 2 );
    add_filter( 'wpep_administration_course_metaboxes', [ $this, 'entity_meta_boxes' ] );
    add_action( 'wpep_administration_assessment_meta_options_block', [ $this, 'assessment_meta_options_block' ], 10, 2 );
  }

  public function wpep_platform_health_pages( $pages ) {
    if( ! class_exists( 'WPEP_Integration_PlatformHealth_Coordinator_User_Course_Log' ) )
      require_once( WPEP_BASE_PATH . "/lib/integrations/PlatformHealth/User_Course_Log.php" );

    $instance = new WPEP_Integration_PlatformHealth_Coordinator_User_Course_Log(
      $this->_controller->get_name() . ' Log',
      $this->_controller->get_slug() . '-log',
      $this->_controller->log_option_name
    );

    $pages[ $instance->get_alias() ] = $instance;

    return $pages;
  }

  public function _admin_script_variables( $variables = [], $hook ) {
    if( !in_array( $hook, [ "post-new.php", "post.php" ] ) )
      return $variables;

    global $post;

    if( isset($post->post_type) && in_array( $post->post_type, [ WPEP_POST_TYPE_COURSE, WPEP_POST_TYPE_ASSESSMENT ] ) )
      $variables[ $this->_controller->get_alias() . '_tags' ] = $this->_controller->get_tag_list_cached( false );

    return $variables;
  }

  public function section_option_field( $field_prefix, $section_options ) {
    $option_name = $this->_controller->section_option_name;

    echo '<div class="wpep-form-group wpep-form-group-section-' . $this->_controller->get_slug() . '-settings">';
    echo      '<div class="wpep-form-group-row">';
    echo        '<label for="wpep-section-title">' . $this->_controller->get_name() . " " . $this->_controller->entity_name . ':</label>';
    echo        $this->_controller->get_tag_input_html(
      $field_prefix . '[' . $option_name . ']',
      ( isset( $section_options->$option_name ) ? $section_options->$option_name : '' )
    );
    echo      '</div>';
    echo '</div>';
  }

  public function lesson_option_field( $field_prefix, $lesson_options ) {
    $option_name = $this->_controller->lesson_option_name;

    echo '<div class="wpep-form-group wpep-form-group-lesson-' . $this->_controller->get_slug() . '-settings">';
    echo      '<div class="wpep-form-group-row">';
    echo        '<label for="wpep-section-title">' . $this->_controller->get_name() . " " . $this->_controller->entity_name . ':</label>';
    echo        $this->_controller->get_tag_input_html(
      $field_prefix . '[' . $option_name . ']',
      ( isset( $lesson_options->$option_name ) ? $lesson_options->$option_name : '' )
    );
    echo      '</div>';
    echo '</div>';
  }

  public function assessment_meta_options_block( $field_prefix, $assessment_id = false ) {
    echo '<h3>' . sprintf( __( "Assessment Status Student %s - %s", 'wpep' ), $this->_controller->entity_name, $this->_controller->get_name() ) . '</h3>';

    echo '<div class="wpep-form-group wpep-form-group-assessment-' . $this->_controller->get_slug() . '-settings">';
    echo   '<div class="wpep-form-group-row">';
    echo     '<label for="wpep-section-title">' . __( "Status Pending", 'wpep' ) . ':</label>';
    echo     $this->_controller->get_tag_input_html(
      $field_prefix . '[' . $this->_controller->post_meta_prefix . 'tag_status_pending]',
      get_post_meta( $assessment_id, $this->_controller->post_meta_prefix . 'tag_status_pending', true )
    );
    echo   '</div>';

    echo   '<div class="wpep-form-group-row">';
    echo     '<label for="wpep-section-title">' . __( "Status Pending Review", 'wpep' ) . ':</label>';
    echo     $this->_controller->get_tag_input_html(
      $field_prefix . '[' . $this->_controller->post_meta_prefix . 'tag_status_pending_review]',
      get_post_meta( $assessment_id, $this->_controller->post_meta_prefix . 'tag_status_pending_review', true )
    );
    echo   '</div>';

    echo   '<div class="wpep-form-group-row">';
    echo     '<label for="wpep-section-title">' . __( "Status Completed", 'wpep' ) . ':</label>';
    echo     $this->_controller->get_tag_input_html(
      $field_prefix . '[' . $this->_controller->post_meta_prefix . 'tag_status_completed]',
      get_post_meta( $assessment_id, $this->_controller->post_meta_prefix . 'tag_status_completed', true )
    );
    echo   '</div>';

    echo   '<div class="wpep-form-group-row">';
    echo     '<label for="wpep-section-title">' . __( "Status Failed", 'wpep' ) . ':</label>';
    echo     $this->_controller->get_tag_input_html(
      $field_prefix . '[' . $this->_controller->post_meta_prefix . 'tag_status_failed]',
      get_post_meta( $assessment_id, $this->_controller->post_meta_prefix . 'tag_status_failed', true )
    );
    echo   '</div>';
    echo '</div>';

  }

  public function entity_meta_boxes( $meta_boxes ) {
    $meta_boxes[ $this->_controller->get_slug() . '-course-settings'] = [
      'id'         => 'wpep_' . $this->_controller->get_alias() . '_course_settings',
      'title'      => sprintf( __( '%s Integration', 'wpep' ), $this->_controller->get_name() ),
      'pages'      => [ WPEP_POST_TYPE_COURSE ],
      'context'    => 'side',
      'priority'   => 'low',
      'show_names' => true,
      'fields'     => [
        $this->_controller->get_tag_input_metabox_default() + [
          'name'  => sprintf( __( 'Started %s', 'wpep' ), $this->_controller->entity_name ),
          'desc'  => sprintf( __( 'When starting the course the user will be given this %s.', 'wpep' ), $this->_controller->entity_name ),
          'id'    => $this->_controller->post_meta_started
        ],
        $this->_controller->get_tag_input_metabox_default() + [
          'name'  => sprintf( __( 'Completed %s', 'wpep' ), $this->_controller->entity_name ),
          'desc'  => sprintf( __( 'After finishing the course the user will be given this %s.', 'wpep' ), $this->_controller->entity_name ),
          'id'    => $this->_controller->post_meta_completed
        ]
      ]
    ];

    return $meta_boxes;
  }

}