<?php

use WPEP\Admin\Container\LicenseAndUpdates as WPEP_Coordinator_LicenseAndUpdates;

class WPEP_Integration_License_And_Updates extends WPEP_Coordinator_LicenseAndUpdates {

  public $product_name;
  public $product_id;
  public $plugin_alias;
  public $plugin_version;
  public $options_prefix;

  public function __construct(
    $product_name,
    $product_id,
    $plugin_alias,
    $plugin_version,
    $options_prefix
  ) {
    $this->product_name   = $product_name;
    $this->product_id     = $product_id;
    $this->plugin_alias   = $plugin_alias;
    $this->plugin_version = $plugin_version;
    $this->options_prefix = $options_prefix;

    parent::__construct();
  }

  public function has_background_license_verification() {
    return true;
  }

  public function get_product_name() {
    return $this->product_name;
  }

  public function get_product_id() {
    return $this->product_id;
  }

  public function get_plugin_alias() {
    return $this->plugin_alias;
  }

  public function get_plugin_version() {
    return $this->plugin_version;
  }

  public function get_options_prefix() {
    return $this->options_prefix;
  }

  public function get_license() {
    if( $this->_use_bundle_license() )
      return wpep_controller()->adminController->license->get_license();

    return get_option( $this->get_option_license_key(), '' );
  }

  public function prepare_license_status( $licenseStatus ) {
    return $licenseStatus;
  }

  public function is_valid_license() {
    $status = get_option( $this->get_option_license_key_status(), null );

    if( $status === 'valid' )
      return true;

    if( intval( $status ) === 1 )
      return true;

    return false;
  }

  public function get_option_license_key() {
    return $this->get_options_prefix() . 'license';
  }

  public function get_option_license_key_status() {
    return $this->get_options_prefix() . 'license_status';
  }

  public function get_option_license_last_verification() {
    return $this->get_options_prefix() . 'last_verification';
  }

  public function get_license_status_text() {
    if( $this->is_valid_license() )
      return '<p style="color:#2ecc71">' . __( "Your License is Valid", "wpep" ) . '</p>';

    if( $this->get_license() != '' )
      return '<p style="color:#e74c3c">' . __( "Your License is invalid", "wpep" ) . '</p>';

    return '<p style="color:#4aa3df">' . __( "Please enter your license in order to receive automatic updates and support", 'wpep' ) . '</p>';
  }

  public function get_license_key_field_as_array() {
    if( $this->_use_bundle_license() )
      return [
        'name'        => __( "License Key", 'wpep' ),
        'type'        => 'html',
        'html'        => __( "Using the WPEP Bundle License", "wpep" )
      ];

    return [
      'name'        => __( "License Key", 'wpep' ),
      'type'        => 'text',
      'value'       => $this->get_license(),
      'description' => $this->get_license_status_text()
    ];
  }

  public function get_license_key_field_as_indexed_array( $separator_text = false, $field_key = 'license' ) {
    if( $this->_use_bundle_license() )
      return [];

    $response = [];

    if( $separator_text != false ) {
      $response[ $field_key . '_standard_separator' ] = [ 'type' => 'html', 'name' => $separator_text, 'html' => '<hr/>' ];
    }

    $response[ $field_key ] = $this->get_license_key_field_as_array();

    return $response;
  }

  public function set_license($license) {
    if( $this->_use_bundle_license() )
      return $this;

    return parent::set_license($license);
  }

  private function _use_bundle_license() {
    if( wpep_controller()->adminController->license === null )
      return false;

    return in_array( $this->get_product_id(), wpep_controller()->adminController->license->get_license_bundled_products() );
  }

}